SystemOrganization addCategory: #'XML-Parser'!
SystemOrganization addCategory: 'XML-Parser-DOM'!
SystemOrganization addCategory: 'XML-Parser-DTD'!
SystemOrganization addCategory: 'XML-Parser-Exceptions'!
SystemOrganization addCategory: 'XML-Parser-File'!
SystemOrganization addCategory: 'XML-Parser-HTTP'!
SystemOrganization addCategory: 'XML-Parser-Namespaces'!
SystemOrganization addCategory: 'XML-Parser-Streams'!
SystemOrganization addCategory: 'XML-Parser-URI'!
SystemOrganization addCategory: 'XML-Parser-Util'!
SystemOrganization addCategory: 'XML-Parser-Validation'!
SystemOrganization addCategory: 'XML-Parser-Validation-DFA'!

!Character methodsFor: '*XML-Parser' stamp: 'monty 10/24/2014 09:29'!
xmlSplit: aCollection
	"faster, portable version"
	| parts |

	parts := OrderedCollection new.
	self xmlSplit: aCollection do: [:each | parts addLast: each].
	^ parts.! !

!Character methodsFor: '*XML-Parser' stamp: 'monty 10/23/2014 12:25'!
xmlSplit: aCollection do: aBlock
	"faster, portable version"
	| start |

	start := 1.
	1 to: aCollection size do: [:end |
		((aCollection at: end) == self)
			ifTrue: [
				aBlock value: (aCollection copyFrom: start to: end - 1).
				start := end + 1]].
	start > aCollection size
		ifTrue: [aBlock value: '']
		ifFalse: [aBlock value: (aCollection copyFrom: start to: aCollection size)].! !

Error subclass: #XMLParserException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLParserException commentStamp: 'monty 1/23/2014 17:59' prior: 0!
A base class for XML exceptions to make catching XML errors easier.!

XMLParserException subclass: #XMLDOMException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLDOMException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked when a DOM tree is manipulated wrong.!

XMLParserException subclass: #XMLEncodingException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLEncodingException commentStamp: 'monty 11/11/2014 04:38' prior: 0!
Invoked for encoding errors.!

XMLParserException subclass: #XMLFileException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLFileException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked for file IO errors.!

XMLParserException subclass: #XMLHTTPException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLHTTPException commentStamp: 'monty 1/16/2014 19:40' prior: 0!
Invoked for HTTP errors.!

XMLParserException subclass: #XMLLimitException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLLimitException commentStamp: 'monty 1/25/2014 00:19' prior: 0!
Invoked when a security limit on input is exceeded. Use the messages in XMLParserLimits and DTDExternalEntityLimits to configure these limits or the removeLimits message to remove them completely.!

XMLParserException subclass: #XMLNamespaceException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLNamespaceException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when namespaces are used improperly and namespace checking is on.!

!XMLParserException class methodsFor: 'signalling' stamp: 'monty 11/6/2014 16:18'!
formatSignal: aString with: aFirstValue
	^ self
		formatSignal: aString
		withAll: (Array with: aFirstValue)! !

!XMLParserException class methodsFor: 'signalling' stamp: 'monty 11/6/2014 16:18'!
formatSignal: aString with: aFirstValue with: aSecondValue
	^ self
		formatSignal: aString
		withAll: (Array with: aFirstValue with: aSecondValue)! !

!XMLParserException class methodsFor: 'signalling' stamp: 'monty 11/6/2014 16:18'!
formatSignal: aString with: aFirstValue with: aSecondValue with: aThirdValue
	^ self
		formatSignal: aString
		withAll: (Array with: aFirstValue with: aSecondValue with: aThirdValue)! !

!XMLParserException class methodsFor: 'signalling' stamp: 'monty 11/6/2014 16:17'!
formatSignal: aString withAll: anArray
	^ self signal: (aString format: anArray)! !

!XMLParserException class methodsFor: 'signalling' stamp: 'monty 11/17/2014 23:00'!
signalMessageFrom: anException
	^ self signal: anException messageText! !

!XMLParserException methodsFor: 'appending' stamp: 'monty 2/3/2014 18:45'!
appendLineNumber: aLineNumber
	self messageText: self messageText, ' (line ', aLineNumber printString, ')'! !

XMLParserException subclass: #XMLValidationException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLValidationException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when invalid XML is detected and validation is on.!

XMLParserException subclass: #XMLWellFormednessException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

!XMLWellFormednessException commentStamp: 'monty 1/16/2014 19:41' prior: 0!
Invoked when XML markup is not well-formed.!

!String methodsFor: '*XML-Parser' stamp: 'monty 11/4/2014 14:48'!
asXMLEscapedString
	| stream |

	stream := (self class new: self size) writeStream.
	self do: [:each |
		each == $<
			ifTrue: [stream nextPutAll: '&lt;']
			ifFalse: [
				each == $&
					ifTrue: [stream nextPutAll: '&amp;']
					ifFalse: [stream nextPut: each]]].
	^ stream contents.! !

!String methodsFor: '*XML-Parser' stamp: 'JAAyer 4/14/2010 21:26'!
asXMLString
	^ XMLString string: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 1/10/2014 00:56'!
asXMLURI
	^ XMLURI fromString: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 19:44'!
expandWithNamespaceURI: aNamespaceUri
	^ aNamespaceUri
		ifEmpty: [self]
		ifNotEmpty: [((aNamespaceUri copyWithFirst: ${) copyWith: $}), self]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/22/2014 20:01'!
hasPrefixBeforeLocalName
	^ (self
		indexOf: $:
		startingAt: 2) > 0! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 19:47'!
isNamespaceAttributeName
	^ (self beginsWith: 'xmlns')
		and: [self size = 5
			or: [(self at: 6) == $:]]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/2/2014 16:02'!
isQualifiedName: aQualifiedName orLocalName: aLocalName
	^ self = aQualifiedName
		or: [(aQualifiedName size > aLocalName size)
			and: [self = aLocalName]]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/2/2014 17:45'!
isReservedAttributeName
	^ self size > 4
		and: [self beginsWith: 'xml:']! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 20:23'!
localNameAfterPrefix
	^ self localNameAfterPrefix: self prefixBeforeLocalName! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 19:41'!
localNameAfterPrefix: aPrefix
	"optimized"
	| prefixSize |

	(prefixSize := aPrefix size) > 0
		ifTrue: [^ self copyFrom: prefixSize + 2 to: self size]
		ifFalse: [^ self]! !

!String methodsFor: '*XML-Parser' stamp: 'monty 10/18/2014 05:47'!
parseXML
	^ XMLDOMParser parse: self! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 20:23'!
prefixBeforeLocalName
	"Split the name into namespace prefix
	(Optimized code)"
	| i |

	i := self size.
	[i > 1]
		whileTrue: [
			((self at: i) == $:)
				ifTrue: [^ (self copyFrom: 1 to: i - 1)].
			i := i - 1].
	^ ''.! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 20:30'!
prefixBeforeLocalName: aLocalName
	| prefixSize |

	(prefixSize := self size - aLocalName size - 1) > 0
		ifTrue: [^ self copyFrom: 1 to: prefixSize]
		ifFalse: [^ '']! !

!String methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 20:31'!
qualifyWithPrefix: aPrefix
	aPrefix size > 0
		ifTrue: [^ (aPrefix copyWith: $:), self]
		ifFalse: [^ self]! !

!Collection methodsFor: '*XML-Parser' stamp: 'monty 11/16/2014 18:09'!
includesQualifiedName: aQualifiedName orLocalName: aLocalName
	self do: [:each |
		(each
			isQualifiedName: aQualifiedName
			orLocalName: aLocalName)
			ifTrue: [^ true]].
	^ false.! !

Collection subclass: #XMLNFAStateSet
	instanceVariableNames: 'set isAccepting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAStateSet commentStamp: 'monty 1/24/2014 22:44' prior: 0!
This class is a set of NFA states. It wraps an IdentitySet and keeps track of if any of its states is accepting.!

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 1/9/2014 22:12'!
= anObject
	^ self == anObject
		or: [self class == anObject class
			and: [set = anObject set]]! !

!XMLNFAStateSet methodsFor: 'adding' stamp: 'monty 12/28/2013 18:52'!
add: aState
	isAccepting
		ifFalse: [isAccepting := aState isAccepting].

	^ set add: aState.! !

!XMLNFAStateSet methodsFor: 'enumerating' stamp: 'monty 12/28/2013 18:47'!
do: aBlock
	set do: aBlock! !

!XMLNFAStateSet methodsFor: 'comparing' stamp: 'monty 12/28/2013 18:50'!
hash
	^ set hash! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:48'!
includes: aState
	^ set includes: aState! !

!XMLNFAStateSet methodsFor: 'initialization' stamp: 'monty 12/28/2013 18:52'!
initialize
	super initialize.

	set := IdentitySet new.
	isAccepting := false.! !

!XMLNFAStateSet methodsFor: 'testing' stamp: 'monty 12/28/2013 18:53'!
isAccepting
	^ isAccepting! !

!XMLNFAStateSet methodsFor: 'removing' stamp: 'monty 12/28/2013 18:46'!
remove: aState ifAbsent: anExceptionBlock
	set
		remove: aState
		ifAbsent: [^ anExceptionBlock value].

	isAccepting
		ifTrue: [isAccepting := set anySatisfy: [:each | each isAccepting]].

	^ aState.! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 18:50'!
set
	^ set! !

!XMLNFAStateSet methodsFor: 'accessing' stamp: 'monty 12/28/2013 19:01'!
size
	^ set size! !

Collection subclass: #XMLOrderedList
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLOrderedList commentStamp: '<historical>' prior: 0!
This class functions as an OrderedCollection that will not raise exceptions when attempts are made to access absent objects. This means messages like #at:, #after:, and #first will return nil when the requested object cannot be retrieved. Messages that modify the collection based on the presence or absence of other objects such as #add:after: or #at:put: will still raise exceptions as usual.

!

XMLOrderedList subclass: #XMLObservableList
	instanceVariableNames: 'observer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLObservableList commentStamp: '<historical>' prior: 0!
This class is an is an ordered list that sends notifications to observers when objects are added to it or removed from it. Subclasses should override the #beforeAdding:, #afterAdding: and #afterRemoving: messages to send custom notifications.!

XMLObservableList subclass: #XMLAttributeList
	instanceVariableNames: 'attributesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeList commentStamp: 'monty 10/28/2014 18:54' prior: 0!
This class represents an ordered list of attribute nodes that also keeps a dictionary mapping attribute names to nodes for faster lookups. You usually shouldn't have to use this class directly.!

!XMLAttributeList methodsFor: 'adding' stamp: 'monty 10/28/2014 14:38'!
addName: aName value: aValue
	^ self addLast:
		(self attributeClass
			name: aName
			value: aValue)! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 18:37'!
afterAdding: anAttribute
	self attributesByName
		at: anAttribute name ifPresent: [self removeAttributeNodeAt: anAttribute name];
		at: anAttribute name put: anAttribute.

	observer ifNotNil: [observer addedAttribute: anAttribute].! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 18:37'!
afterRemoving: anAttribute
	self attributesByName removeKey: anAttribute name.
	observer ifNotNil: [observer removedAttribute: anAttribute].! !

!XMLAttributeList methodsFor: 'converting' stamp: 'monty 11/20/2014 19:49'!
asDictionary
	| dictionary |

	dictionary := OrderPreservingStringDictionary new: self size.
	self do: [:each | dictionary at: each name put: each value].
	^ dictionary.! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 11/6/2014 15:43'!
attributeAssociations
	^ self collect: [:each | each name -> each value]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 14:15'!
attributeClass
	^ XMLAttribute
! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 14:14'!
attributeNames
	^ self collect: [:each | each name]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 14:15'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self do: [:each | aTwoArgumentBlock value: each name value: each value]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 14:15'!
attributeNamesDo: aBlock
	self do: [:each | aBlock value: each name]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 14:14'!
attributeNodeAt: aName
	^ self attributeNodeAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 18:35'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributesByName at: aName ifAbsent: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 16:14'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self
		attributeNodeAt: aLocalName
		namespaceURI: aUri
		ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 16:14'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self
		detect: [:each |
			each name = aLocalName
				and: [each namespaceURI = aUri]]
		ifNone: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 15:00'!
attributeValueAt: aName
	^ self attributeValueAt: aName ifAbsent: ['']! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 18:35'!
attributeValueAt: aName ifAbsent: aBlock
	^ (self attributesByName
		at: aName
		ifAbsent: [^ aBlock value]) value! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 14:14'!
attributeValueAt: aName ifAbsentPut: aBlock
	^ self
		attributeValueAt: aName
		ifAbsent: [self attributeValueAt: aName put: aBlock value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'monty 10/28/2014 18:35'!
attributeValueAt: aName put: aValue
	(self includesAttribute: aName)
		ifTrue: [(self attributeNodeAt: aName) value: aValue]
		ifFalse: [self addName: aName value: aValue].
	^ aValue.! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 18:35'!
attributesByName
	^ attributesByName ifNil: [attributesByName := Dictionary new]! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 15:59'!
beforeAdding: anAttribute
	observer ifNotNil: [observer addingAttribute: anAttribute]! !

!XMLAttributeList methodsFor: 'testing' stamp: 'monty 10/28/2014 18:35'!
includesAttribute: aName
	^ self attributesByName includesKey: aName! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 18:50'!
newWithCollection: aCollection
	^ self class basicNew
		setCollection: aCollection;
		resetDictionaryFromCollection! !

!XMLAttributeList methodsFor: 'removing' stamp: 'monty 10/28/2014 14:42'!
removeAttributeNodeAt: aName
	^ self remove: (self attributeNodeAt: aName).! !

!XMLAttributeList methodsFor: 'removing' stamp: 'monty 11/23/2014 00:53'!
removeAttributeNodeAt: aName ifAbsent: aBlock
	^ self remove:
		(self
			attributeNodeAt: aName
			ifAbsent: [^ aBlock value]).! !

!XMLAttributeList methodsFor: 'notifications' stamp: 'monty 10/28/2014 18:35'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributesByName
		removeKey: anOldName;
		at: aNewName ifPresent: [:attribute | self remove: attribute];
		at: aNewName put: anAttribute! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 18:52'!
resetDictionaryFromCollection
	self attributesByName removeAll.
	collection do: [:each |
		self attributesByName
			at: each name
			put: each].! !

!XMLAttributeList methodsFor: 'private' stamp: 'monty 10/28/2014 18:35'!
setFromDictionary: anOrderPreservingDictionary
	| association |

	"hack to reuse a dictionary for speed"
	attributesByName := anOrderPreservingDictionary dictionary.

	"add the XMLAttributes in order directly to the collection
	to avoid sending notifications and also replace the value in
	the attributesByName association with the XMLAttribute"
	anOrderPreservingDictionary keysDo: [:each |
		(association := attributesByName associationAt: each ifAbsent: nil)
			value:
				(collection addLast: 
					(self attributeClass new
						setName: each
						value: association value
						element: observer))].! !

!XMLAttributeList methodsFor: 'sorting' stamp: 'monty 10/28/2014 14:15'!
sort
	self sort: [:a :b | a name <= b name]! !

!XMLAttributeList methodsFor: 'sorting' stamp: 'monty 10/28/2014 14:15'!
sorted
	^ self sorted: [:a :b | a name <= b name]! !

XMLObservableList subclass: #XMLInternalSubsetList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLInternalSubsetList commentStamp: 'JAAyer 7/27/2010 12:45' prior: 0!
This class represents the list of nodes that comprise a document's internal DTD subset.!

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/18/2014 00:54'!
afterAdding: aNode
	observer ifNotNil: [observer addedNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/18/2014 00:54'!
afterRemoving: aNode
	observer ifNotNil: [observer removedNode: aNode]! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'monty 10/29/2014 09:00'!
beforeAdding: aNode
	observer ifNotNil: [observer addingNode: aNode]! !

XMLObservableList subclass: #XMLNodeList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of child nodes for some kind of node with elements, probably either an XML document or element object. Non-modifying accessing messages like #first and #last will never raise exceptions and return nil instead.!

XMLNodeList subclass: #XMLCachingNodeList
	instanceVariableNames: 'elements elementsByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCachingNodeList commentStamp: '<historical>' prior: 0!
This is a node list that caches child elements by name, keeping both an ordered list of elements and a dictionary that allows for faster retrieval of elements by qualified or local name.

You can inject it into the DOM parser like this:
parser := XMLDOMParser on: xmlSource.
parser nodeListClass: XMLCachingNodeList.!

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 10/18/2014 00:49'!
afterAdding: aNode
	aNode isElement
		ifTrue: [
			((self last == aNode) or: [(self findElementAfter: aNode) isNil])
				ifTrue: [self elementAddedAtEnd: aNode]
				ifFalse: [self elementInsertedBeforeEnd: aNode]].
	super afterAdding: aNode.
			! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 10/18/2014 01:28'!
afterRemoving: aNode
	aNode isElement
		ifTrue: [
			self elements remove: aNode.
			self elementsByNameAt: aNode name removeElement: aNode.
			aNode hasPrefix
				ifTrue: [self elementsByNameAt: aNode localName removeElement: aNode]].
	super afterRemoving: aNode.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementAddedAtEnd: anElement
	self elements add: anElement.
	self
		elementsByNameAt: anElement name
		addElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				addElement: anElement]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elementAt: aName ifAbsent: aBlock
	| elementsWithName |

	elementsWithName := self elementsByName at: aName ifAbsent: [^ aBlock value].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName first]
		ifFalse: [elementsWithName]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementInsertedBeforeEnd: anElement
	| elementAfter |

	elementAfter := self findElementAfter: anElement.
	self elements add: anElement before: elementAfter.

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elements
	^ elements ifNil: [elements := self species new]! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:01'!
elementsAt: aName
	| elementsWithName |

	elementsWithName :=
		self elementsByName
			at: aName
			ifAbsent: [^ self species new].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 10/25/2014 14:50'!
elementsByName
	^ elementsByName ifNil: [elementsByName := Dictionary new]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName addElement: anElement
	| elementsWithName |

	elementsWithName := 
		self elementsByName
			at: aName
			ifAbsent: [
				self elementsByName at: aName put: anElement.
				^ self].

	elementsWithName isCollection
		ifTrue: [elementsWithName add: anElement]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName with: anElement)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName insertElement: anElement
	| elementsWithName elementAfter |

	(elementAfter := self findElementAfter: anElement withName: aName)
		ifNil: [^ self elementsByNameAt: aName addElement: anElement].

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [elementsWithName add: anElement before: elementAfter]
			ifFalse: [
				self elementsByName
					at: aName
					put: (self species with: anElement with: elementAfter)]! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
elementsByNameAt: aName removeElement: anElement
	| elementsWithName |

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [
				elementsWithName remove: anElement.
				elementsWithName ifEmpty: [self elementsByName removeKey: aName]]
			ifFalse: [self elementsByName removeKey: aName]! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self elements detect: aBlock ifNone: anExceptionBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsDo: aBlock
	self elements do: aBlock! !

!XMLCachingNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:02'!
elementsSelect: aBlock
	^ self elements select: aBlock! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
findElementAfter: anElement
	| previousElement |

	previousElement := nil.
	self do: [:each |
		each isElement
			ifTrue: [
				previousElement == anElement
					ifTrue: [^ each].
				previousElement := each]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 11/26/2013 21:56'!
findElementAfter: anElement withName: aName
	| previousElementWithName |

	previousElementWithName := nil.
	self do: [:each |
		(each isElement and: [each isNamed: aName])
			ifTrue: [
				(previousElementWithName == anElement)
					ifTrue: [^ each].
				previousElementWithName := each]].
	^ nil.! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 2/10/2014 23:11'!
firstElement
	^ self elements first! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:01'!
hasElements
	^ elements notNil and: [elements notEmpty]! !

!XMLCachingNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:04'!
includesElement: aName
	^ self elementsByName includesKey: aName! !

!XMLCachingNodeList methodsFor: 'accessing' stamp: 'monty 2/10/2014 23:11'!
lastElement
	^ self elements last! !

!XMLCachingNodeList methodsFor: 'private' stamp: 'monty 10/28/2014 19:29'!
newWithCollection: aCollection
	"copies as superclass for performance"
	^ XMLNodeList basicNew setCollection: aCollection! !

!XMLCachingNodeList methodsFor: 'notifications' stamp: 'monty 11/16/2014 20:23'!
renamedElement: anElement from: anOldName to: aNewName
	| oldPrefix |

	self elementsByNameAt: anOldName removeElement: anElement.
	(oldPrefix := anOldName prefixBeforeLocalName)
		ifNotEmpty: [
			self
				elementsByNameAt: (anOldName localNameAfterPrefix: oldPrefix)
				removeElement: anElement].

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement].! !

!XMLCachingNodeList methodsFor: 'sorting' stamp: 'monty 10/19/2014 07:33'!
sort: aSortBlock
	super sort: aSortBlock.
	self elements sort: aSortBlock.
	self elementsByName valuesDo: [:each |
		each isCollection
			ifTrue: [each sort: aSortBlock]].! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/28/2014 19:04'!
afterAdding: aNode
	observer ifNotNil: [
		aNode isElement
			ifTrue: [observer addedElement: aNode]
			ifFalse: [observer addedNode: aNode]]! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/28/2014 19:05'!
afterRemoving: aNode
	observer ifNotNil: [
		aNode isElement
			ifTrue: [observer removedElement: aNode]
			ifFalse: [observer removedNode: aNode]]! !

!XMLNodeList methodsFor: 'private' stamp: 'monty 10/28/2014 13:48'!
beforeAdding: aNode
	observer ifNotNil: [observer addingNode: aNode]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:08'!
contentNodes
	^ self select: [:each | each isContentNode]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:08'!
contentNodesDo: aBlock
	self do: [:each | each isContentNode ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/25/2014 22:02'!
contentStringAt: aName
	^ (self elementAt: aName ifAbsent: [^ '']) contentString! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:27'!
elementAfter: anElement
	| last |

	self elementsDo: [:each |
		last = anElement
			ifTrue: [^ each].
		last := each].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName
	^ self elementAt: aName ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:13'!
elementAt: aName ifAbsent: aBlock
	^ self
		detect: [:each | each isElement and: [each isNamed: aName]]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:32'!
elementAt: aName namespaceURI: aUri
	^ self
		elementAt: aName
		namespaceURI: aUri
		ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ (self elementsAt: aName)
		detect: [:each | each namespaceURI = aUri]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:27'!
elementBefore: anElement
	| last |

	last := self defaultValue.
	self elementsDo: [:each |
		each = anElement
			ifTrue: [^ last].
		last := each].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/26/2014 08:19'!
elementNames
	^ self elementsCollect: [:each | each name]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:07'!
elements
	^ self select: [:each | each isElement]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:12'!
elementsAt: aName
	^ self elementsSelect: [:each | each isNamed: aName]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:58'!
elementsAt: aName do: aBlock
	(self elementsAt: aName) do: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:33'!
elementsAt: aName namespaceURI: aUri
	^ (self elementsAt: aName) select: [:each | each namespaceURI = aUri]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:34'!
elementsCollect: aBlock
	^ self elements collect: aBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self elementsDetect: aBlock ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:10'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self
		detect: [:each | each isElement and: [aBlock value: each]]
		ifNone: anExceptionBlock! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:11'!
elementsDo: aBlock
	self do: [:each | each isElement ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 11/26/2013 22:08'!
elementsSelect: aBlock
	^ self select: [:each | each isElement and: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:23'!
firstElement
	^ self
		detect: [:each | each isElement]
		ifNone: [self defaultValue]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasContentNodes
	^ self anySatisfy: [:each | each isContentNode]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 11/26/2013 22:03'!
hasElements
	^ self anySatisfy: [:each | each isElement]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasStringNodes
	^ self anySatisfy: [:each | each isStringNode]! !

!XMLNodeList methodsFor: 'testing' stamp: 'monty 10/25/2014 21:04'!
includesElement: aName
	self elementAt: aName ifAbsent: [^ false].
	^ true.! !

!XMLNodeList methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:23'!
lastElement
	self reverseDo: [:each |
		each isElement
			ifTrue: [^ each]].
	^ self defaultValue.! !

!XMLNodeList methodsFor: 'notifications' stamp: 'monty 10/18/2014 01:06'!
renamedElement: anElement from: anOldName to: aNewName! !

!XMLNodeList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:06'!
sort
	self sort: [:a :b |
		(a isElement & b isElement) not
			or: [a name <= b name]]! !

!XMLNodeList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:06'!
sorted
	^ self sorted: [:a :b |
		(a isElement & b isElement) not
			or: [a name <= b name]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 20:59'!
stringNodes
	^ self select: [:each | each isStringNode]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 20:57'!
stringNodesDo: aBlock
	self do: [:each | each isStringNode ifTrue: [aBlock value: each]]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:00'!
strings
	^ self
		select: [:each | each isStringNode]
		thenCollect: [:each | each string]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'monty 10/25/2014 20:58'!
stringsDo: aBlock
	self stringNodesDo: [:each | aBlock value: each string]! !

!XMLObservableList class methodsFor: 'instance creation' stamp: 'monty 10/18/2014 01:02'!
observer: anObject
	^ self new observer: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject after: aPriorObject
	^ self with: [super add: anObject after: aPriorObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:56'!
add: anObject afterIndex: anIndex
	^ self with: [super add: anObject afterIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject before: aSubsequentObject
	^ self with: [super add: anObject before: aSubsequentObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
add: anObject beforeIndex: anIndex
	^ self with: [super add: anObject beforeIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
addFirst: anObject
	^ self with: [super addFirst: anObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'monty 2/6/2014 01:02'!
addLast: anObject
	"inlined for speed"
	self beforeAdding: anObject.
	collection addLast: anObject.
	self afterAdding: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
afterAdding: anObject
	observer ifNotNil: [observer added: anObject to: self]! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
afterRemoving: anObject
	observer ifNotNil: [observer removed: anObject from: self]! !

!XMLObservableList methodsFor: 'accessing' stamp: 'JAA 3/21/2011 20:08'!
at: anIndex put: anObject
	| presentObject |

	(presentObject := self at: anIndex)
		ifNotNil: [
			self
				with: [super at: anIndex put: nil]
				remove: presentObject].

	^ self with: [super at: anIndex put: anObject] add: anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 10/18/2014 00:51'!
beforeAdding: anObject
	observer ifNotNil: [observer adding: anObject to: self]! !

!XMLObservableList methodsFor: 'copying' stamp: 'monty 10/29/2014 09:35'!
deepCopyWithObserver: anObserver
	| newList |

	newList := (self class new: self size) observer: anObserver.
	self do: [:each | newList addLast: each copy].
	^ newList.! !

!XMLObservableList methodsFor: 'observing' stamp: 'monty 10/18/2014 00:50'!
observer
	^ observer! !

!XMLObservableList methodsFor: 'observing' stamp: 'monty 10/18/2014 00:50'!
observer: anObserver
	observer := anObserver! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 7/22/2010 10:58'!
remove: anObject ifAbsent: aBlock
	^ self with: [super remove: anObject ifAbsent: [^ aBlock value]] remove: anObject! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeAt: anIndex
	^ self with: [super removeAt: anIndex] remove: (self at: anIndex)! !

!XMLObservableList methodsFor: 'removing' stamp: 'monty 10/19/2014 14:55'!
reset
	collection do: [:each | self with: [] remove: each].
	collection reset.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 2/2/2014 21:19'!
with: aBlock add: anObject
	self beforeAdding: anObject.
	aBlock value.
	self afterAdding: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'monty 2/2/2014 21:20'!
with: aBlock remove: anObject
	aBlock value.
	self afterRemoving: anObject.
	^ anObject.! !

!XMLOrderedList class methodsFor: 'private' stamp: 'monty 1/29/2014 01:12'!
collectionClass
	^ OrderedCollection! !

!XMLOrderedList class methodsFor: 'private' stamp: 'monty 10/25/2014 20:34'!
inspectorClass
	^ self collectionClass inspectorClass! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/28/2014 05:54'!
new
	^ self new: 5! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/29/2014 01:13'!
new: aSize
	^ self basicNew setCollection: (self collectionClass new: aSize)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/29/2014 01:13'!
new: aSize withAll: anObject
	^ self basicNew setCollection: (self collectionClass new: aSize withAll: anObject)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'monty 1/29/2014 01:13'!
newFrom: aCollection
	^ self basicNew setCollection: (self collectionClass newFrom: aCollection)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:43'!
ofSize: aSize
	^ self new: aSize withAll: nil! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/23/2014 10:54'!
, aCollection 
	^ self
		copyReplaceFrom: self size + 1
		to: self size
		with: aCollection! !

!XMLOrderedList methodsFor: 'comparing' stamp: 'monty 10/22/2014 02:44'!
= aCollection
	^ self == aCollection
		or: [self species == aCollection species
			and: [collection = aCollection collection]]! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:01'!
add: anObject
	^ self addLast: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:08'!
add: anObject after: aPriorObject
	^ collection add: anObject after: aPriorObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject afterIndex: anIndex
	^ collection add: anObject afterIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject before: aSubsequentObject
	^ collection add: anObject before: aSubsequentObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:10'!
add: anObject beforeIndex: anIndex
	^ collection add: anObject beforeIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:02'!
addAll: aCollection
	^ self addAllLast: aCollection! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:10'!
addAllFirst: aCollection
	aCollection reverseDo: [:each | self addFirst: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:11'!
addAllLast: aCollection
	aCollection do: [:each | self addLast: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:11'!
addFirst: anObject
	^ collection addFirst: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:12'!
addLast: anObject
	^ collection addLast: anObject! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:10'!
after: anObject
	^ self after: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
after: anObject ifAbsent: aBlock
	^ collection after: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:56'!
allButFirst
	^ self allButFirst: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 18:05'!
allButFirst: anInteger
	^ self copyFrom: anInteger + 1 to: self size! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:19'!
allButFirstDo: aBlock
	collection allButFirstDo: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:56'!
allButLast
	^ self allButLast: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 18:02'!
allButLast: anInteger
	^ self copyFrom: 1 to: self size - anInteger! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:19'!
allButLastDo: aBlock
	collection allButLastDo: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
anyOne
	^ self first! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/23/2014 11:09'!
asArray
	^ Array newFrom: collection! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/23/2014 11:10'!
asOrderedCollection
	^ OrderedCollection newFrom: collection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 11/7/2014 19:14'!
at: anIndex
	(anIndex > 0 and: [anIndex <= collection size])
		ifTrue: [^ collection at: anIndex]
		ifFalse: [^ self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
at: anIndex ifAbsent: aBlock
	^ collection at: anIndex ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAA 3/25/2011 14:32'!
at: anIndex put: aValue
	^ collection at: anIndex put: aValue! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 00:55'!
atLast: anIndexFromEnd
	^ self atLast: anIndexFromEnd ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:18'!
atLast: anIndexFromEnd ifAbsent: aBlock
	^ collection atLast: anIndexFromEnd ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:11'!
before: anObject
	^ self before: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/19/2014 15:17'!
before: anObject ifAbsent: aBlock
	 ^ collection before: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
beginsWith: aCollection
	^ collection beginsWith: aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:14'!
capacity
	^ collection capacity! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 15:43'!
collect: aBlock
	^ self speciesNewWithCollection: (collection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 15:43'!
collect: aCollectBlock thenSelect: aSelectBlock
	^ self speciesNewWithCollection:
		(collection
			collect: aCollectBlock
			thenSelect: aSelectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAA 3/25/2011 15:35'!
collection
	^ collection! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 18:44'!
copy
	^ self newWithCollection: collection copy! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
copyAfter: anObject
	^ self newWithCollection: (collection copyAfter: anObject)! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
copyAfterLast: anObject
	^ self newWithCollection: (collection copyAfterLast: anObject)! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/29/2014 13:00'!
copyEmpty
	^ self newWithCollection: collection copyEmpty! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
copyFrom: aStart to: anEnd
	^ self newWithCollection:
		(collection
			copyFrom: (aStart max: 1)
			to: (anEnd min: self size))! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
copyReplaceAll: anOldCollection with: aNewCollection
	| copy |

	"workaround so the interal collection is always copied"
	^ self newWithCollection:
		((copy :=
			collection
				copyReplaceAll: anOldCollection
				with: aNewCollection) == collection
			ifTrue: [collection copy]
			ifFalse: [copy])! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 11/6/2014 12:29'!
copyReplaceFrom: aStart to: aStop with: aReplacementCollection
	| start stop |

	"bound the start and stop so they are always in range"
	start := (aStart max: 1) min: self size + 1.
	stop := (aStop max: start - 1) min: self size.
	^ self newWithCollection:
		(collection
			copyReplaceFrom: start
			to: stop
			with: aReplacementCollection)! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
copyUpTo: anObject
	^ self newWithCollection: (collection copyUpTo: anObject)! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
copyUpToLast: anObject
	^ self newWithCollection: (collection copyUpToLast: anObject)! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
copyWith: anElement 
	^ self newWithCollection: (collection copyWith: anElement)! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
copyWithFirst: anObject
	^ self newWithCollection: (collection copyWithFirst: anObject)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 3/12/2010 06:48'!
defaultValue
	^ nil! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:21'!
detect: aBlock
	^ self detect: aBlock ifNone: [self defaultValue]! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:21'!
detect: aBlock ifNone: anExceptionBlock
	^ collection detect: aBlock ifNone: anExceptionBlock! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 12:49'!
do: aBlock
	collection do: aBlock! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:21'!
doWithIndex: aBlock
	^ self withIndexDo: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
eighth
	^ self at: 8! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
endsWith: aCollection
	^ collection endsWith: aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
fifth
	^ self at: 5! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:56'!
first
	^ self at: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:54'!
first: anInteger
	^ self copyFrom: 1 to: anInteger! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
fourth
	^ self at: 4! !

!XMLOrderedList methodsFor: 'copying' stamp: 'monty 10/28/2014 15:44'!
grownBy: aSize
	^ self newWithCollection: (collection grownBy: aSize)! !

!XMLOrderedList methodsFor: 'comparing' stamp: 'monty 10/22/2014 02:44'!
hash
	^ collection hash! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:59'!
identityIndexOf: anObject
	^ self identityIndexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:09'!
identityIndexOf: anObject ifAbsent: aBlock
	^ collection identityIndexOf: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:59'!
indexOf: anObject
	^ self indexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:07'!
indexOf: anObject ifAbsent: aBlock
	^ collection indexOf: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:08'!
indexOf: anObject startingAt: anIndex 
	^ self indexOf: anObject startingAt: anIndex ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 05:45'!
indexOf: anObject startingAt: anIndex ifAbsent: aBlock
	^ collection
		indexOf: anObject
		startingAt: (anIndex max: 1)
		ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'testing' stamp: 'monty 10/22/2014 16:11'!
isSequenceable
	^ true! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:00'!
last
	^ self at: self size! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:55'!
last: anInteger
	^ self copyFrom: self size - anInteger + 1 to: self size! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:06'!
lastIndexOf: anObject
	^ self lastIndexOf: anObject ifAbsent: [0]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/23/2014 07:32'!
lastIndexOf: anObject ifAbsent: aBlock
	^ collection lastIndexOf: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 10/28/2014 15:44'!
newWithCollection: aCollection
	^ self class basicNew setCollection: aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
ninth
	^ self at: 9! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:23'!
pairsDo: aBlock
	collection pairsDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/22/2014 16:10'!
readStream
	^ ReadStream on: self! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 15:44'!
reject: aBlock
	^ self newWithCollection: (collection reject: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/26/2014 09:45'!
reject: aRejectBlock thenCollect: aCollectBlock
	^ self "squeak doesnt have reject:thenCollect:"
		select: [:each | (aRejectBlock value: each) not]
		thenCollect: aCollectBlock! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:33'!
remove: anObject ifAbsent: aBlock
	^ collection remove: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 16:00'!
removeAll
	[self isEmpty] whileFalse: [self removeLast]! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 15:46'!
removeAt: anIndex
	^ collection removeAt: anIndex! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:25'!
removeFirst
	^ self removeAt: 1! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:32'!
removeFirst: aNumber
	| removed |

	removed := self copyFrom: 1 to: aNumber.
	aNumber timesRepeat: [self removeFirst].
	^ removed.! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:24'!
removeLast
	^ self removeAt: self size! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 1/12/2014 20:33'!
removeLast: aNumber
	| removed |

	removed := self copyFrom: self size - (aNumber - 1) to: self size.
	aNumber timesRepeat: [self removeLast].
	^ removed.! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:02'!
replaceFrom: aStart to: aStop with: aReplacementCollection
	^ self
		replaceFrom: aStart
		to: aStop
		with: aReplacementCollection
		startingAt: 1! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 16:04'!
replaceFrom: aStart to: aStop with: aReplacementCollection startingAt: aReplacementStart
	| offset |

	offset := 0.
	[aStart + offset <= aStop]
		whileTrue: [
			self
				at: aStart + offset
				put: (aReplacementCollection at: aReplacementStart + offset).
			offset := offset + 1].! !

!XMLOrderedList methodsFor: 'removing' stamp: 'monty 10/19/2014 06:31'!
reset
	collection reset! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/22/2014 16:10'!
reverse
	^ self reversed! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:35'!
reverseDo: aBlock
	collection reverseDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/28/2014 19:13'!
reversed
	^ self newWithCollection: collection reversed! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
second
	^ self at: 2! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 15:44'!
select: aBlock
	^ self newWithCollection: (collection select: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 15:43'!
select: aSelectBlock thenCollect: aCollectBlock
	^ self speciesNewWithCollection:
		(collection
			select: aSelectBlock
			thenCollect: aCollectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 8/4/2010 13:38'!
setCollection: aCollection
	collection := aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
seventh
	^ self at: 7! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:58'!
sixth
	^ self at: 6! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
size
	^ collection size! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/22/2014 16:18'!
sort
	self sort: [:a :b | a <= b]! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'JAAyer 8/4/2010 13:43'!
sort: aSortBlock
	collection sort: aSortBlock! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/23/2014 08:06'!
sorted
	^ self sorted: [:a :b | a <= b]! !

!XMLOrderedList methodsFor: 'sorting' stamp: 'monty 10/19/2014 08:07'!
sorted: aSortBlock
	^ self copy sort: aSortBlock! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 10/28/2014 15:07'!
species
	^ XMLOrderedList! !

!XMLOrderedList methodsFor: 'private' stamp: 'monty 10/28/2014 15:43'!
speciesNewWithCollection: aCollection
	^ self species basicNew setCollection: aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'monty 10/22/2014 15:57'!
third
	^ self at: 3! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/23/2014 04:54'!
with: aCollection collect: aBlock
	| newCollection |

	newCollection := self species new: self size.
	1 to: (self size min: aCollection size) do: [:i |
		newCollection addLast:
			(aBlock
				value: (self at: i)
				value: (aCollection at: i))].
	^ newCollection.! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/23/2014 04:54'!
with: aCollection do: aBlock
	1 to: (self size min: aCollection size) do: [:i |
		(aBlock
			value: (self at: i)
			value: (aCollection at: i))]! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/28/2014 15:43'!
withIndexCollect: aTwoArgumentBlock
	^ self speciesNewWithCollection: (collection withIndexCollect: aTwoArgumentBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'monty 10/22/2014 16:47'!
withIndexDo: aBlock
	collection withIndexDo: aBlock! !

!XMLOrderedList methodsFor: 'converting' stamp: 'monty 10/22/2014 16:10'!
writeStream
	^ WriteStream on: self! !

!SequenceableCollection methodsFor: '*XML-Parser' stamp: 'monty 10/19/2014 06:23'!
lastOrNil
	"for speed to avoid at:ifAbsent: closure"
	| lastOrNilOffset |

	(lastOrNilOffset := self size) > 0
		ifTrue: [^ self at: lastOrNilOffset]
		ifFalse: [^ nil]! !

Object subclass: #DTDAttributeDefinitionParser
	instanceVariableNames: 'element attribute type defaultPragma defaultValue'
	classVariableNames: 'DefaultValueValidators NonEnumerationValidators'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDAttributeDefinitionParser commentStamp: '<historical>' prior: 0!
This class parses an attribute definition in an ATTLIST declaration and creates a validator for it.!

!DTDAttributeDefinitionParser class methodsFor: 'instance creation' stamp: 'monty 12/30/2013 23:53'!
element: anElement attribute: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		setElement: anElement
		attribute: anAttribute
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 06:30'!
initialize
	"self initialize"

	self
		initializeDefaultValueValidators;
		initializeNonEnumerationValidators.! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:53'!
initializeDefaultValueValidators
	(DefaultValueValidators := Dictionary new)
		at: 'FIXED' put: XMLFixedValueAttributeValidator;
		at: 'IMPLIED' put: XMLImpliedValueAttributeValidator;
		at: 'REQUIRED' put: XMLRequiredValueAttributeValidator! !

!DTDAttributeDefinitionParser class methodsFor: 'class initialization' stamp: 'monty 12/31/2013 02:54'!
initializeNonEnumerationValidators
	(NonEnumerationValidators := Dictionary new)
		at: 'CDATA' put: XMLCDataAttributeValidator;
		at: 'ID' put: XMLIDAttributeValidator;
		at: 'IDREF' put: XMLIDRefAttributeValidator;
		at: 'IDREFS' put: XMLIDRefsAttributeValidator;
		at: 'ENTITY' put: XMLEntityAttributeValidator;
		at: 'ENTITIES' put: XMLEntitiesAttributeValidator;
		at: 'NMTOKEN' put: XMLNameAttributeValidator;
		at: 'NMTOKENS' put: XMLNamesAttributeValidator! !

!DTDAttributeDefinitionParser methodsFor: 'private ' stamp: 'monty 10/31/2014 17:07'!
errorMissingAttributeType
	XMLValidationException signal: 'Missing attribute type'! !

!DTDAttributeDefinitionParser methodsFor: 'private ' stamp: 'monty 1/25/2014 01:25'!
hasNotationType
	^ type beginsWith: 'NOTATION '! !

!DTDAttributeDefinitionParser methodsFor: 'parsing' stamp: 'monty 1/25/2014 01:28'!
parse
	| defaultValueValidator |

	defaultValueValidator :=
		(DefaultValueValidators
			at: defaultPragma
			ifAbsent: [XMLDefaultValueAttributeValidator])
				attribute: attribute
				defaultValue: defaultValue.

	^ self validatorForTypeWithDefaultValueValidator: defaultValueValidator.! !

!DTDAttributeDefinitionParser methodsFor: 'initialization' stamp: 'monty 12/30/2013 23:52'!
setElement: anElement attribute: anAttribute type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	element := anElement.
	attribute := anAttribute.
	type := aType.
	defaultPragma := aDefaultPragma.
	defaultValue := aDefaultValue.! !

!DTDAttributeDefinitionParser methodsFor: 'private ' stamp: 'monty 10/31/2014 17:07'!
validatorClassForType
	^ NonEnumerationValidators
		at: type
		ifAbsent: [
			type isEmpty
				ifTrue: [self errorMissingAttributeType].
			(self hasNotationType)
				ifTrue: [XMLNotationListAttributeValidator]
				ifFalse: [XMLValueListAttributeValidator]]! !

!DTDAttributeDefinitionParser methodsFor: 'private ' stamp: 'monty 1/25/2014 01:29'!
validatorForTypeWithDefaultValueValidator: aDefaultValueValidator
	| attributeValidator |

	attributeValidator :=
		self validatorClassForType
			element: element
			attribute: attribute
			defaultValueValidator: aDefaultValueValidator.
	attributeValidator isValueListValidator
		ifTrue: [attributeValidator setAllowedValuesFromString: type].

	^ attributeValidator.! !

Object subclass: #DTDContentModelParser
	instanceVariableNames: 'element contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDContentModelParser commentStamp: '<historical>' prior: 0!
This is a base class for DTD content model parsers that turn element declarations into validators.!

!DTDContentModelParser class methodsFor: 'instance creation' stamp: 'monty 12/21/2013 03:14'!
element: aName contentModel: aContentModel
	^ self new
		setElement: aName
		contentModel: aContentModel! !

!DTDContentModelParser class methodsFor: 'parsing' stamp: 'monty 12/21/2013 20:00'!
parserForContentModel: aContentModel
	"the tokenizer removes all spaces from the content model"
	^ (aContentModel notEmpty and: [aContentModel first == $(])
		ifTrue: [
			(aContentModel size > 1 and: [aContentModel second == $#])
				ifTrue: [DTDMixedContentModelParser]
				ifFalse: [DTDDFAContentModelParser]]
		ifFalse: [DTDLiteralContentModelParser]! !

!DTDContentModelParser methodsFor: 'parsing' stamp: 'monty 12/21/2013 19:57'!
parse
	self subclassResponsibility! !

!DTDContentModelParser methodsFor: 'initialization' stamp: 'monty 12/21/2013 03:13'!
setElement: aName contentModel: aContentModel
	element := aName.
	contentModel := aContentModel.! !

DTDContentModelParser subclass: #DTDDFAContentModelParser
	instanceVariableNames: 'contentModelStream nameStream operatorStack nfaBuilder'
	classVariableNames: 'OperatorApplicators SpecialCharacterParsers'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDFAContentModelParser commentStamp: 'monty 12/29/2013 04:00' prior: 0!
This class parses a content model using postfix conversion and an NFABuilder to build a NFA before converting it to a DFA.!

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 06:30'!
initialize
	"self initialize"

	self
		initializeOperatorApplicators;
		initializeSpecialCharacterParsers.! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:08'!
initializeOperatorApplicators
	"use Array for faster lookup"
	(OperatorApplicators := Array new: 128)
		at: $, asciiValue put: #applyConcatenation;
		at: $| asciiValue put: #applyBranch;
		at: $? asciiValue put: #applyOptional;
		at: $* asciiValue put: #applyStar;
		at: $+ asciiValue put: #applyPlus! !

!DTDDFAContentModelParser class methodsFor: 'class initialization' stamp: 'monty 12/29/2013 05:08'!
initializeSpecialCharacterParsers
	"use Array for faster lookup"
	(SpecialCharacterParsers := Array new: 128)
		at: $( asciiValue put: #parseParen:;
		at: $) asciiValue put: #parseParen:;
		at: $, asciiValue put: #parseBinaryOperator:;
		at: $| asciiValue put: #parseBinaryOperator:;
		at: $? asciiValue put: #parseUnaryOperator:;
		at: $* asciiValue put: #parseUnaryOperator:;
		at: $+ asciiValue put: #parseUnaryOperator:! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/28/2013 19:46'!
applyBinaryOperatorOnStack
	nfaBuilder totalFragments > 1
		ifFalse: [self errorMissingOperand].

	self applyOperator: operatorStack removeLast.! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/30/2013 22:29'!
applyOperator: aCharacter
	nfaBuilder perform: (OperatorApplicators at: aCharacter asciiValue)! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/18/2013 22:44'!
errorMismatchedParens
	XMLValidationException signal: 'Mismatched parentheses in content model'! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/18/2013 22:44'!
errorMissingOperand
	XMLValidationException signal: 'Missing element name in content model'! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:17'!
errorUnexpectedPCDATA
	XMLValidationException signal: 'Unexpected #PCDATA in content model'! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 11/2/2014 09:17'!
hasOperatorOnStack
	^ (operatorStack isEmpty or: [operatorStack last == $(]) not! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/18/2013 22:44'!
isBinaryOperator: aFirstOperator higherPrecedenceThan: aSecondOperator
	^ aFirstOperator == $, and: [aSecondOperator == $|]! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 1/16/2014 05:09'!
isSpecialCharacter: aCharacter
	^ (self parserForSpecialCharacter: aCharacter) notNil! !

!DTDDFAContentModelParser methodsFor: 'parsing' stamp: 'monty 12/30/2013 22:28'!
parse
	contentModelStream := contentModel readStream.
	nameStream := WriteStream on: (String new: 16).
	operatorStack := OrderedCollection new.
	nfaBuilder := XMLNFABuilder new.

	[contentModelStream atEnd]
		whileFalse: [ 
			(self parserForSpecialCharacter: contentModelStream peek)
				ifNotNil: [:selector |
					self
						perform: selector
						with: contentModelStream next]
				ifNil: [self parseElementName]].
	self popRemainingOperators.

	^ XMLDFAElementValidator
		element: element
		dfa: nfaBuilder firstStateOfAcceptingNFA asDFAState.! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/29/2013 02:23'!
parseBinaryOperator: aCharacter
	self hasOperatorOnStack
		ifTrue: [
			(self
				isBinaryOperator: aCharacter
				higherPrecedenceThan: operatorStack last)
					ifFalse: [self applyBinaryOperatorOnStack]].

	operatorStack addLast: aCharacter.! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:17'!
parseElementName
	| matcher |

	nameStream reset.
	[contentModelStream atEnd
		or: [self isSpecialCharacter: contentModelStream peek]]
		whileFalse: [nameStream nextPut: contentModelStream next].

	matcher := nameStream contents.
	matcher = '#PCDATA'
		ifTrue: [self errorUnexpectedPCDATA].

	nfaBuilder pushMatcher: matcher.! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/29/2013 02:24'!
parseParen: aCharacter
	aCharacter == $)
		ifTrue: [self popParenthesizedOperators]
		ifFalse: [operatorStack addLast: $(].! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 12/29/2013 02:23'!
parseUnaryOperator: aCharacter
	nfaBuilder hasFragments
		ifFalse: [self errorMissingOperand].

	self applyOperator: aCharacter.! !

!DTDDFAContentModelParser methodsFor: 'private' stamp: 'monty 1/10/2014 17:07'!
parserForSpecialCharacter: aCharacter
	| asciiValue |

	^ ((asciiValue := aCharacter asciiValue) > 0
		and: [asciiValue <= SpecialCharacterParsers size])
			ifTrue: [SpecialCharacterParsers at: asciiValue]
			ifFalse: [nil]! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/28/2013 23:02'!
popParenthesizedOperators
	[operatorStack notEmpty]
		whileTrue: [
			operatorStack last == $(
				ifTrue: [
					operatorStack removeLast.
					^ self]
				ifFalse: [self applyBinaryOperatorOnStack]].

	self errorMismatchedParens.! !

!DTDDFAContentModelParser methodsFor: 'evaluation' stamp: 'monty 12/18/2013 22:44'!
popRemainingOperators
	[operatorStack notEmpty]
		whileTrue: [
			operatorStack last == $(
				ifTrue: [self errorMismatchedParens].
			self applyBinaryOperatorOnStack].

	nfaBuilder totalFragments = 1
		ifFalse: [self errorMissingOperand].! !

DTDContentModelParser subclass: #DTDLiteralContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDLiteralContentModelParser commentStamp: 'monty 2/2/2014 06:12' prior: 0!
Parser for ANY and EMPTY elements.!

!DTDLiteralContentModelParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:08'!
errorInvalidContentModel
	XMLValidationException
		formatSignal: 'Invalid content model "{1}" for element <{2}>'
		with: contentModel
		with: element.! !

!DTDLiteralContentModelParser methodsFor: 'parsing' stamp: 'monty 10/31/2014 17:08'!
parse
	contentModel = 'ANY'
		ifTrue: [^ XMLAnyElementValidator element: element].
	contentModel = 'EMPTY'
		ifTrue: [^ XMLEmptyElementValidator element: element].

	self errorInvalidContentModel.! !

DTDContentModelParser subclass: #DTDMixedContentModelParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDMixedContentModelParser commentStamp: 'monty 2/2/2014 06:13' prior: 0!
This class parses mixed content declarations like "(#PCDATA|one|two)" using a DTDMixedContentListParser.!

!DTDMixedContentModelParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:09'!
errorInvalidMixedContentList
	XMLValidationException signal: 'Missing * after mixed content list with elements'! !

!DTDMixedContentModelParser methodsFor: 'parsing' stamp: 'monty 12/29/2013 00:44'!
parse
	^ XMLMixedContentElementValidator
		element: element
		allowedElements: self parseAllowedElements! !

!DTDMixedContentModelParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:09'!
parseAllowedElements
	| allowedElements |

	allowedElements := (DTDMixedContentListParser on: contentModel) parse.
	allowedElements remove: '#PCDATA'.

	(allowedElements isEmpty or: [contentModel last == $*])
		ifFalse: [self errorInvalidMixedContentList].

	^ allowedElements.! !

Object subclass: #DTDDoctypeDefinition
	instanceVariableNames: 'rootElement externalSubsetUri isStandalone elementValidators attributeValidators generalEntities parameterEntities unparsedEntities notations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDoctypeDefinition commentStamp: '<historical>' prior: 0!
This class represents the parsed internal and external subset of a DTD.!

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/16/2014 05:41'!
addExternalGeneralEntity: aName resolver: aResolver uri: aSystemUri from: aDocumentOrEntityUri
	self generalEntities
		at: aName
		ifAbsentPut: [
			aResolver
				resolveExternalGeneralEntity: aName
				uri: aSystemUri
				from: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/16/2014 05:41'!
addExternalParameterEntity: aName resolver: aResolver uri: aSystemUri from: aDocumentOrEntityUri
	self parameterEntities
		at: aName
		ifAbsentPut: [
			aResolver
				resolveExternalParameterEntity: aName
				uri: aSystemUri
				from: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:44'!
addGeneralEntity: aName replacement: aReplacement from: aDocumentOrEntityUri
	self generalEntities
		at: aName
		ifAbsentPut: [
			DTDGeneralEntity 
				name: aName
				replacement: aReplacement
				uri: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 11/14/2013 23:59'!
addNotation: aName
	self notations add: aName! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:45'!
addParameterEntity: aName replacement: aReplacement from: aDocumentOrEntityUri
	self parameterEntities
		at: aName
		ifAbsentPut: [
			DTDParameterEntity
				name: aName
				replacement: aReplacement
				uri: aDocumentOrEntityUri]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 1/4/2014 03:49'!
addUnparsedEntity: aName ndata: aNotation
	self unparsedEntities
		at: aName
		ifAbsentPut: [aNotation]! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 2/2/2014 23:31'!
addValidatorForAttributeIn: anElement named: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	| attributeValidatorsForElement validator |

	attributeValidatorsForElement :=
		self attributeValidators
			at: anElement
			ifAbsentPut: [Dictionary new].
	(attributeValidatorsForElement includesKey: aName)
		ifTrue: [^ self].

	validator :=
		(DTDAttributeDefinitionParser
			element:  anElement
			attribute: aName
			type: aType
			defaultPragma: aDefaultPragma
			defaultValue: aDefaultValue) parse.

	validator
		validateAndAddTo: attributeValidatorsForElement
		usingDeclarationsFrom: self.! !

!DTDDoctypeDefinition methodsFor: 'adding' stamp: 'monty 12/21/2013 03:14'!
addValidatorForElement: anElement contentModel: aContentModel
	self elementValidators
		at: anElement
		ifAbsentPut: [
			((DTDContentModelParser parserForContentModel: aContentModel)
				element: anElement
				contentModel: aContentModel) parse]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/19/2013 01:01'!
attributeValidators
	^ attributeValidators ifNil: [attributeValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/18/2013 19:26'!
elementValidators
	^ elementValidators ifNil: [elementValidators := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/4/2014 03:48'!
externalSubsetURI
	^ externalSubsetUri ifNil: [externalSubsetUri := '']! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 20:51'!
generalEntities
	^ generalEntities ifNil: [generalEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 20:50'!
generalEntityAt: aName
	^ self generalEntities at: aName ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/18/2013 19:45'!
hasAttributeValidators
	^ attributeValidators notNil and: [attributeValidators notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/18/2013 19:45'!
hasElementValidators
	^ elementValidators notNil and: [elementValidators notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 1/4/2014 03:49'!
hasExternalSubset
	^ self isStandalone not and: [self externalSubsetURI notEmpty]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/2/2014 22:17'!
hasValidators
	^ (attributeValidators notNil
		and: [attributeValidators notEmpty])
			or: [elementValidators notNil
				and: [elementValidators isEmpty]]! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/15/2013 00:00'!
includesNotation: aName
	^ self notations includes: aName! !

!DTDDoctypeDefinition methodsFor: 'testing' stamp: 'monty 11/22/2013 09:11'!
isStandalone
	^ isStandalone ifNil: [isStandalone := false]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/22/2013 09:11'!
isStandalone: aBoolean
	isStandalone := aBoolean! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 23:59'!
notations
	^ notations ifNil: [notations := Set new]! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 20:51'!
parameterEntities
	^ parameterEntities ifNil: [parameterEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 20:51'!
parameterEntityAt: aName
	^ self parameterEntities at: aName ifAbsent: [nil]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/19/2013 23:42'!
rootElement
	^ rootElement ifNil: ['']! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 1/4/2014 03:48'!
rootElement: aRootElement externalSubsetURI: aUri
	rootElement := aRootElement.
	externalSubsetUri := aUri.! !

!DTDDoctypeDefinition methodsFor: 'private' stamp: 'monty 11/14/2013 23:50'!
unparsedEntities
	^ unparsedEntities ifNil: [unparsedEntities := Dictionary new]! !

!DTDDoctypeDefinition methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:55'!
unparsedEntityAt: aName
	^ self unparsedEntities at: aName ifAbsent: [nil]! !

Object subclass: #DTDExternalEntityLimits
	instanceVariableNames: 'maxExternalEntities maxExternalEntitySize resolvesLocalFromNonLocal'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityLimits commentStamp: 'monty 1/25/2014 04:01' prior: 0!
This class places security limits on the number and size of external parsed entities. You can set these using the configuration messages in DTDExternalEntityResolver or remove them using removeLimits.!

!DTDExternalEntityLimits class methodsFor: 'accessing' stamp: 'monty 10/3/2014 23:23'!
defaultMaxExternalEntitySize
	^ XMLParserLimits defaultExternalSubsetReadLimit! !

!DTDExternalEntityLimits methodsFor: 'initialization' stamp: 'monty 1/26/2014 04:19'!
initialize
	super initialize.

	maxExternalEntities := 12.
	maxExternalEntitySize := self class defaultMaxExternalEntitySize.
	resolvesLocalFromNonLocal := false.! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntities
	^ maxExternalEntities! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntities: aLimit
	maxExternalEntities := aLimit! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntitySize
	^ maxExternalEntitySize! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 06:14'!
maxExternalEntitySize: aLimit
	maxExternalEntitySize := aLimit! !

!DTDExternalEntityLimits methodsFor: 'removing' stamp: 'monty 1/24/2014 06:48'!
removeAll
	maxExternalEntities := nil.
	maxExternalEntitySize := nil.
	resolvesLocalFromNonLocal := nil.! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/24/2014 06:47'!
resolvesLocalFromNonLocal
	^ resolvesLocalFromNonLocal! !

!DTDExternalEntityLimits methodsFor: 'accessing' stamp: 'monty 1/24/2014 06:47'!
resolvesLocalFromNonLocal: aBoolean
	resolvesLocalFromNonLocal := aBoolean! !

Object subclass: #DTDExternalEntityResolver
	instanceVariableNames: 'externalEntityLimits totalResolvedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityResolver commentStamp: 'monty 1/23/2014 05:48' prior: 0!
This class resolves external entity references using XMLURI, XMLHTTPRequest, and XMLFileHandle.!

DTDExternalEntityResolver subclass: #DTDCachingEntityResolver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

DTDCachingEntityResolver class
	instanceVariableNames: 'entityCache'!

!DTDCachingEntityResolver commentStamp: 'monty 1/10/2014 08:16' prior: 0!
This resolver can cache entities so they don't have to be retrieved again (like over HTTP).!

DTDCachingEntityResolver class
	instanceVariableNames: 'entityCache'!

!DTDCachingEntityResolver class methodsFor: 'clearing' stamp: 'monty 11/15/2014 20:59'!
clearEntityCache
	"self clearEntityCache"
	self entityCache clear! !

!DTDCachingEntityResolver class methodsFor: 'private' stamp: 'monty 11/15/2014 20:56'!
entityCache
	^ entityCache ifNil: [entityCache := XMLKeyValueCache maxSize: 64]! !

!DTDCachingEntityResolver class methodsFor: 'private' stamp: 'monty 11/15/2014 21:02'!
entityCacheAt: aUri ifAbsentPut: aBlock
	^ self entityCache
		at: aUri
		ifAbsentPut: aBlock! !

!DTDCachingEntityResolver class methodsFor: 'private' stamp: 'monty 11/15/2014 21:00'!
entityCacheAt: aUri put: anEntity
	^ self entityCache
		at: aUri
		put: anEntity.! !

!DTDCachingEntityResolver class methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:57'!
maxCachedEntities
	^ self entityCache maxSize! !

!DTDCachingEntityResolver class methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:56'!
maxCachedEntities: anInteger
	self entityCache maxSize: anInteger! !

!DTDCachingEntityResolver methodsFor: 'clearing' stamp: 'monty 1/13/2014 03:45'!
clearEntityCache
	self class clearEntityCache! !

!DTDCachingEntityResolver methodsFor: 'accessing' stamp: 'monty 1/13/2014 17:52'!
maxCachedEntities
	^ self class maxCachedEntities! !

!DTDCachingEntityResolver methodsFor: 'accessing' stamp: 'monty 1/13/2014 17:53'!
maxCachedEntities: anInteger
	self class maxCachedEntities: anInteger! !

!DTDCachingEntityResolver methodsFor: 'resolving' stamp: 'monty 1/16/2014 06:38'!
resolveEntityURI: aUri
	| uri |

	^ (uri := aUri asXMLURI) isLocalURI
		ifTrue: [super resolveEntityURI: uri]
		ifFalse: [
			ReadStream on:
				(self class
					entityCacheAt: uri
					ifAbsentPut: [(super resolveEntityURI: uri) contents])]! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 10/31/2014 21:03'!
checkExternalEntityLimit
	(self maxExternalEntities notNil
		and: [self totalResolvedEntities + 1 > self maxExternalEntities])
		ifTrue: [self erorrExternalEntityLimitExceeded]! !

!DTDExternalEntityResolver methodsFor: 'combining' stamp: 'monty 10/31/2014 21:01'!
combineURI: aBaseUri with: aRelativeUri
	| baseUri combinedUri |

	baseUri := XMLURI fromString: aBaseUri.
	combinedUri := baseUri / aRelativeUri.
	(combinedUri isLocalURI
		and: [baseUri isLocalURI not
			and: [self resolvesLocalFromNonLocal notNil
				and: [self resolvesLocalFromNonLocal not]]])
		ifTrue: [self errorLocalFromNonLocal].

	^ combinedUri.! !

!DTDExternalEntityResolver methodsFor: 'accessing' stamp: 'monty 1/22/2014 18:55'!
defaultDocumentURI
	^ XMLFileHandle currentWorkingDirectory, '/default'! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 11/6/2014 12:21'!
erorrExternalEntityLimitExceeded
	XMLLimitException signal: 'External entity limit exceeded; use #maxExternalEntities: to change'! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 10/31/2014 21:33'!
errorLocalFromNonLocal
	XMLLimitException signal:
		'Local URI referenced from non-local URI; use #resolvesLocalFromNonLocal: to change'! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/16/2014 06:15'!
externalEntityLimits
	^ externalEntityLimits ifNil: [externalEntityLimits := DTDExternalEntityLimits new]! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/23/2014 05:45'!
incrementEntityTotal
	totalResolvedEntities := (totalResolvedEntities ifNil: [0]) + 1! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:17'!
maxExternalEntities
	^ self externalEntityLimits maxExternalEntities! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:54'!
maxExternalEntities: aLimit
	self externalEntityLimits maxExternalEntities: aLimit! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:20'!
maxExternalEntitySize
	^ self externalEntityLimits maxExternalEntitySize! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:54'!
maxExternalEntitySize: aLimit
	self externalEntityLimits maxExternalEntitySize: aLimit! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/16/2014 06:16'!
removeLimits
	self externalEntityLimits removeAll! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 1/23/2014 05:44'!
resolveEntityURI: aUri
	| readStream |

	self checkExternalEntityLimit.
	readStream := aUri asXMLURI getUpToLimit: self maxExternalEntitySize.
	self incrementEntityTotal.

	^ readStream.! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 2/18/2014 04:58'!
resolveExternalEntity: aName uri: anEntityUri from: aSourceUri class: aClass
	| combinedUri replacement replacementStream |

	combinedUri := self combineURI: aSourceUri with: anEntityUri.

	replacementStream := self resolveEntityURI: combinedUri.
	replacement  := replacementStream contents.
	replacementStream close.

	^ aClass
		name: aName
		replacement: replacement
		uri: combinedUri asString.! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 1/16/2014 05:41'!
resolveExternalGeneralEntity: aName uri: anEntityUri from: aSourceUri
	^ self
		resolveExternalEntity: aName
		uri: anEntityUri
		from: aSourceUri
		class: DTDExternalGeneralEntity! !

!DTDExternalEntityResolver methodsFor: 'resolving' stamp: 'monty 1/16/2014 05:41'!
resolveExternalParameterEntity: aName uri: anEntityUri from: aSourceUri
	^ self
		resolveExternalEntity: aName
		uri: anEntityUri
		from: aSourceUri
		class: DTDExternalParameterEntity! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/24/2014 06:50'!
resolvesLocalFromNonLocal
	^ self externalEntityLimits resolvesLocalFromNonLocal! !

!DTDExternalEntityResolver methodsFor: 'configuration' stamp: 'monty 1/24/2014 06:50'!
resolvesLocalFromNonLocal: aBoolean
	self externalEntityLimits resolvesLocalFromNonLocal: aBoolean! !

!DTDExternalEntityResolver methodsFor: 'private' stamp: 'monty 1/16/2014 06:40'!
totalResolvedEntities
	^ totalResolvedEntities ifNil: [totalResolvedEntities := 0]! !

Object subclass: #DTDParsedEntity
	instanceVariableNames: 'name replacement uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParsedEntity commentStamp: '<historical>' prior: 0!
This is a base class for internal and external parsed entities with replacements that can be inserted into an XML document.!

DTDParsedEntity subclass: #DTDGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDGeneralEntity commentStamp: 'monty 1/5/2014 00:54' prior: 0!
This class is for general parsed entities with replacements.
!

DTDGeneralEntity subclass: #DTDBypassedGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDBypassedGeneralEntity commentStamp: 'monty 10/7/2014 02:10' prior: 0!
A bypassed general entity is replaced by an escaped reference to itself.!

!DTDBypassedGeneralEntity methodsFor: 'replacement' stamp: 'monty 11/17/2014 01:51'!
constructReplacementOn: aTokenizer
	aTokenizer constructReplacementForBypassedGeneralEntity: self! !

DTDGeneralEntity subclass: #DTDExternalGeneralEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalGeneralEntity commentStamp: 'monty 10/7/2014 02:19' prior: 0!
A general entity with an external replacement.!

!DTDExternalGeneralEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:57'!
isExternal
	^ true! !

!DTDGeneralEntity methodsFor: 'replacement' stamp: 'monty 11/16/2014 21:00'!
constructReplacementOn: aTokenizer
	aTokenizer constructReplacementForGeneralEntity: self! !

!DTDGeneralEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:03'!
escapedReferenceSymbol
	^ '&amp;'! !

!DTDGeneralEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	^ $&! !

DTDParsedEntity subclass: #DTDParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParameterEntity commentStamp: 'monty 11/20/2014 04:05' prior: 0!
This class is for parameter entities with replacements.!

DTDParameterEntity subclass: #DTDBypassedParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDBypassedParameterEntity commentStamp: 'monty 10/7/2014 02:18' prior: 0!
A bypassed parameter entity is replaced by an escaped reference inside entity declaration replacements or an empty string outside replacements in DTD subsets.!

!DTDBypassedParameterEntity methodsFor: 'replacement' stamp: 'monty 11/17/2014 01:51'!
constructReplacementOn: aTokenizer
	aTokenizer constructReplacementForBypassedParameterEntity: self! !

!DTDBypassedParameterEntity methodsFor: 'replacement' stamp: 'monty 12/7/2013 19:14'!
replacement
	^ self asEscapedReference! !

DTDParameterEntity subclass: #DTDExternalParameterEntity
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalParameterEntity commentStamp: 'monty 10/7/2014 02:19' prior: 0!
A parameter entity with an external replacement.!

!DTDExternalParameterEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:58'!
isExternal
	^ true! !

!DTDParameterEntity methodsFor: 'replacement' stamp: 'monty 11/16/2014 21:00'!
constructReplacementOn: aTokenizer
	aTokenizer constructReplacementForParameterEntity: self! !

!DTDParameterEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:23'!
escapedReferenceSymbol
	^ '&#37;'! !

!DTDParameterEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	^ $%! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 06:02'!
name: aString replacement: aValueString
	^ self new
		setName: aString
		replacement: aValueString
		uri: nil! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 11/16/2013 02:55'!
name: aString replacement: aValueString uri: aUri
	^ self new
		setName: aString
		replacement: aValueString
		uri: aUri! !

!DTDParsedEntity class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:37'!
named: aString
	^ self
		name: aString
		replacement: ''! !

!DTDParsedEntity methodsFor: 'converting' stamp: 'monty 12/7/2013 19:01'!
asEscapedReference
	^ self asReferenceStartingWith: self escapedReferenceSymbol! !

!DTDParsedEntity methodsFor: 'converting' stamp: 'monty 12/7/2013 19:11'!
asReference
	^ self asReferenceStartingWith: self referenceSymbol asString! !

!DTDParsedEntity methodsFor: 'private' stamp: 'monty 12/9/2013 01:29'!
asReferenceStartingWith: aString
	^ (aString, self name) copyWith: $;! !

!DTDParsedEntity methodsFor: 'replacement' stamp: 'monty 11/16/2014 21:00'!
constructReplacementOn: aTokenizer
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:02'!
escapedReferenceSymbol
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'testing' stamp: 'monty 11/16/2013 02:56'!
isExternal
	^ false! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/13/2013 23:45'!
name
	^ name ifNil: [name := '']! !

!DTDParsedEntity methodsFor: 'printing' stamp: 'monty 11/20/2013 01:23'!
printOn: aStream
	self asReference printOn: aStream
	! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/12/2013 20:09'!
referenceSymbol
	self subclassResponsibility! !

!DTDParsedEntity methodsFor: 'replacement' stamp: 'monty 11/13/2013 19:15'!
replacement
	^ replacement ifNil: [replacement := '']! !

!DTDParsedEntity methodsFor: 'initialization' stamp: 'monty 11/16/2013 02:55'!
setName: aName replacement: aReplacement uri: aUri
	name := aName.
	replacement := aReplacement.
	uri := aUri.! !

!DTDParsedEntity methodsFor: 'accessing' stamp: 'monty 11/16/2013 02:55'!
uri
	^ uri ifNil: [uri := '']! !

Object subclass: #DTDValueListParser
	instanceVariableNames: 'valueListStream valueStream listValues'
	classVariableNames: 'ListValueDelimiters'
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDValueListParser commentStamp: '<historical>' prior: 0!
This is an abstract parser of lists in the form of "(one | two | three)"!

DTDValueListParser subclass: #DTDEnumerationListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDEnumerationListParser commentStamp: 'monty 1/25/2014 03:20' prior: 0!
This parses attribute value enumeration lists.!

!DTDEnumerationListParser methodsFor: 'testing' stamp: 'monty 1/25/2014 03:38'!
isValidListValue: aValue
	^ true! !

DTDValueListParser subclass: #DTDMixedContentListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDMixedContentListParser commentStamp: 'monty 1/25/2014 03:19' prior: 0!
This parses element mixed content  (#PCDATA| ... ) lists.!

!DTDMixedContentListParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:21'!
errorInvalidMixedContentList
	XMLValidationException signal: 'Mixed content list can only be followed by "*"'! !

!DTDMixedContentListParser methodsFor: 'testing' stamp: 'monty 2/10/2014 02:43'!
isValidListValue: aValue
	^ aValue = '#PCDATA' or: [XMLWellFormedParserTokenizer isName: aValue]! !

!DTDMixedContentListParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:21'!
parseAfterList
	(valueListStream atEnd
		or: [valueListStream next == $*
			and: [valueListStream atEnd]])
		ifFalse: [self errorInvalidMixedContentList].! !

DTDValueListParser subclass: #DTDNotationListParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDNotationListParser commentStamp: 'monty 1/25/2014 03:36' prior: 0!
This class parses notation attribute value lists.!

!DTDNotationListParser methodsFor: 'testing' stamp: 'monty 2/10/2014 02:43'!
isValidListValue: aValue
	^ XMLWellFormedParserTokenizer isName: aValue! !

!DTDNotationListParser methodsFor: 'initialization' stamp: 'monty 10/30/2014 19:26'!
on: aString
	super on:
		(aString
			copyFrom: (aString indexOf: $()
			to: aString size)! !

!DTDValueListParser class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 06:31'!
initialize
	"self initialize"

	(ListValueDelimiters := BitmapCharacterSet new)
		add: $|;
		add: $);
		addAll: Character separators! !

!DTDValueListParser class methodsFor: 'instance creation' stamp: 'monty 10/30/2014 12:58'!
on: aString
	^ self new on: aString! !

!DTDValueListParser methodsFor: 'private' stamp: 'monty 10/31/2014 21:20'!
errorDuplicateListValue: aValue
	XMLValidationException
		formatSignal: 'Duplicate value "{1}" in list'
		with: aValue! !

!DTDValueListParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:19'!
errorEmptyListValue
	XMLValidationException signal: 'Empty list value'! !

!DTDValueListParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:19'!
errorInvalidListValue: aValue
	XMLValidationException
		formatSignal: 'Invalid value "{1}" in list'
		with: aValue! !

!DTDValueListParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:25'!
errorMissingListEnd
	XMLValidationException signal: 'Expected end of value list ")" character'! !

!DTDValueListParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:24'!
errorMissingListStart
	XMLValidationException signal: 'Expected value list starting with "("'! !

!DTDValueListParser methodsFor: 'testing' stamp: 'monty 12/31/2013 02:08'!
isValidListValue: aValue
	self subclassResponsibility! !

!DTDValueListParser methodsFor: 'initialization' stamp: 'monty 11/6/2014 02:27'!
on: aString
	valueListStream := ReadStream on: aString.
	valueStream := WriteStream on: (String new: (aString size / 4) asInteger).
	listValues := Set new.! !

!DTDValueListParser methodsFor: 'parsing' stamp: 'monty 10/31/2014 17:24'!
parse
	(valueListStream atEnd not
		and: [valueListStream next == $(])
		ifFalse: [self errorMissingListStart].

	[valueListStream atEnd
		or: [valueListStream peek == $)]]
		whileFalse: [
			valueListStream skipSeparators.
			self parseListValue.
			valueListStream skipSeparators.

			(valueListStream atEnd not
				and: [valueListStream peek == $|])
				ifTrue: [valueListStream next]].
	self parseListTerminator.

	^ listValues.! !

!DTDValueListParser methodsFor: 'private' stamp: 'monty 12/31/2013 02:46'!
parseAfterList! !

!DTDValueListParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:25'!
parseListTerminator
	(valueListStream atEnd not
		and: [valueListStream next == $)])
		ifFalse: [self errorMissingListEnd].

	self parseAfterList.! !

!DTDValueListParser methodsFor: 'private' stamp: 'monty 10/31/2014 17:20'!
parseListValue
	| value oldSize |

	valueStream reset.
	[valueListStream atEnd
		or: [ListValueDelimiters includes: valueListStream peek]]
		whileFalse: [valueStream nextPut: valueListStream next].

	(value := valueStream contents)
		ifEmpty: [self errorEmptyListValue].
	(self isValidListValue: value)
		ifFalse: [self errorInvalidListValue: value].

	"checking the size avoids includes: hash lookup"
	oldSize := listValues size.
	listValues add: value.
	(listValues size > oldSize)
		ifFalse: [self errorDuplicateListValue: value].! !

Object subclass: #SAXHandler
	instanceVariableNames: 'input documentUri configuration isInInternalSubset isInCDataSection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXHandler commentStamp: 'monty 10/3/2014 23:17' prior: 0!
This class is an XML parser that uses event handling. The acronym "SAX" refers to Java's "Simple API for XML," on which SAXHandler's API is based. To use this class, create a subclass and override handlers under the "content" and "lexical" categories as needed. By default, namespace support and validation are enabled (but not external entity resolution).

To instantiate a SAXHandler, send any of the #on: or #parse- messages to a subclass.

The class-side #parse- messages take some input source and parse it immediately, returning the result of #parseDocument:
	SAXHandlerSubclass parse: xml.
	SAXHandlerSubclass parseURL: aUrl. 
	SAXHandlerSubclass parse: xml usingNamespaces: false

The #on: messages create new parsers on the given input string, stream, URL, or file and return it; the instance can then be further configured and ultimately sent #parseDocument to parse the given input:
	(SAXHandlerSubclass on: xml)
		isValidating: true;
		resolvesExternalEntities: true;
		parseDocument.

There is also #parseDocumentWhile: to stop parsing before the end.
	(SAXHandlerSubclass on: xml)
		parseDocumentWhile: [self shouldKeepParsing].

To parse incrementally, send #parser to an instance to get the underlying XMLParser object and send it #parseToken repeatedly:
	(handler := SAXHandlerSubclass on: xml)
		isValidating: false; 
		preservesUndeclaredEntityReferences: true.
	parser := handler parser.

	"Only parse the first 10 tokens:"
	10 timesRepeat: [parser parseToken].

There are security limits on input you can remove with #removeLimits or change with messages like #documentReadLimit:
	(SAXHandlerSubclass on: xml)
		removeLimits;
		documentReadLimit: newReadLimit;
		maxEntityReplacementDepth: newMaxEntityDepth;
		parseDocument.!

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 11/18/2014 17:18'!
on: aStringOrStream
	^ self new on: aStringOrStream! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 11/18/2014 17:17'!
on: aStringOrStream documentURI: aUri
	^ self new
		on: aStringOrStream
		documentURI: aUri! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 1/21/2014 23:36'!
onFileNamed: aFileName
	^ self
		on: (XMLFileHandle path: aFileName) readStream
		documentURI: aFileName! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 1/25/2014 19:03'!
onURL: aUrlString
	^ self
		onURL: aUrlString
		upToLimit: XMLParserLimits defaultDocumentReadLimit! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'monty 1/25/2014 19:09'!
onURL: aUrlString upToLimit: aLimit
	^ (self
		on: ((XMLHTTPRequest url: aUrlString) getUpToLimit: aLimit)
		documentURI: aUrlString)
		documentReadLimit: aLimit! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/10/2010 05:21'!
parse: aStringOrStream
	^ (self on: aStringOrStream) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/10/2010 05:30'!
parse: aStringOrStream usingNamespaces: aBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/15/2013 23:33'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/16/2013 07:30'!
parse: aStringOrStream usingNamespaces: aNamespaceBoolean validation: aValidationBoolean externalEntities: anEntityBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aNamespaceBoolean;
		isValidating: aValidationBoolean;
		resolvesExternalEntities: anEntityBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'deprecated' stamp: 'monty 10/27/2014 12:13'!
parseDocumentFrom: aStringOrStream
	self deprecated: 'use #parse: instead'.

	^ self parse: aStringOrStream.! !

!SAXHandler class methodsFor: 'parsing' stamp: 'JAAyer 11/13/2010 20:43'!
parseFileNamed: aFileName
	^ (self onFileNamed: aFileName) parseDocument! !

!SAXHandler class methodsFor: 'parsing' stamp: 'monty 11/15/2013 23:35'!
parseURL: aUrl
	^ (self onURL: aUrl) parseDocument! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 12/30/2013 23:57'!
attributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:27'!
characters: aString
	"This call corresponds to the Java SAX call
	characters(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
comment: aCommentString
	"This call corresponds to the Java SAX ext call
	comment(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:14'!
configuration
	^ configuration! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:13'!
configuration: aConfiguration
	configuration := aConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'JAAyer 11/23/2010 15:35'!
configurationClass
	^ XMLConfiguration! !

!SAXHandler methodsFor: 'private' stamp: 'monty 12/27/2013 03:36'!
currentScope: aNamespaceScope! !

!SAXHandler methodsFor: 'private' stamp: 'monty 1/9/2014 05:28'!
defaultDocumentURI
	^ self externalEntityResolver defaultDocumentURI! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/19/2014 03:51'!
documentReadLimit
	^ self parserLimits documentReadLimit! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 10/3/2014 00:11'!
documentReadLimit: aLimit
	"change the document read limit or nil to disable"

	self parserLimits documentReadLimit: aLimit! !

!SAXHandler methodsFor: 'accessing' stamp: 'monty 11/15/2013 21:14'!
documentURI
	^ documentUri ifNil: [documentUri := self defaultDocumentURI]! !

!SAXHandler methodsFor: 'private' stamp: 'monty 2/9/2014 02:43'!
driverClass
	^ SAXParserDriver! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/13/2013 23:54'!
elementDeclaration: aName contentModel: aContentModel
! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 2/21/2010 15:13'!
endCData
	"This call corresponds to the Java SAX ext call
	endCData()."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/27/2014 14:38'!
endCDataSection
	isInCDataSection := false! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/12/2013 18:01'!
endDTD
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/27/2010 06:50'!
endDocument
	"This call corresponds to the Java SAX call
	endDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:23'!
endElement: aQualifiedName
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:50'!
endElement: aQualifiedName prefix: aPrefix uri: aUri
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:13'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName prefix: aPrefix uri:  aUri! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
endEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	endEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/27/2014 14:45'!
endInternalSubset
	isInInternalSubset := false! !

!SAXHandler methodsFor: 'content' stamp: 'JAA 3/22/2011 11:32'!
endPrefixMapping: aPrefix
	"This call corresonds to the Java SAX call
	endPrefixMapping(java.lang.String prefix)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/8/2013 02:33'!
externalEntityResolver
	^ self configuration externalEntityResolver! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 01:55'!
externalEntityResolver: anExternalEntityResolver
	"This can be used to set a different DTDExternalEntityResolver"

	self configuration externalEntityResolver: anExternalEntityResolver! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/19/2014 03:52'!
externalSubsetReadLimit
	^ self parserLimits externalSubsetReadLimit! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 10/3/2014 00:11'!
externalSubsetReadLimit: aLimit
	"change the external subset read limit or nil to disable"

	self parserLimits externalSubsetReadLimit: aLimit! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:01'!
generalEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:02'!
generalEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:25'!
ignorableWhitespace: aString
	"This call corresonds to the Java SAX call
	ignorableWhitespace(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/22/2014 03:19'!
input
	^ input! !

!SAXHandler methodsFor: 'testing' stamp: 'monty 10/27/2014 09:15'!
isInCDataSection
	^ isInCDataSection! !

!SAXHandler methodsFor: 'testing' stamp: 'monty 11/18/2014 01:24'!
isInInternalSubset
	^ isInInternalSubset! !

!SAXHandler methodsFor: 'configuration' stamp: 'JAAyer 11/23/2010 14:20'!
isValidating
	^ self configuration isValidating! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:00'!
isValidating: aBoolean
	"If true and document contstraints are present (in a DTD),
	the document will be checked against them (default).

	Enabling disables preservesUndeclaredEntityReferences.
	Disabling disables requiresDocumentConstraints."

	self configuration isValidating: aBoolean! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/19/2014 03:52'!
maxEntityReplacementDepth
	^ self parserLimits maxEntityReplacementDepth! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 10/3/2014 00:11'!
maxEntityReplacementDepth: aLimit
	"change the max replacement depth or nil to disable"

	self parserLimits maxEntityReplacementDepth: aLimit! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 10/4/2014 03:34'!
notationDeclaration: aName publicID: aPublicId systemID: aSystemId
! !

!SAXHandler methodsFor: 'initialization' stamp: 'monty 11/18/2014 17:17'!
on: aStringOrStream
	self
		on: aStringOrStream
		documentURI: nil! !

!SAXHandler methodsFor: 'initialization' stamp: 'monty 11/18/2014 17:19'!
on: aStringOrStream documentURI: aUri
	input := aStringOrStream.
	documentUri := aUri.
	configuration
		ifNil: [configuration := self configurationClass new].
	isInInternalSubset := false.
	isInCDataSection := false.! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:05'!
parameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!SAXHandler methodsFor: 'declaration' stamp: 'monty 11/14/2013 04:05'!
parameterEntityDeclaration: aName replacement: aReplacement! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:37'!
parseDocument
	self parser parseDocument.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 07:17'!
parseDocumentWhile: aBlock
	self parser parseDocumentWhile: aBlock.

	^ self parsingResult.! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 10/22/2014 03:29'!
parser
	self configuration parserHandlerClass: self class.

	^ self parserClass
		driver: (self driverClass saxHandler: self)
		on: self input.! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/3/2014 00:18'!
parserClass
	^ XMLParser! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/16/2014 18:46'!
parserLimits
	^ self configuration parserLimits! !

!SAXHandler methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:37'!
parsingResult
	^ self! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/7/2013 19:18'!
preservesUndeclaredEntityReferences
	^ self configuration preservesUndeclaredEntityReferences! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:01'!
preservesUndeclaredEntityReferences: aBoolean
	"If true, references to undeclared entities in content are
	preserved (not default).
	Disables validation"

	self configuration preservesUndeclaredEntityReferences: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'JAA 3/22/2011 11:31'!
processingInstruction: aTarget data: aDataString
	"This call corresonds to the Java SAX call
	processingInstruction(java.lang.String target, java.lang.String data)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 10/3/2014 00:16'!
removeLimits
	"remove all input security limits (unsafe)"

	self parserLimits removeAll.
	self externalEntityResolver removeLimits.! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/21/2013 20:07'!
requiresDocumentConstraints
	^ self configuration requiresDocumentConstraints! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:23'!
requiresDocumentConstraints: aBoolean
	"If true, constraints on elements and attributes (in a DTD)
	will be required (not default).
	Enables validation."

	self configuration requiresDocumentConstraints: aBoolean! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 11/16/2013 07:33'!
resolvesExternalEntities
	^ self configuration resolvesExternalEntities! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 1/24/2014 23:52'!
resolvesExternalEntities: aBoolean
	"When true, external DTDs and parsed entities will be
	fetched (not default)."

	self configuration resolvesExternalEntities: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
skippedEntity: anEntityName
	"This call corresonds to the Java SAX call
	skippedEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:53'!
startCData
	"This call corresponds to the Java SAX ext call
	startCData()."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/27/2014 14:38'!
startCDataSection
	isInCDataSection := true! !

!SAXHandler methodsFor: 'declaration' stamp: 'JAAyer 11/10/2010 19:21'!
startDTD: aRootElementName publicID: aPublicId systemID: aSystemId
	"This call corresponds to the Java SAX ext call
	startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:45'!
startDocument
	"This call corresonds to the Java SAX call
	startDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:13'!
startElement: aQualifiedName attributes: aDictionary
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:47'!
startElement: aQualifiedName prefix: aPrefix uri: aUri attributes: aDictionary
	self startElement: aQualifiedName attributes: aDictionary! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:14'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	"This call corresonds to the Java SAX call
	startElement(java.lang.String namespaceURI, java.lang.String localName,
		java.lang.String qName, Attributes atts)."

	self
		startElement: aQualifiedName
		prefix: aPrefix
		uri: aUri
		attributes: aDictionary! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:22'!
startEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	startEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'private' stamp: 'monty 10/27/2014 14:45'!
startInternalSubset
	isInInternalSubset := true! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
startPrefixMapping: aPrefix uri: aUri
	"This call corresonds to the Java SAX call
	startPrefixMapping(java.lang.String prefix, java.lang.String uri)."! !

!SAXHandler methodsFor: 'configuration' stamp: 'JAAyer 11/23/2010 14:20'!
usesNamespaces
	^ self configuration usesNamespaces! !

!SAXHandler methodsFor: 'configuration' stamp: 'monty 12/29/2013 03:03'!
usesNamespaces: aBoolean
	"Namespace checking when true (default)"

	self configuration usesNamespaces: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'monty 11/14/2013 00:14'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
! !

SAXHandler subclass: #XMLDOMParser
	instanceVariableNames: 'document nodeStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMParser commentStamp: 'monty 11/6/2014 12:11' prior: 0!
This class is an XML parser that produces a tree of nodes representing the parsed document's structure. The root of this tree will typically be a document node, and it will have a single child element node as its root. The root element can have any number of additional elements as its children, as well as non-element child nodes. Those elements can themselves have an arbitrary number of children, and so on. To see what type of node objects a node tree created by this parser will contain and for how to manipulate them browse the XML-Parser-DOM category.

By default, CDATA sections are treated as regular string nodes and comments are ignored entirely. To suppress this behavior, enable #preservesCDataNodes: or #preservesCommentNodes: before parsing.

Lastly. you can control what node classes the DOM parser uses to create node objects by injecting a custom node factory with #nodeFactory: prior to parsing. See XMLNodeFactory for more information.!

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/31/2013 00:02'!
attributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self topNode addNode:
		(self nodeFactory attributeDeclarationClass
			element: anElement
			attribute: aName
			type: aType
			defaultPragma: aDefaultPragma
			defaultValue: aDefaultValue)! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 11/1/2014 23:03'!
characters: aString
	(self isInCDataSection and: [self preservesCDataNodes])
		ifTrue: [self topNode addNode: (self nodeFactory newCData: aString)]
		ifFalse: [self topNode addContent: (self nodeFactory newString: aString)]
! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'monty 11/1/2014 23:02'!
comment: aComment
	self preservesCommentNodes
		ifTrue: [self topNode addNode: (self nodeFactory newComment: aComment)]! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/23/2010 14:37'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 12/27/2013 03:43'!
currentScope: aNamespaceScope
	self topNode setNamespaceScope: aNamespaceScope! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 3/9/2010 01:38'!
document
	^ document! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'monty 11/6/2014 14:07'!
document: aDocument
	document := aDocument! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:14'!
elementDeclaration: aName contentModel: aContentModel
	self topNode addNode:
		(self nodeFactory elementDeclarationClass
			element: aName
			contentModel: aContentModel)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 11/12/2013 23:00'!
endDTD
	self popNode! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAA 3/21/2011 19:45'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	self popNode! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:01'!
generalEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId
			ndata: aNotation)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
generalEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory generalEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 12/9/2013 01:37'!
nodeListClass
	^ self configuration nodeListClass! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'monty 12/9/2013 01:37'!
nodeListClass: aClass
	self configuration nodeListClass: aClass! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 10/27/2014 09:12'!
nodeStack
	^ nodeStack! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:03'!
notationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self topNode addNode:
		(self nodeFactory notationDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId)! !

!XMLDOMParser methodsFor: 'initialization' stamp: 'monty 11/18/2014 17:17'!
on: aStringOrStream documentURI: aUri
	super
		on: aStringOrStream
		documentURI: aUri.

	nodeStack := OrderedCollection new: 10.! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
parameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			publicID: aPublicId
			systemID: aSystemId)! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 12/21/2013 03:02'!
parameterEntityDeclaration: aName replacement: aReplacement
	self topNode addNode:
		(self nodeFactory parameterEntityDeclarationClass
			name: aName
			replacement: aReplacement)! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 11/15/2014 16:44'!
parser
	| parser |

	parser := super parser.
	self topNode
		ifNotNil: [:root |
			parser initialContext: root innerXMLContext.

			self nodeStack do: [:each |
				each isElement
					ifTrue: [
						self
							startInnerXMLElement: each
							forParser: parser]]].

	^ parser.! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'monty 12/19/2013 00:38'!
parsingResult
	^ self document! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:31'!
popNode
	^ self nodeStack removeLast! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCDataNodes
	^ self configuration preservesCDataNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:17'!
preservesCDataNodes: aBoolean
	self configuration preservesCDataNodes: aBoolean! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes
	^ self configuration preservesCommentNodes! !

!XMLDOMParser methodsFor: 'configuring' stamp: 'JAAyer 11/23/2010 14:18'!
preservesCommentNodes: aBoolean
	self configuration preservesCommentNodes: aBoolean! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 11/1/2014 23:04'!
processingInstruction: aTarget data: aDataString
	self topNode addNode:
		(self nodeFactory newPI
			target: aTarget;
			data: aDataString)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 2/4/2014 01:33'!
pushElement: anElement
	self topNode addNode: anElement.
	^ self pushNode: anElement.! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 11/14/2014 21:11'!
pushInnerXMLRootNode: aRootNode
	self
		configuration: aRootNode configuration;
		document: aRootNode document;
		pushNode: aRootNode.! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/28/2010 13:25'!
pushNode: aNode
	^ self nodeStack addLast: aNode
! !

!XMLDOMParser methodsFor: 'declaration' stamp: 'monty 11/12/2013 23:02'!
startDTD: aRootElementName publicID: aPublicId systemID: aSystemId
	| doctypeDeclaration |

	doctypeDeclaration := 
		self nodeFactory doctypeDeclarationClass
			root: aRootElementName
			publicID: aPublicId
			systemID: aSystemId.
	self pushNode: doctypeDeclaration.
	self document doctypeDeclaration: doctypeDeclaration.! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 11/6/2014 14:07'!
startDocument
	self topNode ifNotNil: [^ self].

	self
		document: self nodeFactory newDocument;
		pushNode: self document.

	self document configuration: self configuration.! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 11/16/2014 17:38'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	self pushElement:
		((self nodeFactory
			classForElement: aQualifiedName
			prefix: aPrefix
			uri: aUri
			localName: aLocalName) new
				setQualifiedName: aQualifiedName
				localName: aLocalName
				attributes: aDictionary)! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 11/15/2014 04:07'!
startInnerXMLElement: anElement forParser: aParser
	aParser driver
		setRootElement: anElement name
		currentScope: anElement namespaceScope! !

!XMLDOMParser methodsFor: 'private' stamp: 'monty 10/7/2014 03:06'!
topNode
	^ self nodeStack lastOrNil! !

!XMLDOMParser methodsFor: 'content' stamp: 'monty 11/19/2013 05:37'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self document ifNotNil: [
		self document
			version: aVersionFloat;
			encoding: anEncoding;
			isStandalone: aBoolean]! !

Object subclass: #XMLAbstractImplementation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Util'!

XMLAbstractImplementation class
	instanceVariableNames: 'implementation'!

!XMLAbstractImplementation commentStamp: 'monty 11/17/2014 03:42' prior: 0!
An abstract class for dynamically choosing supported concrete implementations.!

XMLAbstractImplementation class
	instanceVariableNames: 'implementation'!

!XMLAbstractImplementation class methodsFor: 'private' stamp: 'monty 11/12/2014 18:22'!
errorUnsupported
	self subclassResponsibility! !

!XMLAbstractImplementation class methodsFor: 'private' stamp: 'monty 11/18/2014 07:03'!
implementation
	^ implementation
		ifNil: [
			self isSupported
				ifTrue: [implementation := self]
				ifFalse: [implementation := self selectImplementation]]! !

!XMLAbstractImplementation class methodsFor: 'testing' stamp: 'monty 11/12/2014 18:21'!
isSupported
	^ false! !

!XMLAbstractImplementation class methodsFor: 'instance creation' stamp: 'monty 11/12/2014 18:24'!
new
	^ self implementation basicNew initialize! !

!XMLAbstractImplementation class methodsFor: 'private' stamp: 'monty 11/12/2014 18:22'!
preferredImplementation
	self subclassResponsibility! !

!XMLAbstractImplementation class methodsFor: 'private' stamp: 'monty 11/12/2014 18:25'!
selectImplementation
	self preferredImplementation isSupported
		ifTrue: [^ self preferredImplementation].

	self allSubclassesDo: [:each |
		each isSupported
			ifTrue: [^ each]].

	self errorUnsupported.! !

XMLAbstractImplementation subclass: #XMLFileHandle
	instanceVariableNames: 'path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileHandle commentStamp: 'monty 11/20/2014 02:27' prior: 0!
This is an abstract class for file handles to read/write/delete files. Subclasses need to implement the abstract methods.!

XMLFileHandle subclass: #XMLFileDirectoryFileHandle
	instanceVariableNames: 'directoryEntry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileDirectoryFileHandle commentStamp: 'monty 10/24/2014 16:06' prior: 0!
This class uses FileDirectory/DirectoryEntry to implement file IO.!

!XMLFileDirectoryFileHandle class methodsFor: 'private' stamp: 'monty 10/22/2014 04:43'!
currentWorkingDirectoryPathString
	^ (Smalltalk at: #FileDirectory) default pathName! !

!XMLFileDirectoryFileHandle class methodsFor: 'testing' stamp: 'monty 10/25/2014 20:40'!
isSupported
	^ Smalltalk includesKey: #FileDirectory! !

!XMLFileDirectoryFileHandle methodsFor: 'private' stamp: 'monty 10/24/2014 15:28'!
deleteFileOrDo: aBlock
	[(self directoryEntry ifNil: [^ aBlock value]) delete]
		on: Error
		do: aBlock! !

!XMLFileDirectoryFileHandle methodsFor: 'private' stamp: 'monty 10/23/2014 11:01'!
directoryEntry
	^ directoryEntry
		ifNil: [directoryEntry := (Smalltalk at: #FileDirectory) directoryEntryFor: path]! !

!XMLFileDirectoryFileHandle methodsFor: 'accessing' stamp: 'monty 10/24/2014 15:29'!
fileSize
	^ (self directoryEntry ifNil: [^ 0]) size! !

!XMLFileDirectoryFileHandle methodsFor: 'testing' stamp: 'monty 10/22/2014 05:27'!
isFile
	^ self directoryEntry notNil
		and: [self directoryEntry isDirectory not]! !

!XMLFileHandle class methodsFor: 'accessing' stamp: 'monty 1/23/2014 01:34'!
currentWorkingDirectory
	^ self implementation currentWorkingDirectoryPathString! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 1/22/2014 18:50'!
currentWorkingDirectoryPathString
	self subclassResponsibility! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 10/31/2014 20:54'!
errorUnsupported
	XMLFileException signal: 'File IO unsupported'.! !

!XMLFileHandle class methodsFor: 'instance creation' stamp: 'monty 10/25/2014 17:49'!
path: aSystemPath
	^ self new path: aSystemPath! !

!XMLFileHandle class methodsFor: 'private' stamp: 'monty 10/25/2014 18:56'!
preferredImplementation
	^ XMLFileSystemFileHandle! !

!XMLFileHandle methodsFor: 'deleting' stamp: 'monty 11/17/2014 23:00'!
delete
	self deleteFileOrDo: [:error | XMLFileException signalMessageFrom: error]! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 1/21/2014 22:32'!
deleteFileOrDo: aBlock
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 11/6/2014 12:16'!
errorFileSize: aFileSize exceedsMaxSize: aMaxSize
	XMLLimitException
		formatSignal: 'File of size {1} exceeds max size limit of {2}: {3}'
		with: aFileSize
		with: aMaxSize
		with: path! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 10/31/2014 20:49'!
errorNotAFile
	XMLFileException
		formatSignal: 'Not a file: {1}'
		with: path! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/22/2014 19:07'!
fileSize
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'requesting' stamp: 'monty 1/13/2014 18:14'!
get
	^ self readStream! !

!XMLFileHandle methodsFor: 'requesting' stamp: 'monty 1/13/2014 18:14'!
getUpToLimit: aMaxSize
	^ self readStreamUpToLimit: aMaxSize! !

!XMLFileHandle methodsFor: 'testing' stamp: 'monty 1/9/2014 01:58'!
isFile
	self subclassResponsibility! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/9/2014 04:34'!
path
	^ path! !

!XMLFileHandle methodsFor: 'accessing' stamp: 'monty 1/9/2014 04:23'!
path: aPath
	path := aPath! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 1/13/2014 18:13'!
readStream
	^ self readStreamUpToLimit: nil! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 11/17/2014 00:40'!
readStreamOnFileOrDo: aBlock
	^ [XMLFileReadStreamFactory on: path]
		on: Error
		do: aBlock! !

!XMLFileHandle methodsFor: 'reading' stamp: 'monty 11/17/2014 23:00'!
readStreamUpToLimit: aMaxSize
	| fileSize |

	self isFile
		ifFalse: [self errorNotAFile].
	
	(fileSize := self fileSize) > 0
		ifFalse: [ ^ '' readStream].
	(aMaxSize notNil and: [fileSize > aMaxSize])
		ifTrue: [self errorFileSize: fileSize exceedsMaxSize: aMaxSize].

	^ self readStreamOnFileOrDo: [:error | XMLFileException signalMessageFrom: error]! !

!XMLFileHandle methodsFor: 'writing' stamp: 'monty 11/17/2014 23:00'!
writeStream
	^ self writeStreamOnFileOrDo: [:error | XMLFileException signalMessageFrom: error]! !

!XMLFileHandle methodsFor: 'private' stamp: 'monty 11/17/2014 00:49'!
writeStreamOnFileOrDo: aBlock
	^ [XMLFileWriteStreamFactory on: path]
		on: Error
		do: aBlock! !

XMLFileHandle subclass: #XMLFileSystemFileHandle
	instanceVariableNames: 'fileReference'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileSystemFileHandle commentStamp: 'monty 1/9/2014 23:29' prior: 0!
This class uses the Pharo FileSystem API to implement file IO.!

!XMLFileSystemFileHandle class methodsFor: 'private' stamp: 'monty 10/22/2014 08:30'!
currentWorkingDirectoryPathString
	^ String streamContents: [:stream |
		(Smalltalk at: #FileSystem) workingDirectory
			path segments do: [:each |
				stream
					nextPut: $/;
					nextPutAll: each]].! !

!XMLFileSystemFileHandle class methodsFor: 'testing' stamp: 'monty 10/25/2014 20:40'!
isSupported
	^ Smalltalk includesKey: #FileSystem! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/22/2014 19:10'!
deleteFileOrDo: aBlock
	[self fileReference delete]
		on: self fileSystemError 
		do: aBlock! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/13/2014 04:37'!
fileReference
	^ fileReference ifNil: [fileReference := path asFileReference]! !

!XMLFileSystemFileHandle methodsFor: 'accessing' stamp: 'monty 1/22/2014 19:07'!
fileSize
	^ self fileReference size! !

!XMLFileSystemFileHandle methodsFor: 'private' stamp: 'monty 1/22/2014 19:09'!
fileSystemError
	^ Smalltalk at: #FileSystemError! !

!XMLFileSystemFileHandle methodsFor: 'testing' stamp: 'monty 1/13/2014 04:37'!
isFile
	^ self fileReference isFile! !

XMLAbstractImplementation subclass: #XMLFileReadStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileReadStreamFactory commentStamp: 'monty 11/20/2014 02:27' prior: 0!
An abstract factory for file read streams used by XMLFileHandle.!

!XMLFileReadStreamFactory class methodsFor: 'private' stamp: 'monty 11/17/2014 00:44'!
errorUnsupported
	XMLFileException signal: 'File reading unsupported'! !

!XMLFileReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:38'!
fileReadStreamOn: aPath
	self subclassResponsibility! !

!XMLFileReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:38'!
on: aPath
	^ self implementation fileReadStreamOn: aPath! !

!XMLFileReadStreamFactory class methodsFor: 'private' stamp: 'monty 11/20/2014 02:32'!
preferredImplementation
	^ XMLStandardFileStreamReadStreamFactory! !

XMLFileReadStreamFactory subclass: #XMLStandardFileStreamReadStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLStandardFileStreamReadStreamFactory commentStamp: 'monty 11/20/2014 02:28' prior: 0!
A StandardFileStream read stream factory. This stream is preferred for Pharo/Squeak because it does no encoding conversion so XMLParser can do its own.!

!XMLStandardFileStreamReadStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 03:19'!
fileReadStreamOn: aPath
	^ (Smalltalk at: #StandardFileStream) readOnlyFileNamed: aPath! !

!XMLStandardFileStreamReadStreamFactory class methodsFor: 'testing' stamp: 'monty 11/17/2014 00:36'!
isSupported
	^ (Smalltalk includesKey: #StandardFileStream)! !

XMLAbstractImplementation subclass: #XMLFileWriteStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileWriteStreamFactory commentStamp: 'monty 11/20/2014 02:28' prior: 0!
An abstract factory for file write streams used by XMLFileHandle.!

XMLFileWriteStreamFactory subclass: #XMLFileStreamWriteStreamFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-File'!

!XMLFileStreamWriteStreamFactory commentStamp: 'monty 11/20/2014 02:28' prior: 0!
A FileStream write stream factory, default for Pharo/Squeak.!

!XMLFileStreamWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:46'!
fileWriteStreamOn: aPath
	^ (Smalltalk at: #FileStream) fileNamed: aPath! !

!XMLFileStreamWriteStreamFactory class methodsFor: 'testing' stamp: 'monty 11/17/2014 00:46'!
isSupported
	^ (Smalltalk includesKey: #FileStream)! !

!XMLFileWriteStreamFactory class methodsFor: 'private' stamp: 'monty 11/17/2014 00:44'!
errorUnsupported
	XMLFileException signal: 'File writing unsupported'! !

!XMLFileWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:43'!
fileWriteStreamOn: aPath
	self subclassResponsiblity! !

!XMLFileWriteStreamFactory class methodsFor: 'instance creation' stamp: 'monty 11/17/2014 00:43'!
on: aPath
	^ self implementation fileWriteStreamOn: aPath! !

!XMLFileWriteStreamFactory class methodsFor: 'private' stamp: 'monty 11/17/2014 00:44'!
preferredImplementation
	^ XMLFileStreamWriteStreamFactory! !

XMLAbstractImplementation subclass: #XMLHTTPRequest
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLHTTPRequest commentStamp: 'monty 11/20/2014 02:29' prior: 0!
A base class for HTTP requests. Subclasses need to implement the abstract requesting method to return a ReadStream on the respone or raise an exception if it is too large or an HTTP error happens.!

!XMLHTTPRequest class methodsFor: 'private' stamp: 'monty 10/31/2014 20:54'!
errorUnsupported
	XMLHTTPException signal: 'HTTP unsupported'.! !

!XMLHTTPRequest class methodsFor: 'private' stamp: 'monty 10/25/2014 18:56'!
preferredImplementation
	^ XMLZincHTTPRequest! !

!XMLHTTPRequest class methodsFor: 'instance creation' stamp: 'monty 10/25/2014 18:04'!
url: aUrl
	^ self new url: aUrl! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 10/26/2014 09:14'!
defaultTimeout
	^ 30! !

!XMLHTTPRequest methodsFor: 'requesting' stamp: 'monty 1/9/2014 05:08'!
get
	^ self getUpToLimit: nil! !

!XMLHTTPRequest methodsFor: 'private' stamp: 'monty 1/13/2014 19:42'!
getResponseUpToLimit: aMaxSize withTimeout: aTimeout onError: aBlock
	self subclassResponsibility! !

!XMLHTTPRequest methodsFor: 'requesting' stamp: 'monty 11/17/2014 23:00'!
getUpToLimit: aMaxSize
	^ self
		getResponseUpToLimit: aMaxSize
		withTimeout: self defaultTimeout
		onError: [:error |
			(error isKindOf: XMLLimitException)
				ifTrue: [
					error
						messageText: 'HTTP reponse limit exceeded';
						pass]
				ifFalse: [XMLHTTPException signalMessageFrom: error]]! !

!XMLHTTPRequest methodsFor: 'accessing' stamp: 'monty 1/10/2014 00:10'!
url
	^ url! !

!XMLHTTPRequest methodsFor: 'accessing' stamp: 'monty 1/10/2014 00:10'!
url: aUrl
	url := aUrl! !

XMLHTTPRequest subclass: #XMLZincHTTPRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-HTTP'!

!XMLZincHTTPRequest commentStamp: '<historical>' prior: 0!
This client uses Zinc-HTTP, but does not refer to Zinc classes directly to avoid compile errors on platforms without Zinc.!

!XMLZincHTTPRequest class methodsFor: 'testing' stamp: 'monty 10/25/2014 17:43'!
isSupported
	^ Smalltalk includesKey: #ZnClient! !

!XMLZincHTTPRequest methodsFor: 'private' stamp: 'monty 11/17/2014 23:00'!
getResponseUpToLimit: aMaxSize withTimeout: aTimeout onError: aBlock
	^ [(Smalltalk at: #ZnMaximumEntitySize)
		value: aMaxSize
		during: [
			((Smalltalk at: #ZnClient) new
				timeout: aTimeout;
				ifFail: [:error |
					(error isKindOf: (Smalltalk at: #ZnEntityTooLarge))
						ifTrue: [XMLLimitException signalMessageFrom: error]
						ifFalse: [error pass]];
				url: url;
				get) readStream]]
		on: Error
		do: aBlock! !

Object subclass: #XMLAttributeDefaultValidator
	instanceVariableNames: 'defaultValue attribute'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeDefaultValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute default value validators that implement the constraints (like #REQUIRED) from the default declaration.!

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 1/21/2014 05:35'!
attribute: anAttribute
	^ self attribute: anAttribute defaultValue: ''! !

!XMLAttributeDefaultValidator class methodsFor: 'instance creation' stamp: 'monty 12/10/2013 06:16'!
attribute: anAttribute defaultValue: aDefaultValue
	^ self new setAttribute: anAttribute defaultValue: aDefaultValue! !

!XMLAttributeDefaultValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:48'!
attribute
	^ attribute! !

!XMLAttributeDefaultValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:48'!
defaultValue
	^ defaultValue! !

!XMLAttributeDefaultValidator methodsFor: 'initialization' stamp: 'monty 12/10/2013 06:17'!
setAttribute: anAttribute defaultValue: aDefaultValue
	attribute := anAttribute.
	defaultValue := aDefaultValue.! !

!XMLAttributeDefaultValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:02'!
validateAttributeDefaultIn: anAttributeDictionary
	self subclassResponsibility! !

XMLAttributeDefaultValidator subclass: #XMLDefaultValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDefaultValueAttributeValidator commentStamp: 'monty 2/2/2014 04:47' prior: 0!
This class will put a default value for an attribute if it has none.!

!XMLDefaultValueAttributeValidator methodsFor: 'validation' stamp: 'monty 10/30/2014 12:48'!
validateAttributeDefaultIn: anAttributeDictionary
	((anAttributeDictionary at: self attribute ifAbsent: ['']) isEmpty 
		and: [self defaultValue notEmpty])
		ifTrue: [anAttributeDictionary at: self attribute put: self defaultValue]! !

XMLDefaultValueAttributeValidator subclass: #XMLFixedValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLFixedValueAttributeValidator commentStamp: 'monty 2/2/2014 08:40' prior: 0!
This class fixes the value of an attribute to a default.!

!XMLFixedValueAttributeValidator methodsFor: 'private' stamp: 'monty 11/4/2014 15:09'!
errorNonDefaultValue
	XMLValidationException
		formatSignal: '#FIXED attribute "{1}" can only have "{2}" as value'
		with: self attribute
		with: self defaultValue! !

!XMLFixedValueAttributeValidator methodsFor: 'validation' stamp: 'monty 11/4/2014 15:09'!
validateAttributeDefaultIn: anAttributeDictionary
	(anAttributeDictionary
		at: self attribute
		ifAbsentPut: [self defaultValue]) = self defaultValue
		ifFalse: [self errorNonDefaultValue]! !

XMLAttributeDefaultValidator subclass: #XMLImpliedValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLImpliedValueAttributeValidator commentStamp: 'monty 2/2/2014 06:07' prior: 0!
Attributes declared #IMPLIED are optional.
!

!XMLImpliedValueAttributeValidator methodsFor: 'validation' stamp: 'monty 12/27/2013 15:36'!
validateAttributeDefaultIn: anAttributeDictionary! !

XMLAttributeDefaultValidator subclass: #XMLRequiredValueAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLRequiredValueAttributeValidator commentStamp: 'monty 2/2/2014 16:37' prior: 0!
Attributes declared #REQUIRED require values.!

!XMLRequiredValueAttributeValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:11'!
errorRequiredAttributeMissing
	XMLValidationException
		formatSignal: 'Required attribute "{1}" is missing'
		with: self attribute! !

!XMLRequiredValueAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:10'!
validateAttributeDefaultIn: anAttributeDictionary
	(anAttributeDictionary includesKey: self attribute)
		ifFalse: [self errorRequiredAttributeMissing]! !

Object subclass: #XMLAttributeValidator
	instanceVariableNames: 'element attribute defaultValueValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAttributeValidator commentStamp: '<historical>' prior: 0!
This is a base class for attribute validators.!

!XMLAttributeValidator class methodsFor: 'instance creation' stamp: 'monty 12/10/2013 06:15'!
element: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	^ self new
		setElement: anElement
		attribute: anAttribute
		defaultValueValidator: aDefaultValueValidator! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
attribute
	^ attribute! !

!XMLAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/10/2013 06:17'!
element
	^ element! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:12'!
errorInvalidID: anId
	XMLValidationException
		formatSignal: 'Invalid ID "{1}" in attribute "{2}"'
		with: anId
		with: self attribute! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:11'!
errorInvalidName: aName
	XMLValidationException
		formatSignal: 'Invalid name "{1}" in attribute "{2}"'
		with: aName
		with: self attribute! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 11/2/2014 09:13'!
furtherNormalizeAttributeValueIn: anAttributeDictionary
	| attributeValue writeStream isInWhitespace |

	attributeValue := anAttributeDictionary at: self attribute ifAbsent: [^ self].
	writeStream := WriteStream on: (String new: attributeValue size).
	isInWhitespace := false.
	attributeValue do: [:each |
		isInWhitespace
			ifTrue: [
				each isSeparator
					ifFalse: [
						writeStream position > 0
							ifTrue: [writeStream space].
						isInWhitespace := false.
						writeStream nextPut: each]]
			ifFalse: [
				each isSeparator
					ifTrue: [isInWhitespace := true]
					ifFalse: [writeStream nextPut: each]]].
	anAttributeDictionary
		at: self attribute
		put: writeStream contents.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDRefs
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDs
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 1/4/2014 05:02'!
hasNotationValues
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/10/2013 06:06'!
isIDValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/9/2013 02:24'!
isValueListValidator
	^ false! !

!XMLAttributeValidator methodsFor: 'private' stamp: 'monty 10/30/2014 12:51'!
listValuesIn: aSpaceSeparatedList do: aBlock
	Character space
		xmlSplit: aSpaceSeparatedList
		do: aBlock! !

!XMLAttributeValidator methodsFor: 'initialization' stamp: 'monty 12/10/2013 06:15'!
setElement: anElement attribute: anAttribute defaultValueValidator: aDefaultValueValidator
	element := anElement.
	attribute := anAttribute.
	defaultValueValidator := aDefaultValueValidator.! !

!XMLAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ true! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 11/19/2014 19:26'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary
		at: self attribute
		put: self! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 19:58'!
validateAttributeDefaultIn: anAttributeDictionary
	defaultValueValidator validateAttributeDefaultIn: anAttributeDictionary.

	self shouldFurtherNormalizeAttributeValue
		ifTrue: [self furtherNormalizeAttributeValueIn: anAttributeDictionary].! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:03'!
validateAttributeValue: aValue
	self subclassResponsibility! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 11/19/2014 18:35'!
validateAttributes: anAttributeDictionary
	self
		validateAttributeDefaultIn: anAttributeDictionary;
		validateAttributeValue:
			(anAttributeDictionary
				at: self attribute
				ifAbsent: [^ self])! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:12'!
validateID: anId
	(XMLWellFormedParserTokenizer isName: anId)
		ifFalse: [self errorInvalidID: anId]! !

!XMLAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:11'!
validateName: aName
	(XMLWellFormedParserTokenizer isNmtoken: aName)
		ifFalse: [self errorInvalidName: aName]! !

XMLAttributeValidator subclass: #XMLCDataAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLCDataAttributeValidator commentStamp: 'monty 2/2/2014 08:12' prior: 0!
CDATA attribute values are left alone (whitespace is not normalized further).!

!XMLCDataAttributeValidator methodsFor: 'testing' stamp: 'monty 12/11/2013 02:14'!
shouldFurtherNormalizeAttributeValue
	^ false! !

!XMLCDataAttributeValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 20:04'!
validateAttributeValue: aValue
	! !

XMLAttributeValidator subclass: #XMLEntityAttributeValidator
	instanceVariableNames: 'unparsedEntities'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntityAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes that contain unparsed entity names (that must be declared).!

XMLEntityAttributeValidator subclass: #XMLEntitiesAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEntitiesAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes with lists of unparsed entity name values separated by spaces.!

!XMLEntitiesAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 19:50'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each | self validateEntityReference: each]! !

!XMLEntityAttributeValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:16'!
errorInvalidUnparsedEntityReference: aName
	XMLValidationException
		formatSignal: 'Unrecognized unparsed entity "{1}" in "{2}" attribute'
		with: aName
		with: self attribute! !

!XMLEntityAttributeValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 22:00'!
isEntityValidator
	^ true! !

!XMLEntityAttributeValidator methodsFor: 'accessing' stamp: 'monty 10/27/2014 15:06'!
unparsedEntities
	^ unparsedEntities ifNil: [unparsedEntities := Dictionary new]! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 1/4/2014 22:50'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	unparsedEntities := aDeclarationDelegate unparsedEntities.
	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate.! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	self validateEntityReference: aValue! !

!XMLEntityAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:15'!
validateEntityReference: aName
	self validateName: aName.
	(self unparsedEntities includesKey: aName)
		ifFalse: [self errorInvalidUnparsedEntityReference: aName]! !

XMLAttributeValidator subclass: #XMLIDAttributeValidator
	instanceVariableNames: 'ids'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDAttributeValidator commentStamp: 'monty 2/2/2014 05:57' prior: 0!
This class validates ID attributes that must be unique in each element of the type.!

!XMLIDAttributeValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:26'!
errorDuplicateID: anId
	XMLValidationException
		formatSignal: 'ID "{1}" used more than once for attribute "{2}"'
		with: anId
		with: self attribute! !

!XMLIDAttributeValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:26'!
errorElementWithMultipleIDAttributes
	XMLValidationException
		formatSignal: 'Element <{1}> has multiple ID attributes'
		with: self element! !

!XMLIDAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:57'!
hasIDs
	^ self ids notEmpty! !

!XMLIDAttributeValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 08:18'!
ids
	^ ids ifNil: [ids := Set new]
	! !

!XMLIDAttributeValidator methodsFor: 'testing' stamp: 'monty 12/10/2013 06:07'!
isIDValidator
	^ true! !

!XMLIDAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:25'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary valuesDo: [:each |
		each isIDValidator
			ifTrue: [self errorElementWithMultipleIDAttributes]].

	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate! !

!XMLIDAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:26'!
validateAttributeValue: aValue
	self validateID: aValue.
	(self ids includes: aValue)
		ifTrue: [self errorDuplicateID: aValue].

	self ids add: aValue.! !

XMLIDAttributeValidator subclass: #XMLReservedIDAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLReservedIDAttributeValidator commentStamp: 'monty 11/19/2014 18:02' prior: 0!
A validator for xml:id attributes.!

!XMLReservedIDAttributeValidator methodsFor: 'accessing' stamp: 'monty 11/19/2014 17:51'!
attribute
	^ 'xml:id'! !

!XMLReservedIDAttributeValidator methodsFor: 'validation' stamp: 'monty 11/19/2014 18:35'!
validateAttributes: anAttributeDictionary
	self validateAttributeValue:
		(anAttributeDictionary
			at: self attribute
			ifAbsent: [^ self])! !

!XMLReservedIDAttributeValidator methodsFor: 'validation' stamp: 'monty 11/19/2014 18:02'!
validateID: anId
	((XMLWellFormedParserTokenizer isName: anId)
		and: [(anId indexOf: $:) = 0])
		ifFalse: [self errorInvalidID: anId]! !

XMLAttributeValidator subclass: #XMLIDRefAttributeValidator
	instanceVariableNames: 'idRefs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefAttributeValidator commentStamp: 'monty 2/2/2014 16:44' prior: 0!
This class validates attributes that refer to an ID of another element with a ID attribute.!

!XMLIDRefAttributeValidator methodsFor: 'testing' stamp: 'monty 12/29/2013 02:58'!
hasIDRefs
	^ self idRefs notEmpty! !

!XMLIDRefAttributeValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:30'!
idRefs
	^ idRefs ifNil: [idRefs := Set new].! !

!XMLIDRefAttributeValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:30'!
idRefs: anObject
	^ idRefs := anObject! !

!XMLIDRefAttributeValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 02:58'!
validateAttributeValue: aValue
	self validateID: aValue.
	self idRefs add: aValue.! !

XMLIDRefAttributeValidator subclass: #XMLIDRefsAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLIDRefsAttributeValidator commentStamp: 'monty 2/2/2014 08:13' prior: 0!
This class validates attributes with lists of ID ref values separated by spaces.!

!XMLIDRefsAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 19:50'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each |
			self validateID: each.
			self idRefs add: each].! !

XMLAttributeValidator subclass: #XMLNameAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNameAttributeValidator commentStamp: 'monty 2/2/2014 06:06' prior: 0!
This class validates name attributes.!

!XMLNameAttributeValidator methodsFor: 'validation' stamp: 'monty 12/11/2013 02:12'!
validateAttributeValue: aValue
	self validateName: aValue! !

XMLNameAttributeValidator subclass: #XMLNamesAttributeValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNamesAttributeValidator commentStamp: 'monty 2/2/2014 06:07' prior: 0!
This class validates attributes with lists of name values separated by spaces.!

!XMLNamesAttributeValidator methodsFor: 'validation' stamp: 'monty 2/10/2014 19:50'!
validateAttributeValue: aValue
	self
		listValuesIn: aValue
		do: [:each | self validateName: each]! !

XMLAttributeValidator subclass: #XMLValueListAttributeValidator
	instanceVariableNames: 'allowedValues'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLValueListAttributeValidator commentStamp: 'monty 2/2/2014 08:13' prior: 0!
This class validates attributes that can have any value taken from a list like "(one|two|thee)".!

XMLValueListAttributeValidator subclass: #XMLNotationListAttributeValidator
	instanceVariableNames: 'notations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLNotationListAttributeValidator commentStamp: 'monty 2/2/2014 08:10' prior: 0!
This class validates attributes that can have NOTATION names from a list as values.!

!XMLNotationListAttributeValidator methodsFor: 'private ' stamp: 'monty 10/31/2014 17:33'!
errorMultipleNotationAttributes
	XMLValidationException
		formatSignal: 'Element <{1}> has multiple NOTATION list attributes'
		with: self element! !

!XMLNotationListAttributeValidator methodsFor: 'private ' stamp: 'monty 10/31/2014 17:29'!
errorUndeclaredNotation: aNotation
	XMLValidationException
		formatSignal: 'Undeclared NOTATION "{1}" in attribute "{2}"'
		with: aNotation
		with: self attribute! !

!XMLNotationListAttributeValidator methodsFor: 'testing' stamp: 'monty 1/4/2014 04:58'!
hasNotationValues
	^ true! !

!XMLNotationListAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/4/2014 05:00'!
notations
	^ notations ifNil: [notations := Set new]! !

!XMLNotationListAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:33'!
validateAndAddTo: anAttributeValidatorDictionary usingDeclarationsFrom: aDeclarationDelegate
	anAttributeValidatorDictionary valuesDo: [:each |
		each hasNotationValues
			ifTrue: [self errorMultipleNotationAttributes]].

	notations := aDeclarationDelegate notations.
	super
		validateAndAddTo: anAttributeValidatorDictionary
		usingDeclarationsFrom: aDeclarationDelegate.! !

!XMLNotationListAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:29'!
validateAttributeValue: aValue
	super validateAttributeValue: aValue.

	(self notations includes: aValue)
		ifFalse: [self errorUndeclaredNotation: aValue]. ! !

!XMLNotationListAttributeValidator methodsFor: 'private ' stamp: 'monty 1/25/2014 01:17'!
valueListParserClass
	^ DTDNotationListParser! !

!XMLValueListAttributeValidator methodsFor: 'accessing' stamp: 'monty 12/9/2013 02:17'!
allowedValues
	^ allowedValues ifNil: [allowedValues := Set new]! !

!XMLValueListAttributeValidator methodsFor: 'private ' stamp: 'monty 10/31/2014 17:30'!
errorInvalidValue: aValue
	XMLValidationException
		formatSignal: 'Invalid value "{1}" for attribute "{2}"'
		with: aValue
		with: self attribute! !

!XMLValueListAttributeValidator methodsFor: 'testing' stamp: 'monty 12/9/2013 02:24'!
isValueListValidator
	^ true! !

!XMLValueListAttributeValidator methodsFor: 'accessing' stamp: 'monty 1/25/2014 01:24'!
setAllowedValuesFromString: aString
	allowedValues := (self valueListParserClass on: aString) parse! !

!XMLValueListAttributeValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:30'!
validateAttributeValue: aValue
	(self allowedValues includes: aValue)
		ifFalse: [self errorInvalidValue: aValue]! !

!XMLValueListAttributeValidator methodsFor: 'private ' stamp: 'monty 1/25/2014 01:17'!
valueListParserClass
	^ DTDEnumerationListParser! !

Object subclass: #XMLConfiguration
	instanceVariableNames: 'isValidating usesNamespaces resolvesExternalEntities preservesUndeclaredEntityReferences externalEntityResolver parserHandlerClass parserLimits requiresDocumentConstraints'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLConfiguration commentStamp: '<historical>' prior: 0!
This class stores the shared settings and components for a parser.!

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/10/2014 08:08'!
externalEntityResolver
	^ externalEntityResolver ifNil: [externalEntityResolver := DTDCachingEntityResolver new]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/18/2013 22:18'!
externalEntityResolver: anExternalEntityResolver
	externalEntityResolver := anExternalEntityResolver! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:09'!
isValidating
	^ isValidating ifNil: [isValidating := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:42'!
isValidating: aBoolean
	(isValidating := aBoolean)
		ifTrue: [self preservesUndeclaredEntityReferences: false]
		ifFalse: [self requiresDocumentConstraints: false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass
	^ parserHandlerClass ifNil: [parserHandlerClass := SAXHandler]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/16/2013 03:10'!
parserHandlerClass: aClass
	parserHandlerClass := aClass! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/16/2014 21:27'!
parserLimits
	^ parserLimits ifNil: [parserLimits := XMLParserLimits new]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:16'!
preservesUndeclaredEntityReferences
	^ preservesUndeclaredEntityReferences ifNil: [preservesUndeclaredEntityReferences := false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 12/29/2013 02:42'!
preservesUndeclaredEntityReferences: aBoolean
	(preservesUndeclaredEntityReferences := aBoolean)
		ifTrue: [self isValidating: false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/25/2014 21:13'!
requiresDocumentConstraints
	^ requiresDocumentConstraints ifNil: [requiresDocumentConstraints := false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/25/2014 21:13'!
requiresDocumentConstraints: aBoolean
	(requiresDocumentConstraints := aBoolean)
		ifTrue: [self isValidating: true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 1/24/2014 23:52'!
resolvesExternalEntities
	^ resolvesExternalEntities ifNil: [resolvesExternalEntities := false]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'monty 11/13/2013 22:18'!
resolvesExternalEntities: aBoolean
	resolvesExternalEntities := aBoolean! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:07'!
usesNamespaces
	^ usesNamespaces ifNil: [usesNamespaces := true]! !

!XMLConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:06'!
usesNamespaces: aBoolean
	usesNamespaces := aBoolean! !

XMLConfiguration subclass: #XMLDOMConfiguration
	instanceVariableNames: 'preservesCDataNodes preservesCommentNodes nodeFactory nodeListClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMConfiguration commentStamp: '<historical>' prior: 0!
This class stores additional settings and components relevant to the DOM parser and DOM nodes.!

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory
	^ nodeFactory ifNil: [nodeFactory := XMLNodeFactory new]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:10'!
nodeFactory: aFactory
	nodeFactory := aFactory! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:22'!
nodeListClass
	^ nodeListClass ifNil: [nodeListClass := XMLNodeList]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 11/26/2013 22:22'!
nodeListClass: aClass
	nodeListClass := aClass! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'monty 10/28/2014 18:06'!
parserHandlerClass
	^ parserHandlerClass ifNil: [parserHandlerClass := XMLDOMParser]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes
	^ preservesCDataNodes ifNil: [preservesCDataNodes := false]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCDataNodes: aBoolean
	preservesCDataNodes := aBoolean! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes
	^ preservesCommentNodes ifNil: [preservesCommentNodes := false]! !

!XMLDOMConfiguration methodsFor: 'accessing' stamp: 'JAAyer 11/23/2010 14:08'!
preservesCommentNodes: aBoolean
	preservesCommentNodes := aBoolean! !

Object subclass: #XMLDFAState
	instanceVariableNames: 'nfaStates transitions dfaStateCache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This class is a DFA state made by grouping one or more NFA states.!

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:11'!
nfaStates: anNfaStateSet
	^ self
		nfaStates: anNfaStateSet
		dfaStateCache: nil! !

!XMLDFAState class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:09'!
nfaStates: anNfaStateSet dfaStateCache: aDfaStateCache
	^ self new
		setNFAStates: anNfaStateSet
		dfaStateCache: aDfaStateCache! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 12/28/2013 00:21'!
detectNextStateMatching: aMatcher
	^ self detectNextStateMatching: aMatcher ifNone: [nil]! !

!XMLDFAState methodsFor: 'enumerating' stamp: 'monty 12/28/2013 19:54'!
detectNextStateMatching: aMatcher ifNone: aNoneBlock
	^ (self transitions
		at: aMatcher
		ifAbsentPut: [self dfaStateMatching: aMatcher])
			ifNil: [aNoneBlock value]! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/29/2013 03:33'!
dfaStateMatching: aMatcher
	| matchingNfaStates |

	matchingNfaStates := XMLNFAStateSet new.
	nfaStates do: [:state |
		(state matches: aMatcher)
			ifTrue: [state nextState addTo: matchingNfaStates]].

	^ matchingNfaStates
		ifNotEmpty: [dfaStateCache dfaStateFor: matchingNfaStates]
		ifEmpty: [nil].! !

!XMLDFAState methodsFor: 'testing' stamp: 'monty 12/28/2013 18:54'!
isAccepting
	^ nfaStates isAccepting! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/29/2013 01:10'!
nfaStates
	^ nfaStates! !

!XMLDFAState methodsFor: 'initialization' stamp: 'monty 12/29/2013 03:25'!
setNFAStates: anNfaStateSet dfaStateCache: aDfaStateCache
	nfaStates := anNfaStateSet.
	dfaStateCache := (aDfaStateCache ifNil: [XMLDFAStateCache firstState: self]).! !

!XMLDFAState methodsFor: 'accessing' stamp: 'monty 12/28/2013 01:09'!
transitions
	^ transitions ifNil: [transitions := Dictionary new]! !

Object subclass: #XMLDFAStateCache
	instanceVariableNames: 'firstState cachedStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLDFAStateCache commentStamp: 'monty 12/29/2013 03:32' prior: 0!
This class uses a dictionary with NFAStateSet keys to cache DFA states.!

!XMLDFAStateCache class methodsFor: 'instance creation' stamp: 'monty 12/29/2013 01:02'!
firstState: aFirstState
	^ self new firstState: aFirstState! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 03:29'!
clearCache
	cachedStates
		removeAll;
		at: firstState nfaStates
		put: firstState! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 12/29/2013 06:33'!
dfaStateFor: aStateSet
	^ cachedStates
		at: aStateSet
		ifAbsent: [
			cachedStates size >= self maxCachedStates
				ifTrue: [self clearCache].

			cachedStates
				at: aStateSet
				put: (self newDFAStateFor: aStateSet)].! !

!XMLDFAStateCache methodsFor: 'accessing' stamp: 'monty 11/19/2014 19:28'!
firstState: aFirstState
	firstState := aFirstState.

	(cachedStates := Dictionary new: 5)
		at: firstState nfaStates
		put: firstState.! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 01:07'!
maxCachedStates
	^ 32! !

!XMLDFAStateCache methodsFor: 'private' stamp: 'monty 12/29/2013 03:29'!
newDFAStateFor: aStateSet
	^ XMLDFAState
		nfaStates: aStateSet
		dfaStateCache: self.! !

Object subclass: #XMLDecodingReadStreamAdapter
	instanceVariableNames: 'stream streamDecoder peekChar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLDecodingReadStreamAdapter commentStamp: 'monty 11/20/2014 03:57' prior: 0!
This is a stream adapter for basic read streams that support peek, next, and atEnd that can also decode them using stream decoder classes.!

!XMLDecodingReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 11/20/2014 00:18'!
on: aStream
	^ self
		on: aStream
		streamDecoder: XMLStreamDecoder defaultStreamDecoder new! !

!XMLDecodingReadStreamAdapter class methodsFor: 'instance creation' stamp: 'monty 11/13/2014 05:07'!
on: aStream streamDecoder: aDecoder
	^ self new
		setStream: aStream
		streamDecoder: aDecoder! !

!XMLDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 11/12/2014 23:06'!
atEnd
	^ peekChar == nil and: [stream atEnd]! !

!XMLDecodingReadStreamAdapter methodsFor: 'private' stamp: 'monty 11/11/2014 18:40'!
basicNext
	stream atEnd
		ifTrue: [^ nil]
		ifFalse: [^ stream next].! !

!XMLDecodingReadStreamAdapter methodsFor: 'file open/close' stamp: 'monty 11/13/2014 05:08'!
close
	(stream respondsTo: #close)
		ifTrue: [stream close]! !

!XMLDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 11/20/2014 01:55'!
isBinary
	^ false! !

!XMLDecodingReadStreamAdapter methodsFor: 'testing' stamp: 'monty 11/20/2014 01:55'!
isStream
	^ true! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/17/2014 21:14'!
next
	peekChar
		ifNil: [^ streamDecoder nextFromStream: self]
		ifNotNil: [:nextChar |
			peekChar := nil.
			^ nextChar]! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/17/2014 21:14'!
peek
	^ peekChar ifNil: [peekChar := streamDecoder nextFromStream: self]! !

!XMLDecodingReadStreamAdapter methodsFor: 'initialization' stamp: 'monty 11/13/2014 05:07'!
setStream: aStream streamDecoder: aDecoder
	stream := aStream.
	streamDecoder := aDecoder.! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/11/2014 15:48'!
streamDecoder
	^ streamDecoder! !

!XMLDecodingReadStreamAdapter methodsFor: 'accessing' stamp: 'monty 11/11/2014 15:49'!
streamDecoder: aStreamDecoder
	streamDecoder := aStreamDecoder! !

Object subclass: #XMLDocumentValidator
	instanceVariableNames: 'elementValidatorStack xmlIdValidator'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDocumentValidator commentStamp: '<historical>' prior: 0!
This is a base class for XML document validators.!

XMLDocumentValidator subclass: #DTDDocumentValidator
	instanceVariableNames: 'dtd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDDocumentValidator commentStamp: '<historical>' prior: 0!
This class validates documents using a DTD.!

!DTDDocumentValidator class methodsFor: 'instance creation' stamp: 'monty 11/18/2013 05:13'!
dtd: aDtd
	^ self new dtd: aDtd! !

!DTDDocumentValidator methodsFor: 'enumerating' stamp: 'monty 12/9/2013 02:29'!
allAttributeValidatorsDo: aBlock
	^ self dtd attributeValidators valuesDo: [:each | each valuesDo: aBlock]! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 19:43'!
attributeValidatorsFor: anElement
	^ self dtd attributeValidators
		at: anElement
		ifAbsent: [nil]! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 05:13'!
dtd
	^ dtd! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 05:13'!
dtd: aDtd
	dtd := aDtd! !

!DTDDocumentValidator methodsFor: 'accessing' stamp: 'monty 10/30/2014 19:43'!
elementValidatorFor: aName
	^ (self dtd elementValidators
		at: aName
		ifAbsent: [nil]) copy! !

!DTDDocumentValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:14'!
errorInvalidIDReference: anId
	XMLValidationException
		formatSignal: 'Invalid ID reference "{1}"'
		with: anId! !

!DTDDocumentValidator methodsFor: 'private' stamp: 'monty 10/31/2014 17:14'!
errorInvalidRoot: anElementName
	XMLValidationException
		formatSignal: 'Element <{1}> cannot be root'
		with: anElementName! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 10/30/2014 19:43'!
hasAttributeValidators
	^ self dtd hasAttributeValidators! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 10/30/2014 19:43'!
hasElementValidators
	^ self dtd hasElementValidators! !

!DTDDocumentValidator methodsFor: 'testing' stamp: 'monty 11/2/2014 22:18'!
hasValidators
	^ self dtd hasValidators! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 10/30/2014 19:33'!
unvalidatedAttributes: anAttributeCollection
	XMLValidationException
		formatSignal: 'Undeclared attribute "{1}"'
		with: anAttributeCollection anyOne! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 10/30/2014 19:33'!
unvalidatedElement: aName
	XMLValidationException
		formatSignal: 'Undeclared element <{1}>'
		with: aName! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 02:38'!
validateEndDocument
	self hasAttributeValidators
		ifTrue: [self validateIDsAndReferences]	! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:14'!
validateIDsAndReferences
	| ids references |

	ids := Set new.
	references := Set new.
	self allAttributeValidatorsDo: [:each |
		each hasIDs
			ifTrue: [ids addAll: each ids]
			ifFalse: [
				each hasIDRefs
					ifTrue: [references addAll: each idRefs]]].

	references do: [:each |
		(ids includes: each)
			ifFalse: [self errorInvalidIDReference: each]].! !

!DTDDocumentValidator methodsFor: 'validation' stamp: 'monty 10/31/2014 17:13'!
validateRootElement: anElementName
	(self dtd rootElement notEmpty
		and: [self dtd rootElement ~= anElementName])
		ifTrue: [self errorInvalidRoot: anElementName]! !

!XMLDocumentValidator methodsFor: 'enumerating' stamp: 'monty 11/19/2013 18:26'!
allAttributeValidatorsDo: aBlock
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:04'!
attributeValidatorsFor: anElement
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:47'!
currentElementValidator
	^ self elementValidatorStack last! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 20:07'!
defaultElementValidatorFor: aName
	^ XMLAnyElementValidator element: aName! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:57'!
elementValidatorFor: aName
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'accessing' stamp: 'monty 11/18/2013 03:46'!
elementValidatorStack
	^ elementValidatorStack ifNil: [elementValidatorStack := OrderedCollection new]! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorMissingContentFor: aName
	XMLValidationException
		formatSignal: 'Element <{1}> terminated prematurely'
		with: aName! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:31'!
hasAttributeValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 03:32'!
hasElementValidators
	self subclassResponsibility! !

!XMLDocumentValidator methodsFor: 'testing' stamp: 'monty 11/2/2014 22:18'!
hasValidators
	^ self hasElementValidators or: [self hasAttributeValidators]! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:29'!
unvalidatedAttributes: anAttributeCollection! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:29'!
unvalidatedElement: aName! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 10/30/2014 15:39'!
validateAttributes: anAttributeDictionary with: aValidatorDictionary
	| unvalidatedAttributes |

	unvalidatedAttributes := Set new: anAttributeDictionary size.
	anAttributeDictionary keysDo: [:each | unvalidatedAttributes add: each].
	aValidatorDictionary
		ifNotNil: [
			aValidatorDictionary keysAndValuesDo: [:attribute :validator |
				validator validateAttributes: anAttributeDictionary.
				unvalidatedAttributes
					remove: attribute
					ifAbsent: nil]].

	unvalidatedAttributes
		ifNotEmpty: [self unvalidatedAttributes: unvalidatedAttributes].! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 02:22'!
validateEndDocument
! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/7/2013 19:45'!
validateEndTag: aName
	self hasElementValidators
		ifTrue: [
			(self currentElementValidator element = aName)
				ifTrue: [
					self currentElementValidator atEnd
						ifFalse: [self errorMissingContentFor: aName]].
			self elementValidatorStack removeLast]! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 03:44'!
validatePCData: aString
	self hasElementValidators
		ifTrue: [self currentElementValidator validatePCData: aString]! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 11/2/2014 20:22'!
validateReservedAttributes: aDictionary for: aStartTag! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 20:08'!
validateRootElement: anElementName! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/21/2013 23:56'!
validateStartTag: aName
	self elementValidatorStack
		ifNotEmpty: [self currentElementValidator validateStartTag: aName].

	self elementValidatorStack addLast:
		((self elementValidatorFor: aName)
			ifNil: [
				self
					unvalidatedElement: aName;
					defaultElementValidatorFor: aName]).! !

!XMLDocumentValidator methodsFor: 'validation' stamp: 'monty 12/29/2013 05:05'!
validateStartTag: aName attributes: anAttributeDictionary
	(self hasElementValidators or: [self hasAttributeValidators])
		ifFalse: [^ self].

	self validateStartTag: aName.
	self
		validateAttributes: anAttributeDictionary
		with: (self attributeValidatorsFor: aName).! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 11/2/2014 21:06'!
validateXMLIDAttributeIn: anAttributeDictionary
	self xmlIDValidator validateAttributes: anAttributeDictionary! !

!XMLDocumentValidator methodsFor: 'private' stamp: 'monty 11/19/2014 18:01'!
xmlIDValidator
	^ xmlIdValidator ifNil: [xmlIdValidator := XMLReservedIDAttributeValidator new]! !

Object subclass: #XMLElementNester
	instanceVariableNames: 'openElements rootElement hasExternalRootElement scopedAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLElementNester commentStamp: '<historical>' prior: 0!
This is a base class for element nesters.!

!XMLElementNester methodsFor: 'private' stamp: 'monty 1/25/2014 21:08'!
copiedScopedAttributes
	| copiedAttributes |

	copiedAttributes :=
		self hasScopedAttributes
			ifTrue: [self scopedAttributes removeLast copy]
			ifFalse: [Dictionary new].
	self scopedAttributes addLast: copiedAttributes.

	^ copiedAttributes.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 10/7/2014 03:16'!
currentElement
	^ self openElements lastOrNil! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:09'!
currentScopedAttributes
	^ self scopedAttributes lastOrNil! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 10/22/2014 13:56'!
enclosedElementsAsString: anElement
	^ String streamContents: [:stream |
		self
			printFrom: (self openElements lastIndexOf: anElement) + 1
			on: stream]! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 1/20/2014 19:59'!
endElement: anElementName
	self openElements removeLast.
	self hasScopedAttributes
		ifTrue: [scopedAttributes removeLast].! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 2/2/2014 02:45'!
endNesting! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 11/3/2014 13:20'!
hasExternalRootElement
	^ hasExternalRootElement! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 12/6/2013 17:29'!
hasOpenElement: anElementName
	^ self openElements includes: anElementName! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 11/3/2014 13:18'!
hasOpenElements
	^ self openElements size > 0! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 1/20/2014 19:59'!
hasScopedAttributes
	^ scopedAttributes notNil and: [scopedAttributes notEmpty]! !

!XMLElementNester methodsFor: 'initialization' stamp: 'monty 11/3/2014 13:20'!
initialize
	super initialize.

	openElements := OrderedCollection new: 10.
	hasExternalRootElement := false.! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 2/10/2014 02:12'!
isCurrentElementRoot
	^ self openElements size = 1! !

!XMLElementNester methodsFor: 'testing' stamp: 'monty 11/3/2014 13:19'!
isInElement
	^ self openElements size > 0 or: [self hasExternalRootElement]! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:07'!
openElements
	^ openElements! !

!XMLElementNester methodsFor: 'enumerating' stamp: 'monty 2/15/2014 03:58'!
openElementsReverseDo: aBlock
	"reimplemented so it is always safe to remove at the end while scanning backwards"
	| i |

	i := self openElements size.
	[i > 0]
		whileTrue: [
			aBlock value: (openElements at: i).
			i := i - 1].! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 10/22/2014 13:52'!
printFrom: aStart on: aStream
	self
		printFrom: aStart
		to: aStart + self printLimit - 1
		on: aStream! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 10/23/2014 12:04'!
printFrom: aStart to: aStop on: aStream
	aStart to: aStop do: [:i |
		i > self openElements size
			ifTrue: [^ self].
		i > aStart
			ifTrue: [aStream nextPutAll: ', '].

		aStream
			nextPut: $<;
			nextPutAll: (self openElements at: i);
			nextPut: $>].

	(aStop < self openElements size)
		ifTrue: [aStream nextPutAll: '...'].! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 10/22/2014 13:52'!
printLimit
	^ 5! !

!XMLElementNester methodsFor: 'printing' stamp: 'monty 10/22/2014 13:52'!
printOn: aStream
	self
		printFrom: 1
		on: aStream! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/20/2014 20:00'!
scopedAttributeAt: aName
	^ self
		scopedAttributeAt: aName
		ifAbsent: ['']! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/21/2014 02:46'!
scopedAttributeAt: aName ifAbsent: aBlock
	^ self currentScopedAttributes
		ifNil: [aBlock value]
		ifNotNil: [:attributes |
			attributes
				at: aName
				ifAbsent: aBlock]! !

!XMLElementNester methodsFor: 'private' stamp: 'monty 1/20/2014 19:33'!
scopedAttributes
	^ scopedAttributes ifNil: [scopedAttributes := OrderedCollection new]! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 11/3/2014 13:20'!
setExternalRootElement: anElement
	rootElement := anElement.
	hasExternalRootElement := true.! !

!XMLElementNester methodsFor: 'accessing' stamp: 'monty 1/25/2014 21:24'!
setScopedAttributesNamed: aNameCollection from: aDictionary
	| attributes |

	aNameCollection do: [:each |
		(aDictionary at: each ifAbsent: [nil])
			ifNotNil: [:value |
				(attributes ifNil: [attributes := self copiedScopedAttributes])
					at: each
					put: value]].! !

!XMLElementNester methodsFor: 'checking' stamp: 'monty 1/20/2014 19:59'!
startElement: anElementName
	self openElements addLast: anElementName.
	self hasScopedAttributes
		ifTrue: [scopedAttributes addLast: self currentScopedAttributes].! !

XMLElementNester subclass: #XMLWellFormedElementNester
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLWellFormedElementNester commentStamp: '<historical>' prior: 0!
The class keeps track of elements as they start and end and ensures that they nest properly.!

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 12/11/2013 21:43'!
endElement: anElementName
	(self currentElement = anElementName)
		ifFalse: [
			^ (self hasOpenElement: anElementName)
				ifTrue: [self errorImproperlyNestedElements: anElementName]
				ifFalse: [self errorUnmatchedEndTag: anElementName]].

	super endElement: anElementName.! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 11/3/2014 13:25'!
endNesting
	self hasOpenElements
		ifTrue: [self errorUnterminatedElements]! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorImproperlyNestedElements: anElementName
	XMLWellFormednessException
		formatSignal: '<{1}> element ended before its child element(s) {2}'
		with: anElementName
		with: (self enclosedElementsAsString: anElementName)! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 11/3/2014 13:25'!
errorMultipleRoots
	XMLWellFormednessException signal: 'Document cannot have multiple root elements'! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorUnmatchedEndTag: anElementName
	XMLWellFormednessException
		formatSignal: 'End </{1}> tag with no corresponding start <{1}> tag'
		with: anElementName! !

!XMLWellFormedElementNester methodsFor: 'private' stamp: 'monty 11/3/2014 13:26'!
errorUnterminatedElements
	XMLWellFormednessException signal: 'Unterminated elements: ', self printString! !

!XMLWellFormedElementNester methodsFor: 'checking' stamp: 'monty 11/3/2014 13:29'!
startElement: anElementName
	rootElement
		ifNil: [rootElement := anElementName]
		ifNotNil: [
			(self openElements isEmpty
				and: [self hasExternalRootElement not])
				ifTrue: [self errorMultipleRoots]].

	super startElement: anElementName.! !

Object subclass: #XMLElementValidator
	instanceVariableNames: 'element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLElementValidator commentStamp: '<historical>' prior: 0!
This is a base class for element content validators.!

XMLElementValidator subclass: #XMLAnyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLAnyElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain any content.!

!XMLAnyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:42'!
atEnd
	^ true! !

XMLElementValidator subclass: #XMLDFAElementValidator
	instanceVariableNames: 'dfa currentState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLDFAElementValidator commentStamp: '<historical>' prior: 0!
This class can use a DFA to validate element content.
!

!XMLDFAElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:47'!
element: aName dfa: aDfa
	^ self new
		setElement: aName
		dfa: aDfa! !

!XMLDFAElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 05:43'!
atEnd
	^ currentState isAccepting! !

!XMLDFAElementValidator methodsFor: 'copying' stamp: 'monty 11/18/2013 05:41'!
postCopy
	currentState := dfa! !

!XMLDFAElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:47'!
setElement: aName dfa: aDfa
	element := aName.
	dfa := aDfa.
	currentState := dfa.! !

!XMLDFAElementValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 00:03'!
validatePCData: aString
	self errorUnexpectedPCData: aString! !

!XMLDFAElementValidator methodsFor: 'validation' stamp: 'monty 12/28/2013 00:21'!
validateStartTag: aName
	currentState :=
		currentState
			detectNextStateMatching: aName
			ifNone: [self errorUnexpectedElement: aName]! !

!XMLElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:48'!
element: aName
	^ self new element: aName! !

!XMLElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	self subclassResponsibility! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element
	^ element! !

!XMLElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:48'!
element: aName
	element := aName! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorUnexpectedElement: aName
	XMLValidationException
		formatSignal: 'Unexpected <{1}> element in <{2}> element'
		with: aName
		with: self element! !

!XMLElementValidator methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorUnexpectedPCData: aString
	XMLValidationException
		formatSignal: 'Unexpected PCDATA in <{1}> element'
		with: self element! !

!XMLElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 05:24'!
validatePCData: aString! !

!XMLElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 05:24'!
validateStartTag: aName! !

XMLElementValidator subclass: #XMLEmptyElementValidator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLEmptyElementValidator commentStamp: '<historical>' prior: 0!
This class validates empty elements.
!

!XMLEmptyElementValidator methodsFor: 'testing' stamp: 'monty 11/18/2013 04:41'!
atEnd
	^ true! !

!XMLEmptyElementValidator methodsFor: 'private' stamp: 'monty 10/30/2014 19:33'!
errorMustBeEmpty
	XMLValidationException
		formatSignal: 'Element <{1}> must be empty'
		with: self element! !

!XMLEmptyElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 04:32'!
validatePCData: aString
	self errorMustBeEmpty! !

!XMLEmptyElementValidator methodsFor: 'validation' stamp: 'monty 11/18/2013 04:34'!
validateStartTag: aName
	self errorMustBeEmpty! !

XMLElementValidator subclass: #XMLMixedContentElementValidator
	instanceVariableNames: 'allowedElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation'!

!XMLMixedContentElementValidator commentStamp: '<historical>' prior: 0!
This class validates elements that can contain a mix of PCDATA and certain elements.!

!XMLMixedContentElementValidator class methodsFor: 'instance creation' stamp: 'monty 12/7/2013 19:50'!
element: aName allowedElements: aNameCollection
	^ self new
		setElement: aName
		allowedElements: aNameCollection! !

!XMLMixedContentElementValidator methodsFor: 'accessing' stamp: 'monty 12/7/2013 19:51'!
allowedElements
	^ allowedElements! !

!XMLMixedContentElementValidator methodsFor: 'testing' stamp: 'monty 11/19/2013 01:14'!
atEnd
	^ true! !

!XMLMixedContentElementValidator methodsFor: 'initialization' stamp: 'monty 12/7/2013 19:50'!
setElement: aName allowedElements: aNameCollection
	element := aName.
	allowedElements := aNameCollection.! !

!XMLMixedContentElementValidator methodsFor: 'validation' stamp: 'monty 12/7/2013 19:51'!
validateStartTag: aName
	(self allowedElements includes: aName)
		ifFalse: [self errorUnexpectedElement: aName]
	! !

Object subclass: #XMLEncodingDetector
	instanceVariableNames: 'stream remainder streamDecoder'
	classVariableNames: 'NullCharacter UTF16BigEndianByteOrderMark UTF16LittleEndianByteOrderMark UTF32BigEndianByteOrderMark UTF32LittleEndianByteOrderMark UTF8ByteOrderMark'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLEncodingDetector commentStamp: 'monty 11/15/2014 05:04' prior: 0!
An encoding auto-detector for streams.!

!XMLEncodingDetector class methodsFor: 'accessing' stamp: 'monty 11/15/2014 07:07'!
handlerForLeadByte: aByte
	aByte = 0
		ifTrue: [^ #detectBigEndianUTF32OrImplicitEncdoing].
	aByte = XMLUTF16BigEndianStreamDecoder byteOrderMarkBytes first
		ifTrue: [^ #detectBigEndianUTF16ByteOrderMark].
	aByte = XMLUTF8StreamDecoder byteOrderMarkBytes first
		ifTrue: [^ #detectUTF8ByteOrderMark].
	aByte = XMLUTF16LittleEndianStreamDecoder byteOrderMarkBytes first
		ifTrue: [^ #detectLittleEndianUTF16OrUTF32ByteOrderMark].
	^ nil.! !

!XMLEncodingDetector class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 07:05'!
initialize
	"self initialize"

	NullCharacter := Character value: 0.

	UTF8ByteOrderMark := XMLUTF8StreamDecoder byteOrderMark.
	UTF16BigEndianByteOrderMark := XMLUTF16BigEndianStreamDecoder byteOrderMark.
	UTF16LittleEndianByteOrderMark := XMLUTF16LittleEndianStreamDecoder byteOrderMark.
	UTF32BigEndianByteOrderMark := XMLUTF32BigEndianStreamDecoder byteOrderMark.
	UTF32LittleEndianByteOrderMark := XMLUTF32LittleEndianStreamDecoder byteOrderMark.! !

!XMLEncodingDetector class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 04:58'!
on: aStream
	^ self new on: aStream! !

!XMLEncodingDetector methodsFor: 'encodings' stamp: 'monty 11/15/2014 06:47'!
detectBigEndianUTF16ByteOrderMark
	self
		detectByteOrderMark: UTF16BigEndianByteOrderMark
		forDecoderClass: XMLUTF16BigEndianStreamDecoder! !

!XMLEncodingDetector methodsFor: 'encodings' stamp: 'monty 11/19/2014 05:50'!
detectBigEndianUTF32OrImplicitEncdoing
	self
		detectByteOrderMark: UTF32BigEndianByteOrderMark
		forDecoderClass: XMLUTF32BigEndianStreamDecoder.
	(remainder ifNil: [^ self]) do: [:each |
		each == NullCharacter
			ifFalse: [^ self]].

	(remainder size = 1
		and: [stream atEnd not
			and: [stream peek == $<]])
		ifTrue: [
			stream next.
			streamDecoder := XMLUTF16BigEndianStreamDecoder new.
			remainder := '<'.
			^ self].
	(remainder size = 2
		and: [stream atEnd not
			and: [stream peek == NullCharacter]])
		ifTrue: [
			stream next.
			stream atEnd
				ifFalse: [ 
					remainder := stream next asString.
					streamDecoder := XMLUTF32BigEndianStreamDecoder new]].! !

!XMLEncodingDetector methodsFor: 'private' stamp: 'monty 11/15/2014 07:30'!
detectByteOrderMark: aByteOrderMark forDecoderClass: aClass
	stream next.
	2 to: aByteOrderMark size do: [:i |
		(stream atEnd not
			and: [stream peek == (aByteOrderMark at: i)])
			ifFalse: [
				remainder := aByteOrderMark copyFrom: 1 to: i - 1.
				^ self].
		stream next].
	streamDecoder := aClass new.! !

!XMLEncodingDetector methodsFor: 'encodings' stamp: 'monty 11/15/2014 07:20'!
detectLittleEndianImplicitEncoding
	stream next.
	(stream atEnd not
		and: [stream peek == NullCharacter])
		ifTrue: [
			stream next.
			stream atEnd
				ifFalse: [stream next].
			streamDecoder := XMLUTF32LittleEndianStreamDecoder new]
		ifFalse: [streamDecoder := XMLUTF16LittleEndianStreamDecoder new]! !

!XMLEncodingDetector methodsFor: 'encodings' stamp: 'monty 11/15/2014 07:11'!
detectLittleEndianUTF16OrUTF32ByteOrderMark
	self
		detectByteOrderMark: UTF32LittleEndianByteOrderMark
		forDecoderClass: XMLUTF32LittleEndianStreamDecoder.

	remainder = UTF16LittleEndianByteOrderMark
		ifTrue: [
			remainder := nil.
			streamDecoder := XMLUTF16LittleEndianStreamDecoder new].! !

!XMLEncodingDetector methodsFor: 'encodings' stamp: 'monty 11/15/2014 06:46'!
detectUTF8ByteOrderMark
	self
		detectByteOrderMark: UTF8ByteOrderMark
		forDecoderClass: XMLUTF8StreamDecoder! !

!XMLEncodingDetector methodsFor: 'initialization' stamp: 'monty 11/15/2014 04:58'!
on: aStream
	stream := aStream! !

!XMLEncodingDetector methodsFor: 'accessing' stamp: 'monty 11/15/2014 06:10'!
remainder
	^ remainder! !

!XMLEncodingDetector methodsFor: 'accessing' stamp: 'monty 11/15/2014 06:10'!
streamDecoder
	^ streamDecoder! !

Object subclass: #XMLIntegerReader
	instanceVariableNames: 'stream'
	classVariableNames: 'CharacterDigitValues'
	poolDictionaries: ''
	category: 'XML-Parser-Util'!

!XMLIntegerReader commentStamp: 'monty 1/21/2014 01:36' prior: 0!
This class reads hex or decimal integers from a minimal stream supporting next, peek, and atEnd.!

!XMLIntegerReader class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 07:04'!
initialize
	"self initialize"

	"array instead of dictionary for speed"
	CharacterDigitValues := Array new: 256 withAll: -1.
	($0 to: $9) do: [:each |
		CharacterDigitValues
			at: each asciiValue
			put: each asciiValue - $0 asciiValue].
	($A to: $F) do: [:each |
		CharacterDigitValues
			at: each asciiValue
			put: each asciiValue - $A asciiValue + 10].
	($a to: $f) do: [:each |
		CharacterDigitValues
			at: each asciiValue
			put: each asciiValue - $a asciiValue + 10].! !

!XMLIntegerReader class methodsFor: 'private' stamp: 'monty 10/24/2014 20:28'!
maxBase
	^ 16! !

!XMLIntegerReader class methodsFor: 'private' stamp: 'monty 10/24/2014 20:15'!
maxIntegerValue
	"SmallInteger maxVal"
	^ 1073741823! !

!XMLIntegerReader class methodsFor: 'instance creation' stamp: 'monty 11/16/2014 04:55'!
on: aStream
	^ self new on: aStream! !

!XMLIntegerReader class methodsFor: 'instance creation' stamp: 'monty 10/27/2014 08:36'!
readFrom: aStream
	^ (self on: aStream) nextInteger! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 10/27/2014 08:37'!
readFrom: aStream withBase: aBase
	^ (self on: aStream) nextIntegerWithBase: aBase! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 10/27/2014 08:43'!
readFrom: aStream withBase: aFirstBase startingWithAny: aCharacterCollection orBase: aSecondBase
	^ (self on: aStream)
		nextIntegerWithBase: aFirstBase
		startingWithAny: aCharacterCollection
		orBase: aSecondBase! !

!XMLIntegerReader class methodsFor: 'reading' stamp: 'monty 1/21/2014 17:42'!
readFrom: aStream withBase: aBase upTo: anInteger
	^ (self on: aStream)
		nextIntegerWithBase: aBase
		upTo: anInteger! !

!XMLIntegerReader methodsFor: 'testing' stamp: 'monty 10/27/2014 08:39'!
atEnd
	^ stream atEnd! !

!XMLIntegerReader methodsFor: 'accessing' stamp: 'monty 10/27/2014 08:39'!
next
	^ stream next! !

!XMLIntegerReader methodsFor: 'private' stamp: 'monty 10/27/2014 08:40'!
nextDigitWithBase: aBase
	| digit |
	
	digit :=
		CharacterDigitValues
			at: self peek asciiValue
			ifAbsent: [^ nil].
	(digit >= 0 and: [digit < aBase])
		ifFalse: [^ nil].
	self next.
	^ digit.! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:37'!
nextInteger
	^ self nextIntegerWithBase: 10! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:35'!
nextIntegerWithBase: aBase
	^ self nextIntegerWithBase: aBase upTo: nil! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 10/27/2014 08:44'!
nextIntegerWithBase: aFirstBase startingWithAny: aCharacterCollection orBase: aSecondBase
	(aCharacterCollection includes: self peek)
		ifTrue: [
			self next.
			^ self nextIntegerWithBase: aFirstBase]
		ifFalse: [^ self nextIntegerWithBase: aSecondBase]! !

!XMLIntegerReader methodsFor: 'reading' stamp: 'monty 11/14/2014 20:15'!
nextIntegerWithBase: aBase upTo: aReadLimit
	| integer maxValue maxBaseValue digitValue readLimit |

	(aBase > 1 and: [aBase <= self class maxBase])
		ifFalse: [Error signal: 'Unsupported base ', aBase printString].

	maxValue := self class maxIntegerValue.
	maxBaseValue := (maxValue / aBase) floor.
	readLimit := aReadLimit.
	[self atEnd
		or: [(readLimit notNil and: [readLimit <= 0])
			or: [(digitValue := self nextDigitWithBase: aBase) isNil]]]
		whileFalse: [
			integer ifNil: [integer := 0].
	
			integer > maxBaseValue
				ifTrue: [^ maxValue].
			integer := integer * aBase.

			integer > (maxValue - digitValue)
				ifTrue: [^ maxValue].
			integer := integer + digitValue.
			readLimit ifNotNil: [readLimit := readLimit - 1]].
	^ integer.! !

!XMLIntegerReader methodsFor: 'initialization' stamp: 'monty 11/16/2014 04:54'!
on: aStream
	stream := aStream! !

!XMLIntegerReader methodsFor: 'accessing' stamp: 'monty 10/27/2014 08:39'!
peek
	^ stream peek! !

Object subclass: #XMLKeyValueCache
	instanceVariableNames: 'collection maxSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Util'!

!XMLKeyValueCache commentStamp: 'monty 11/11/2014 23:48' prior: 0!
A simple key-value cache that empties when full.
!

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:57'!
collection: aCollection maxSize: anInteger
	^ self basicNew initialize
		setCollection: aCollection
		maxSize: anInteger! !

!XMLKeyValueCache class methodsFor: 'private' stamp: 'monty 11/20/2014 02:25'!
defaultMaxSize
	^ 16! !

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:56'!
maxSize: anInteger
	^ self
		collection: (Dictionary new: anInteger)
		maxSize: anInteger! !

!XMLKeyValueCache class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 20:56'!
new
	^ self maxSize: self defaultMaxSize! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 21:01'!
at: aKey
	^ self collection at: aKey! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 21:01'!
at: aKey ifAbsent: aBlock
	^ self collection at: aKey ifAbsent: aBlock! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 21:02'!
at: aKey ifAbsentPut: aBlock
	^ self collection
		at: aKey
		ifAbsent: [
			self collection size >= self maxSize
				ifTrue: [self clear].

			self collection
				at: aKey
				put: aBlock value]! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/16/2014 00:04'!
at: aKey put: aValue
	(self collection includesKey: aKey)
		ifFalse: [
			self collection size >= self maxSize
				ifTrue: [self clear]].
	^ self collection at: aKey put: aValue.! !

!XMLKeyValueCache methodsFor: 'clearing' stamp: 'monty 11/15/2014 20:59'!
clear
	self collection removeAll! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/11/2014 23:45'!
collection
	^ collection! !

!XMLKeyValueCache methodsFor: 'testing' stamp: 'monty 11/15/2014 23:59'!
includesKey: aKey
	^ self collection includesKey: aKey! !

!XMLKeyValueCache methodsFor: 'testing' stamp: 'monty 11/12/2014 04:03'!
isEmpty
	^ self collection isEmpty! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:57'!
maxSize
	^ maxSize! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:59'!
maxSize: anInteger
	anInteger < maxSize
		ifTrue: [self clear].

	maxSize := anInteger.! !

!XMLKeyValueCache methodsFor: 'copying' stamp: 'monty 11/16/2014 07:27'!
postCopy
	collection := collection copy! !

!XMLKeyValueCache methodsFor: 'initialization' stamp: 'monty 11/15/2014 20:57'!
setCollection: aCollection maxSize: anInteger
	collection := aCollection.
	maxSize := anInteger.! !

!XMLKeyValueCache methodsFor: 'accessing' stamp: 'monty 11/15/2014 20:48'!
size
	^ self collection size! !

Object subclass: #XMLNFABuilder
	instanceVariableNames: 'fragmentStack'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABuilder commentStamp: 'monty 1/7/2014 19:39' prior: 0!
This class builds an NFA using a stack of NFAFragments.!

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:26'!
applyBranch
	| branchState leftFragment  rightFragment |

	leftFragment := self popFragment.
	rightFragment := self popFragment.
	branchState :=
		XMLNFABranchingState
			leftBranch: leftFragment firstState
			rightBranch: rightFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalStates: (leftFragment combinedTerminalStatesFrom: rightFragment)).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 10/24/2014 12:44'!
applyConcatenation
	| firstFragment secondFragment |

	secondFragment := self popFragment.
	firstFragment := self popFragment.

	firstFragment connectToFragment: secondFragment.

	self pushFragment: firstFragment.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 10/24/2014 13:07'!
applyOptional
	| optionalFragment branchingState |

	optionalFragment := self popFragment.
	branchingState :=
		XMLNFABranchingState branch: optionalFragment firstState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchingState
			terminalStates: (optionalFragment combinedTerminalStatesWith: branchingState))! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 01:19'!
applyPlus
	| plusFragment branchState |

	plusFragment := self topFragment.
	branchState := XMLNFABranchingState branch: plusFragment firstState.
	plusFragment
		connectToState: branchState;
		terminalState: branchState.! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 23:55'!
applyStar
	| branchState starFragment |

	starFragment := self popFragment.
	branchState := XMLNFABranchingState branch: starFragment firstState.
	starFragment connectToState: branchState.

	self pushFragment:
		(XMLNFAFragment
			firstState: branchState
			terminalState: branchState)! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/28/2013 22:51'!
firstStateOfAcceptingNFA
	| firstFragment |

	[self totalFragments > 1]
		whileTrue: [self applyConcatenation].

	firstFragment := self popFragment.
	firstFragment connectToState: XMLNFAAcceptingState new.

	^ firstFragment firstState.! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 14:16'!
fragmentStack
	^ fragmentStack ifNil: [fragmentStack := OrderedCollection new]! !

!XMLNFABuilder methodsFor: 'testing' stamp: 'monty 12/6/2013 06:40'!
hasFragments
	^ self fragmentStack notEmpty! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
popFragment
	^ self fragmentStack removeLast! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 08:01'!
pushFragment: aFragment
	self fragmentStack addLast: aFragment! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 12/27/2013 21:41'!
pushMatcher: aMatcher
	| firstState |

	firstState := XMLNFAMatchingState matcher: aMatcher.
	self pushFragment:
		(XMLNFAFragment
			firstState: firstState
			terminalState: firstState).! !

!XMLNFABuilder methodsFor: 'building' stamp: 'monty 11/17/2013 12:01'!
topFragment
	^ self fragmentStack last! !

!XMLNFABuilder methodsFor: 'accessing' stamp: 'monty 11/17/2013 15:04'!
totalFragments
	^ self fragmentStack size! !

Object subclass: #XMLNFAFragment
	instanceVariableNames: 'firstState terminalStates'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAFragment commentStamp: 'monty 12/28/2013 21:51' prior: 0!
This is a fragment used to build an NFA by the NFABuilder.!

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalState: aTerminalState
	^ self new
		firstState: aFirstState;
		terminalState: aTerminalState! !

!XMLNFAFragment class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:50'!
firstState: aFirstState terminalStates: aTerminalStateCollection
	^ self new
		firstState: aFirstState;
		terminalStates: aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 10/24/2014 13:21'!
combinedTerminalStatesFrom: aFragment
	^ terminalStates isCollection
		ifTrue: [aFragment terminalStatesAddedTo: terminalStates]
		ifFalse: [
			aFragment terminalStates isCollection
				ifTrue: [self terminalStatesAddedTo: aFragment terminalStates]
				ifFalse: [
					self
						terminalStateCollectionWith: terminalStates
						and: aFragment terminalStates]]! !

!XMLNFAFragment methodsFor: 'converting' stamp: 'monty 10/24/2014 13:07'!
combinedTerminalStatesWith: aState
	^ terminalStates isCollection
		ifTrue: [
			terminalStates
				addLast: aState;
				yourself]
		ifFalse: [self terminalStateCollectionWith: terminalStates and: aState]! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 10/24/2014 12:45'!
connectToFragment: aFragment
	self connectToState: aFragment firstState.
	terminalStates := aFragment terminalStates.! !

!XMLNFAFragment methodsFor: 'connecting' stamp: 'monty 10/24/2014 13:06'!
connectToState: aState
	terminalStates isCollection
		ifTrue: [terminalStates do: [:each | each connectToState: aState]]
		ifFalse: [terminalStates connectToState: aState]! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 10:48'!
firstState
	^ firstState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 11/17/2013 11:51'!
firstState: aState
	firstState := aState! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 10/24/2014 12:35'!
terminalState: aTerminalState
	terminalStates := aTerminalState! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 12:52'!
terminalStateCollectionWith: aFirstState and: aSecondState
	^ (OrderedCollection new: 4)
		addLast: aFirstState;
		addLast: aSecondState;
		yourself! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 12:40'!
terminalStates
	^ terminalStates! !

!XMLNFAFragment methodsFor: 'accessing' stamp: 'monty 12/27/2013 23:50'!
terminalStates: aTerminalStateCollection
	terminalStates := aTerminalStateCollection! !

!XMLNFAFragment methodsFor: 'private' stamp: 'monty 10/24/2014 13:21'!
terminalStatesAddedTo: aCollection
	terminalStates isCollection
		ifTrue: [aCollection addAll: terminalStates]
		ifFalse: [aCollection addLast: terminalStates].
	^ aCollection.! !

Object subclass: #XMLNFAState
	instanceVariableNames: 'lastAddedTo'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAState commentStamp: 'monty 12/28/2013 22:34' prior: 0!
This is a base class for NFA states.!

XMLNFAState subclass: #XMLNFAAcceptingState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAAcceptingState commentStamp: 'monty 10/7/2014 01:27' prior: 0!
An NFA in this state is considered to accept the input.!

!XMLNFAAcceptingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:23'!
isAccepting
	^ true! !

XMLNFAState subclass: #XMLNFABranchingState
	instanceVariableNames: 'leftBranch rightBranch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFABranchingState commentStamp: 'monty 10/7/2014 01:27' prior: 0!
This state can branch off into two different states without matching against anything.!

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 23:43'!
branch: aLeftBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: nil! !

!XMLNFABranchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 22:04'!
leftBranch: aLeftBranch rightBranch: aRightBranch
	^ self new
		setLeftBranch: aLeftBranch
		rightBranch: aRightBranch! !

!XMLNFABranchingState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			leftBranch addTo: aStateSet.
			rightBranch addTo: aStateSet]! !

!XMLNFABranchingState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState
	leftBranch ifNil: [leftBranch := aState].
	rightBranch ifNil: [rightBranch := aState].! !

!XMLNFABranchingState methodsFor: 'initialization' stamp: 'monty 12/28/2013 00:45'!
setLeftBranch: aLeftBranch rightBranch: aRightBranch
	leftBranch := aLeftBranch.
	rightBranch := aRightBranch.! !

XMLNFAState subclass: #XMLNFAMatchingState
	instanceVariableNames: 'matcher nextState'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Validation-DFA'!

!XMLNFAMatchingState commentStamp: 'monty 10/9/2014 20:58' prior: 0!
This state can pass to a next state if the input matches against an object.!

!XMLNFAMatchingState class methodsFor: 'instance creation' stamp: 'monty 12/27/2013 21:15'!
matcher: aMatcher
	^ self new matcher: aMatcher! !

!XMLNFAMatchingState methodsFor: 'connecting' stamp: 'monty 12/28/2013 21:52'!
connectToState: aState
	nextState := aState! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:16'!
matcher: aMatcher
	matcher := aMatcher! !

!XMLNFAMatchingState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:15'!
matches: anObject
	^ matcher = anObject! !

!XMLNFAMatchingState methodsFor: 'accessing' stamp: 'monty 12/27/2013 21:13'!
nextState
	^ nextState! !

!XMLNFAState methodsFor: 'adding' stamp: 'monty 12/29/2013 01:10'!
addTo: aStateSet
	lastAddedTo == aStateSet
		ifFalse: [
			"This avoids getting caught in a loop"
			lastAddedTo := aStateSet.

			aStateSet add: self]! !

!XMLNFAState methodsFor: 'converting' stamp: 'monty 12/28/2013 22:33'!
asDFAState
	| nfaStates |

	nfaStates := XMLNFAStateSet new.
	self addTo: nfaStates.

	^ XMLDFAState nfaStates: nfaStates.! !

!XMLNFAState methodsFor: 'connecting' stamp: 'monty 12/27/2013 23:45'!
connectToState: aState! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 1/7/2014 19:37'!
isAccepting
	^ false! !

!XMLNFAState methodsFor: 'testing' stamp: 'monty 12/27/2013 21:10'!
matches: anObject
	^ false! !

Object subclass: #XMLNamespaceScope
	instanceVariableNames: 'prefixMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScope commentStamp: 'monty 2/2/2014 06:17' prior: 0!
A namespace scope is a possibly empty set of mappings from namespace prefixes to URIs and an optional default namespace URI to be used for unprefixed elements. Scopes can be created from other scopes by copying.!

!XMLNamespaceScope class methodsFor: 'predefined namespaces' stamp: 'monty 10/8/2014 23:27'!
xmlNamespaceURI
	^ 'http://www.w3.org/XML/1998/namespace'! !

!XMLNamespaceScope class methodsFor: 'predefined namespaces' stamp: 'monty 10/8/2014 23:27'!
xmlnsNamespaceURI
	^ 'http://www.w3.org/2000/xmlns/'! !

!XMLNamespaceScope methodsFor: 'comparing' stamp: 'monty 10/12/2014 06:32'!
= anObject
	self == anObject
		ifTrue: [^ true].
	self class == anObject class
		ifFalse: [^ false].

	^ self hasNamespaces
		ifTrue: [anObject hasNamespaces and: [prefixMappings = anObject prefixMappings]]
		ifFalse: [anObject hasNamespaces not].! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/10/2014 05:47'!
defaultNamespace
	^ prefixMappings
		ifNil: ['']
		ifNotNil: [prefixMappings at: '' ifAbsent: ['']]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/31/2014 16:52'!
defaultNamespace: aUri
	(aUri = self class xmlNamespaceURI
		or: [aUri = self class xmlnsNamespaceURI])
		ifTrue: [self errorCannotUseXmlOrXmlnsURIAsDefault].

	self prefixMappings at: '' put: aUri.! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:38'!
errorAttribute: anAttribute aliasesNamespace: aNamespace
	XMLNamespaceException
		formatSignal: 'Attribute "{1}" has alias for namespace "{2}"'
		with: anAttribute
		with: aNamespace! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/6/2014 12:19'!
errorCannotAliasXmlOrXmlnsURI
	XMLNamespaceException signal: 'Cannot alias the "xml" or "xmlns" prefixes'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/6/2014 12:18'!
errorCannotMapPrefixToEmptyURI
	XMLNamespaceException signal: 'Cannot map namespace prefixes to empty URIs'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:46'!
errorCannotMapXmlnsPrefix
	XMLNamespaceException signal:
		'Cannot map "xmlns" prefix to any namespace, even the current "xmlns" namespace'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:41'!
errorCannotRemapXmlPrefix
	XMLNamespaceException signal: 'Cannot map the "xml" prefix to a new namespace URI'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/6/2014 18:29'!
errorCannotUseXmlOrXmlnsURIAsDefault
	XMLNamespaceException signal:
		'Cannot use "xml" or  "xmlns" namespace URI as default namespace'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/6/2014 18:28'!
errorCannotUseXmlnsPrefix	
	XMLNamespaceException signal: 'Cannot use reserved "xmlns" prefix for elements or attributes'! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:42'!
errorInvalidPrefix: aPrefix	
	XMLNamespaceException
		formatSignal: 'Prefix "{1}" contains ":" character'
		with: aPrefix! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:40'!
errorInvalidUnprefixedName: aName
	XMLNamespaceException
		formatSignal: 'Name "{1}" contains ":" character'
		with: aName! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:50'!
errorUnmappedPrefix: aPrefix
	XMLNamespaceException
		formatSignal: 'Unmapped namespace prefix "{1}" in name'
		with: aPrefix! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/12/2014 03:21'!
hasDefaultNamespace
	^ self includesPrefix: ''! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 10/10/2014 05:51'!
hasNamespaces
	^ prefixMappings notNil and: [prefixMappings notEmpty]! !

!XMLNamespaceScope methodsFor: 'comparing' stamp: 'monty 10/28/2014 22:09'!
hash
	| hash |

	hash := self class hash.
	self prefixesAndURIsDo: [:prefix :uri | hash := hash bitXor: uri hash].
	^ hash.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 11/17/2014 16:45'!
includesPrefix: aPrefix
	self prefixMappingAt: aPrefix ifAbsent: [^ false].
	^ true.! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'monty 11/17/2014 16:45'!
includesPrefix: aPrefix mappedTo: aUri
	^ (self prefixMappingAt: aPrefix ifAbsent: [^ false]) = aUri! !

!XMLNamespaceScope methodsFor: 'initializing' stamp: 'monty 10/12/2014 05:05'!
inheritMappingsFrom: anEnclosingScope
	anEnclosingScope hasNamespaces
		ifTrue: [
			anEnclosingScope prefixesAndURIsDo: [:prefix :uri |
				(self includesPrefix: prefix)
					ifFalse: [self mapPrefix: prefix to: uri]]]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/31/2014 17:01'!
mapPrefix: aPrefix to: aUri
	aPrefix isEmpty
		ifTrue: [^ self defaultNamespace: aUri].
	(aPrefix at: 1) == $x "skip comparisons below"
		ifTrue: [
			aPrefix = 'xml'
				ifTrue: [
					aUri = self class xmlNamespaceURI
						ifFalse: [self errorCannotRemapXmlPrefix].
					self prefixMappings at: 'xml' put: aUri.
					^ self].
			aPrefix = 'xmlns'
				ifTrue: [self errorCannotMapXmlnsPrefix]].
	(aPrefix indexOf: $:) > 0 
		ifTrue: [self errorInvalidPrefix: aPrefix].

	aUri isEmpty
		ifTrue: [self errorCannotMapPrefixToEmptyURI].
	(aUri = self class xmlNamespaceURI or: [aUri = self class xmlnsNamespaceURI])
		ifTrue: [self errorCannotAliasXmlOrXmlnsURI].
	self prefixMappings at: aPrefix put: aUri.! !

!XMLNamespaceScope methodsFor: 'copying' stamp: 'monty 11/22/2014 08:21'!
postCopy
	prefixMappings
		ifNotNil: [prefixMappings := prefixMappings copy]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 11/17/2014 16:45'!
prefixMappingAt: aPrefix ifAbsent: aBlock
	^ prefixMappings
		ifNil: [aBlock value]
		ifNotNil: [prefixMappings at: aPrefix ifAbsent: aBlock]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/20/2014 18:46'!
prefixMappings
	^ prefixMappings ifNil: [prefixMappings := OrderPreservingStringDictionary new: 5]! !

!XMLNamespaceScope methodsFor: 'enumerating' stamp: 'monty 11/23/2014 23:20'!
prefixesAndURIsDo: aTwoArgumentBlock
	prefixMappings
		ifNotNil: [prefixMappings keysAndValuesDo: aTwoArgumentBlock]! !

!XMLNamespaceScope methodsFor: 'printing' stamp: 'monty 10/14/2014 05:42'!
printOn: aStream
	self prefixesAndURIsDo: [:prefix :uri |
		aStream nextPutAll: 'xmlns'.
		prefix ifNotEmpty: [
			aStream
				nextPut: $:;
				nextPutAll: prefix].
		aStream
			nextPut: $=;
			nextPut: $";
			nextPutAll: uri;
			nextPut: $";
			space]! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 10/12/2014 02:28'!
resolvePrefix: aPrefix
	^ self resolvePrefix: aPrefix ifUnresolvable: ['']! !

!XMLNamespaceScope methodsFor: 'resolving' stamp: 'monty 11/17/2014 16:45'!
resolvePrefix: aPrefix ifUnresolvable: aBlock
	^ self
		prefixMappingAt: aPrefix
		ifAbsent: [
			aPrefix = 'xml'
				ifTrue: [self class xmlNamespaceURI]
				ifFalse: [
					aPrefix = 'xmlns'
						ifTrue: [self class xmlnsNamespaceURI]
						ifFalse: [aBlock value]]]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'monty 10/30/2014 19:33'!
unmapPrefix: aPrefix
	self prefixMappings
		removeKey: aPrefix
		ifAbsent: [
			XMLNamespaceException
				formatSignal: 'Prefix "{1}" was not mapped and cannot be unmapped'
				with: aPrefix]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 11/17/2014 16:45'!
validateAndResolvePrefix: aPrefix
	^ self
		prefixMappingAt: aPrefix
		ifAbsent: [
			aPrefix = 'xml'
				ifTrue: [self class xmlNamespaceURI]
				ifFalse: [					
					(aPrefix = 'xmlns'
						ifTrue: [self errorCannotUseXmlnsPrefix]
						ifFalse: [self errorUnmappedPrefix: aPrefix])]]! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'monty 10/9/2014 17:39'!
validateAndResolvePrefix: aPrefix ofElement: aName
	aPrefix isEmpty
		ifTrue: [
			self validateUnprefixedName: aName.
			^ self defaultNamespace]
		ifFalse: [^ self validateAndResolvePrefix: aPrefix]! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'monty 11/16/2014 20:23'!
validateAttributePrefixes: aDictionary
	"performance critical method"
	| namespacedAttributes |

	aDictionary keysDo: [:each | | prefix |
		(prefix := each prefixBeforeLocalName) isEmpty "avoid ifNotEmpty: closure overhead"
			ifTrue: [self validateUnprefixedName: each]
			ifFalse: [ | namespace oldSize |
				"not initialized unless needed"
				namespacedAttributes
					ifNil: [namespacedAttributes := Set new: aDictionary size].
				namespace := self validateAndResolvePrefix: prefix.

				"checking size after is faster than includes:"
				oldSize := namespacedAttributes size.
				namespacedAttributes add: namespace, (each localNameAfterPrefix: prefix).
				namespacedAttributes size > oldSize
					ifFalse: [self errorAttribute: each aliasesNamespace: namespace]]]! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'monty 10/31/2014 16:40'!
validateUnprefixedName: aName
	"catch names like ':name' with an empty prefix"
	(aName size > 0
		and: [(aName at: 1) == $:])
		ifTrue: [self errorInvalidUnprefixedName: aName]! !

Object subclass: #XMLNamespaceScopeAllocator
	instanceVariableNames: 'scopes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScopeAllocator commentStamp: '<historical>' prior: 0!
This class handles the allocation of namespace scopes using a stack. Sending #enterScope to an instance will cause it to push a new scope onto the stack that inherits all mappings from the previous, enclosing scope and makes it the current scope. Sending #exitScop causes that scope to be removed and the parent scope to again be the current scope.!

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:26'!
currentScope
	^ self scopes lastOrNil ifNil: [self emptyScope]! !

!XMLNamespaceScopeAllocator methodsFor: 'accessing' stamp: 'monty 10/30/2014 12:27'!
currentScope: aScope
	self scopes addLast: aScope! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'JAAyer 3/13/2010 13:44'!
emptyScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'monty 10/30/2014 12:27'!
enterScope
	^ self scopes addLast: self currentScope copy! !

!XMLNamespaceScopeAllocator methodsFor: 'scope' stamp: 'monty 10/30/2014 12:27'!
exitScope
	^ self scopes removeLast! !

!XMLNamespaceScopeAllocator methodsFor: 'initialization' stamp: 'monty 10/27/2014 09:21'!
initialize
	super initialize.
	scopes := OrderedCollection new: 10.! !

!XMLNamespaceScopeAllocator methodsFor: 'private' stamp: 'monty 10/30/2014 12:26'!
scopes
	^ scopes! !

Object subclass: #XMLNestedStreamReader
	instanceVariableNames: 'stream onClose nestedStreams peekChar writeBuffer normalizedLineEndingChar currentLineNumber readLimit isEscapingQuotes'
	classVariableNames: 'CarriageReturn EncodingHandlerTable LineFeed'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNestedStreamReader commentStamp: '<historical>' prior: 0!
This class represents a stream reader that implements pushBack: using nested streams. This enables subsitution/replacement to be performed without modifying the underlying collections streamed-over or having to copy them. It also performs line-ending normalization, transforming CR and CRLF sequences into a single LF character.

(The code in this class is optimized; refactor with care.)!

!XMLNestedStreamReader class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 07:05'!
initialize
	"self initialize"

	CarriageReturn := Character cr.
	LineFeed := Character lf.

	EncodingHandlerTable := Array new: 256.
	0 to: 255 do: [:each |
		EncodingHandlerTable
			at: each + 1
			put: (XMLEncodingDetector handlerForLeadByte: each)]! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'monty 11/12/2014 21:36'!
on: aStream
	^ self
		on: aStream
		readLimit: nil! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'monty 1/4/2014 08:05'!
on: anInitialStream readLimit: aReadLimit
	^ self new
		setStream: anInitialStream
		readLimit: aReadLimit! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 11/13/2014 04:57'!
atEnd
	^ peekChar == nil
		and: [stream atEnd
			and: [self hasNestedStreams not
				or: [nestedStreams allSatisfy: [:each | each key atEnd]]]]! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 2/6/2014 01:01'!
atQuote
	^ self peek == $"
		or: [peekChar == $']! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 11/17/2014 03:40'!
basicNext
	"Returns next character in the stream after performing line-ending normalization."
	| nextChar |

	stream atEnd
		ifTrue: [
			[self hasNestedStreams and: [stream atEnd]]
				whileTrue: [self popCurrentStream].
			stream atEnd
				ifTrue: [^ nil]].
	(nextChar := stream next) == CarriageReturn
		ifTrue: [
			[stream atEnd and: [self hasNestedStreams]]
				whileTrue: [self popCurrentStream].
			stream atEnd
				ifFalse: [
					(stream peek == LineFeed)
						ifTrue: [stream next]].
			^ normalizedLineEndingChar].
	(nextChar == LineFeed)
		ifTrue: [^ normalizedLineEndingChar].
	isEscapingQuotes
		ifTrue: [^ self escapeQuote: nextChar].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/13/2014 05:11'!
closeCurrentStream
	(stream respondsTo: #close)
		ifTrue: [stream close].
	onClose ifNotNil: [onClose value].! !

!XMLNestedStreamReader methodsFor: 'file open/close' stamp: 'monty 11/24/2014 19:26'!
closeStreams
	[self hasNestedStreams]
		whileTrue: [self popCurrentStream].
	self closeCurrentStream.! !

!XMLNestedStreamReader methodsFor: 'encodings' stamp: 'monty 11/18/2014 19:18'!
convertFromEncoding: anEncodingName
	| streamDecoder |

	"encoding declarations for the default encoding are ignored on ReadStreams,
	which are assumed to be on strings that don't need decoding"
	streamDecoder := XMLStreamDecoder newForEncoding: anEncodingName.
	(streamDecoder == nil
		or: [(stream isKindOf: ReadStream)
			and: [streamDecoder isDefaultStreamDecoder]])
		ifFalse: [self decodeStreamWithDecoder: streamDecoder].! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 7/14/2010 00:39'!
currentLineNumber
	^ currentLineNumber! !

!XMLNestedStreamReader methodsFor: 'encodings' stamp: 'monty 11/20/2014 00:17'!
decodeStreamWithDecoder: aStreamDecoder
	(stream respondsTo: #streamDecoder:)
		ifTrue: [stream streamDecoder: aStreamDecoder]
		ifFalse: [
			stream :=
				XMLDecodingReadStreamAdapter
					on: stream
					streamDecoder: aStreamDecoder]! !

!XMLNestedStreamReader methodsFor: 'encodings' stamp: 'monty 11/16/2014 05:21'!
detectCurrentEncoding
	| firstChar characterCode selector encodingDetector |

	stream atEnd
		ifTrue: [^ self].
	firstChar := stream peek.
	((characterCode := firstChar asciiValue) < 256
		and: [(selector := EncodingHandlerTable at: characterCode + 1) notNil])
		ifTrue: [
			(encodingDetector := XMLEncodingDetector on: stream)
				perform: selector]
		ifFalse: [
			firstChar == $<
				ifFalse: [^ self].
			peekChar := stream next.
			(stream atEnd
				or: [stream peek asciiValue > 0])
				ifTrue: [^ self].
			(encodingDetector := XMLEncodingDetector on: stream)
				detectLittleEndianImplicitEncoding].
	encodingDetector streamDecoder
		ifNotNil: [:decoder | self decodeStreamWithDecoder: decoder].
	encodingDetector remainder
		ifNotNil: [:remainder | self pushBack: remainder].! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 10/31/2014 21:21'!
errorNestingLimitExceeded
	XMLLimitException signal: 'Cannot further nest input streams'! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/1/2014 15:39'!
errorReadLimitExceeded
	XMLLimitException signal:
		'Security read limit on input exceeded; ',
		'use #documentReadLimit: or #externalSubsetReadLimit: ',
		'to change or disable'! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/17/2014 03:11'!
escapeQuote: aCharacter
	aCharacter ==  $"
		ifTrue: [
			self pushBack: '#34;'.
			^ $&].
	aCharacter == $'
		ifTrue: [
			self pushBack: '#39;'.
			^ $&].
	^ aCharacter.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'monty 1/4/2014 08:07'!
hasNestedStreams
	^ nestedStreams notNil and: [nestedStreams notEmpty]! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 11/15/2014 23:37'!
isEscapingQuotes: aBoolean
	isEscapingQuotes := aBoolean! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/18/2014 05:43'!
maxNestedStreams
	^ 32! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 10/31/2014 15:03'!
next
	| nextChar |

	peekChar
		ifNil: [nextChar := self basicNext]
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil].

	nextChar == normalizedLineEndingChar
		ifTrue: [currentLineNumber := currentLineNumber + 1].
	readLimit ifNotNil: [
		(readLimit := readLimit - 1) >= 0
			ifFalse: [self errorReadLimitExceeded]].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 10/26/2014 14:18'!
nextInteger
	^ XMLIntegerReader
		readFrom: self
		withBase: 16
		startingWithAny: 'x'
		orBase: 10! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 11/18/2014 04:55'!
nextMatchAll: aString
	| nextChar  |

	"inlined peek"
	(peekChar ifNil: [peekChar := self basicNext]) == (aString at: 1)
		ifFalse: [^ false].
	self next.

	"we unrolled the first iteration with the above #peek"
	2 to: aString size do: [:i |
		(nextChar := self peek) == (aString at: i)
			ifFalse: [
				self pushBack: (aString copyFrom: 1 to: i -1).
				^ false].
			self next].
	^ true.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 1/16/2014 09:08'!
normalizedLineEndingChar
	^ normalizedLineEndingChar! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:25'!
normalizedLineEndingChar: aCharacter
	normalizedLineEndingChar := aCharacter! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 2/2/2014 02:43'!
peek
	^ peekChar ifNil: [peekChar := self basicNext]! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'monty 11/17/2014 06:12'!
popCurrentStream
	| nextStreamAndCallback |

	self closeCurrentStream.
	nextStreamAndCallback := nestedStreams removeLast.
	stream := nextStreamAndCallback key.
	onClose := nextStreamAndCallback value.! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 11/16/2014 04:42'!
pushBack: aString
	aString size > 1
		ifTrue: [^ self pushStream: (ReadStream on: aString)].
	aString size = 1
		ifTrue: [self pushBackCharacter: (aString at: 1)]! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 11/20/2014 00:14'!
pushBackCharacter: aCharacter
	self
		pushStream: (XMLSingleCharacterReadStream on: aCharacter)
		onClose: nil! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 11/17/2014 03:37'!
pushStream: aStream
	self
		pushStream: aStream
		onClose: nil! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 11/20/2014 00:14'!
pushStream: aStream onClose: aBlock
	nestedStreams
		ifNil: [(nestedStreams := OrderedCollection new: self maxNestedStreams)].
	(nestedStreams size >= self maxNestedStreams)
		ifTrue: [self errorNestingLimitExceeded].

	nestedStreams addLast: stream -> onClose.
	peekChar
		ifNotNil: [
			nestedStreams addLast: (XMLSingleCharacterReadStream on: peekChar) -> nil.
			peekChar := nil].

	stream := aStream.
	onClose := aBlock.! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'monty 11/15/2014 23:34'!
setStream: anInitialStream readLimit: aReadLimit
	writeBuffer := WriteStream on: (String new: 64).
	currentLineNumber := 1.
	normalizedLineEndingChar := LineFeed.
	stream := anInitialStream.
	readLimit := aReadLimit.
	isEscapingQuotes := false.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'monty 2/10/2014 06:39'!
skipSeparators
	"inlined peek"
	[(peekChar ifNil: [peekChar := self basicNext]) notNil
		and: [peekChar isSeparator]]
		whileTrue: [self next].! !

!XMLNestedStreamReader methodsFor: 'positioning' stamp: 'monty 1/4/2014 08:09'!
skipTo: aDelimiter
	[self atEnd] whileFalse: [
		self next == aDelimiter
			ifTrue: [^ true]].
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 08:01'!
stream
	^ stream! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/22/2011 11:49'!
upTo: aDelimiter
	| nextChar |

	writeBuffer reset.
	[self atEnd or: [(nextChar := self next) == aDelimiter]]
		whileFalse: [writeBuffer nextPut: nextChar].

	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAA 3/22/2011 11:49'!
upToAll: aDelimitingString
	"Answer a subcollection from the current access position to the occurrence
	(if any, but not inclusive) of delimitingString. If delimitingString is not
	in the stream, answer the entire rest of the stream."

	writeBuffer reset.
	[self atEnd or: [self nextMatchAll: aDelimitingString]]
		whileFalse: [writeBuffer nextPut: self next].
	^ writeBuffer contents.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'monty 11/13/2013 02:59'!
upToEnd
	writeBuffer reset.
	[self atEnd] whileFalse: [writeBuffer nextPut: self next].
	^ writeBuffer contents.! !

Object subclass: #XMLNode
	instanceVariableNames: 'parent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNode commentStamp: '<historical>' prior: 0!
This is a base class for XML nodes. It contains testing messages and methods to set and access the parent node and sibling nodes. It also provides a number messages to control printing.!

XMLNode subclass: #XMLAttribute
	instanceVariableNames: 'name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttribute commentStamp: 'monty 2/8/2014 02:27' prior: 0!
The class represents an attribute node, to be stored in an attribute node list. You generally won't need to interact with these objects directly; the various #attribute- messages element nodes respond to will normally suffice. The name of an attribute node can be namespaced using prefixes; however, the prefix must be mapped to a namespace URI in the containing element node. Unprefixed attributes have no namespace, not even a default namespace URI if one is in scope, per the XML namespace standard.!

!XMLAttribute class methodsFor: 'instance creation' stamp: 'monty 10/18/2014 04:03'!
name: aName value: aValue
	^ self new	
		setName: aName
		value: aValue
		element: nil! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/24/2014 05:04'!
configuration
	^ self element ifNotNil: [:element | element configuration]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/24/2014 05:06'!
document
	^ self element ifNotNil: [:element | element document]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 10/29/2014 07:42'!
element
	"reuse instance variable to save memory"
	^ parent! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 10/29/2014 07:42'!
element: anElement
	"reuse instance variable to save memory"
	parent := anElement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 19:44'!
expandedName
	^ self localName expandWithNamespaceURI: self namespaceURI! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 11/22/2014 19:58'!
hasPrefix
	^ self name hasPrefixBeforeLocalName! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 11/16/2014 17:28'!
isNamed: aName
	^ aName
		isQualifiedName: self name
		orLocalName: self localName! !

!XMLAttribute methodsFor: 'testing' stamp: 'monty 11/16/2014 18:10'!
isNamedAny: aNameCollection
	^ aNameCollection
		includesQualifiedName: self name
		orLocalName: self localName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 20:21'!
localName
	^ self name localNameAfterPrefix! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 17:36'!
name
	^ name ifNil: [name := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 18:01'!
name: aName
	self renameWith: [name := aName]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/22/2014 19:52'!
namespaceURI
	| prefix |

	"attributes must have prefixes to be namespaced"
	(self element notNil
		and: [(prefix := self prefix) notEmpty])
		ifFalse: [^ ''].
	
	^ self element resolvePrefix: prefix.! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/24/2014 18:36'!
nextNode
	^ self element ifNotNil: [:element | element attributeNodes after: self]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/6/2014 12:09'!
parent
	"attributes have no parents and aren't part of the DOM tree"
	self shouldNotImplement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/6/2014 12:09'!
parent: aParentNode
	"attributes have no parents and aren't part of the DOM tree"
	self shouldNotImplement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 19:41'!
prefix
	^ self name prefixBeforeLocalName: self localName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/16/2014 19:50'!
prefix: aPrefix
	self renameWith: [name := self localName qualifyWithPrefix:  aPrefix]! !

!XMLAttribute methodsFor: 'accessing' stamp: 'monty 11/24/2014 18:36'!
previousNode
	^ self element ifNotNil: [:element | element attributeNodes before: self]! !

!XMLAttribute methodsFor: 'private' stamp: 'monty 10/29/2014 07:39'!
renameWith: aBlock
	| oldName |

	oldName := self name.
	aBlock value.
	(self name ~= oldName
		and: [self element notNil])
			ifTrue: [
				self element
					renamedAttribute: self
					from: oldName
					to: self name].! !

!XMLAttribute methodsFor: 'initialization' stamp: 'monty 11/16/2014 17:35'!
setName: aName value: aValue element: anElement
	name := aName.
	value := aValue.
	parent := anElement.! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
value
	^ value ifNil: [value := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 08:18'!
value: aString
	value := aString! !

!XMLAttribute methodsFor: 'printing' stamp: 'monty 2/9/2014 04:31'!
writeXMLOn: aWriter
	aWriter
		writeName: self name
		escapedContentValue: self value! !

XMLNode subclass: #XMLComment
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLComment commentStamp: '<historical>' prior: 0!
This class represents comments preserved as nodes. Comments are ignored during parsing by default. If you want them preserved as comment nodes, use #preservesCommentNodes: with the DOM parser before parsing.!

!XMLComment class methodsFor: 'instance creation' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	^ self new string: aString! !

!XMLComment methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ true! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string
	^ string ifNil: [string := '']! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	string := aString! !

!XMLComment methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter comment: self string! !

XMLNode subclass: #XMLDeclarationNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDeclarationNode commentStamp: 'monty 10/26/2014 06:12' prior: 0!
This is a generic class to represent markup declarations in the internal subset. Each subclass represents a specific kind of declaration.!

XMLDeclarationNode subclass: #XMLAttributeDeclaration
	instanceVariableNames: 'element attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLAttributeDeclaration commentStamp: 'monty 2/2/2014 16:47' prior: 0!
This is a class for <!!ATTLIST> declarations with definitions for single attributes. (An ATTLIST with multiple attribute defs is the same as a series of separate ATTLISTs for each def)!

!XMLAttributeDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:11'!
element: anElementName attribute: anAttributeName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	^ self new
		element: anElementName;
		attribute: anAttributeName;
		type: aType;
		defaultPragma: aDefaultPragma;
		defaultValue: aDefaultValue! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:53'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:07'!
attribute: anAttributeName
	attribute := anAttributeName! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:14'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
defaultPragma: aDefaultPragma
	defaultPragma := aDefaultPragma! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:11'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:10'!
element: anElementName
	element := anElementName! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
type
	^ type ifNil: [type := '']! !

!XMLAttributeDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:09'!
type: aType 
	type := aType! !

!XMLAttributeDeclaration methodsFor: 'printing' stamp: 'monty 12/31/2013 00:16'!
writeXMLOn: aWriter
	aWriter attributeDeclaration
		element: self element;
		attribute: self attribute;
		type: self type;
		defaultPragma: self defaultPragma;
		defaultValue: self defaultValue! !

!XMLDeclarationNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ true! !

XMLDeclarationNode subclass: #XMLElementDeclaration
	instanceVariableNames: 'contentModel element'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElementDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is a class for <!!ELEMENT> declarations.!

!XMLElementDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:04'!
element: aName contentModel: aContentModel
	^ self new
		element: aName;
		contentModel: aContentModel! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:03'!
contentModel: aContentModel
	contentModel := aContentModel! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:21'!
element
	^ element ifNil: [element := '']! !

!XMLElementDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:03'!
element: aName
	element := aName! !

!XMLElementDeclaration methodsFor: 'printing' stamp: 'monty 12/21/2013 03:25'!
writeXMLOn: aWriter
	aWriter elementDeclaration
		element: self element;
		contentModel: self contentModel! !

XMLDeclarationNode subclass: #XMLEntityDeclaration
	instanceVariableNames: 'name replacement publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLEntityDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is an abstract class for general and parameter <!!ENTITY> declarations.!

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:06'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new
		name: aName;
		publicID: aPublicId;
		systemID: aSystemId! !

!XMLEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:06'!
name: aName replacement: aReplacement
	^ self new
		name: aName;
		replacement: aReplacement! !

!XMLEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/23/2014 19:02'!
isParameter
	^ false! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:01'!
name: aName
	name := aName! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:01'!
publicID: aPublicId
	publicId := aPublicId! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:05'!
replacement: aReplacement
	replacement := aReplacement! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLEntityDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:05'!
systemID: aSystemId
	systemId := aSystemId! !

!XMLEntityDeclaration methodsFor: 'printing' stamp: 'monty 11/13/2013 19:35'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: self isParameter;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		ndata: self ndata! !

XMLEntityDeclaration subclass: #XMLGeneralEntityDeclaration
	instanceVariableNames: 'ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLGeneralEntityDeclaration commentStamp: 'monty 2/2/2014 16:47' prior: 0!
This is a class for internal/external parsed/unparsed general <!!ENTITY> declarations.!

!XMLGeneralEntityDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 06:06'!
name: aName publicID: aPublicId systemID: aSystemId ndata: aNotationName
	^ self new
		name: aName;
		publicID: aPublicId;
		systemID: aSystemId;
		ndata: aNotationName! !

!XMLGeneralEntityDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:54'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLGeneralEntityDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 18:54'!
ndata: anNdata
	ndata := anNdata! !

!XMLGeneralEntityDeclaration methodsFor: 'printing' stamp: 'monty 11/23/2014 18:55'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: false;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		ndata: self ndata! !

XMLEntityDeclaration subclass: #XMLParameterEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLParameterEntityDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is class for internal/external parameter <!!ENTITY> declarations.!

!XMLParameterEntityDeclaration methodsFor: 'testing' stamp: 'monty 11/13/2013 19:34'!
isParameter
	^ true! !

!XMLParameterEntityDeclaration methodsFor: 'printing' stamp: 'monty 11/23/2014 18:55'!
writeXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		isParameter: true;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID! !

XMLDeclarationNode subclass: #XMLNotationDeclaration
	instanceVariableNames: 'name publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNotationDeclaration commentStamp: 'monty 2/2/2014 16:46' prior: 0!
This is class for <!!NOTATION> declarations.
!

!XMLNotationDeclaration class methodsFor: 'instance creation' stamp: 'monty 10/26/2014 10:34'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new
		name: aName;
		publicID: aPublicId;
		systemID: aSystemId! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:22'!
name
	^ name ifNil: [name := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 10:35'!
name: aName
	name := aName! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:00'!
publicID: aPublicId
	publicId := aPublicId! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLNotationDeclaration methodsFor: 'accessing' stamp: 'monty 10/26/2014 06:00'!
systemID: aSystemId
	systemId := aSystemId! !

!XMLNotationDeclaration methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter notationDeclaration
		name: self name;
		publicID: self publicID;
		systemID: self systemID! !

!XMLNode methodsFor: 'private' stamp: 'monty 2/11/2014 00:13'!
acceptsContent
	^ false! !

!XMLNode methodsFor: 'adding' stamp: 'JAAyer 7/20/2010 10:15'!
addContent: aStringOrStringNode
	XMLDOMException signal: 'Node cannot contain content'! !

!XMLNode methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:59'!
allNodesDo: aBlock
	aBlock value: self! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 11/6/2014 23:57'!
ancestorElements
	| ancestorElements |

	ancestorElements := self newNodeListForSelect.
	self ancestorElementsDo: [:each | ancestorElements addLast: each].
	^ ancestorElements.! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:07'!
ancestorElementsDo: aBlock
	self parent
		ifNotNil: [:node |
			node isElement
				ifTrue: [aBlock value: node].
			node ancestorElementsDo: aBlock]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 11/2/2014 00:03'!
ancestorNodes
	| ancestors |

	ancestors := self newNodeListForSelect.
	self ancestorNodesDo: [:each | ancestors addLast: each].
	^ ancestors.! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 11/6/2014 16:57'!
ancestorNodesDo: aBlock
	self parent
		ifNotNil: [:node |
			aBlock value: node.
			node ancestorNodesDo: aBlock]! !

!XMLNode methodsFor: 'converting' stamp: 'monty 2/6/2014 20:06'!
asString
	"for compatibility if Object>>asString isn't defined"
	^ self printString! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/23/2014 20:53'!
canHaveChildren
	^ false! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/19/2014 23:49'!
canonicallyPrintOn: aStream
 	"Prints on aStream as canonical XML"
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer
										isCanonical: true;
										writeWith: [ :wr | self writeXMLOn: wr ] ]
		whenAbsent: [self printOn: aStream]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/19/2014 23:49'!
canonicallyPrinted
 	"Prints as canonical XML"
	^ String streamContents: [:stream | self canonicallyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 11/2/2014 19:15'!
configuration
	^ self document ifNotNil: [:node | node configuration]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 11/20/2014 02:36'!
contentString
 	"This returns the combined content string of all descendants with spaces inserted where necessary."
	^ String streamContents: [:stream | self printContentOn: stream]! !

!XMLNode methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:30'!
descendentNodesDo: aBlock! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 10/30/2014 22:39'!
document
	^ self parent ifNotNil: [:node | node document]! !

!XMLNode methodsFor: 'private' stamp: 'monty 10/31/2014 20:56'!
errorXMLWritingUnsupported
	XMLDOMException signal: 'The XMLWriter package is required for writng DOM objects'! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
hasChildren
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 10/29/2014 09:00'!
hasParent
	^ parent notNil! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/30/2010 19:19'!
isContentNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 12/22/2013 06:10'!
isDeclaration
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:50'!
isDocument
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/5/2014 11:36'!
isElementNamed: aName
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/5/2014 11:36'!
isElementNamedAny: aNameCollection
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:33'!
isStringNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'monty 11/5/2014 12:09'!
isStringNode: aStringOrStringNode
	^ false! !

!XMLNode methodsFor: 'private' stamp: 'monty 11/7/2014 02:17'!
newListForCollect
	^ XMLOrderedList new! !

!XMLNode methodsFor: 'private' stamp: 'monty 10/28/2014 19:41'!
newNodeListForSelect
	^ XMLNodeList new! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 10/29/2014 12:54'!
nextNode
	^ self parent ifNotNil: [:node | node nodeAfter: self]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent
	^ parent! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent: aParentNode
	parent := aParentNode! !

!XMLNode methodsFor: 'copying' stamp: 'monty 10/29/2014 07:40'!
postCopy
	parent := nil! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:07'!
prettyPrintOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer
									enablePrettyPrinting;
									writeWith: [ :wr | self writeXMLOn: wr ] ]
		whenAbsent: [ self printOn: aStream ]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:18'!
prettyPrinted
	^ String streamContents: [:stream | self prettyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 10/29/2014 12:55'!
previousNode
	^ self parent ifNotNil: [:node | node nodeBefore: self]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/20/2014 00:23'!
printContentOn: aStream
 	"Prints on aStream the content strings of all descendants with spaces inserted where necessary."
	self printRawContentOn: (XMLNodeContentWriteStream on: aStream)! !

!XMLNode methodsFor: 'printing' stamp: 'DamienCassou 4/30/2013 17:41'!
printOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer | writer writeWith: [ :wr | self writeXMLOn: wr] ]
		whenAbsent: [ super printOn: aStream ]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 11/19/2014 23:54'!
printRawContentOn: aStream 
 	"Prints on aStream the content strings of all descendants without spaces inserted."! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:26'!
printWithoutSelfClosingTagsOn: aStream
	self
		withNewXMLWriterOn: aStream
		do: [ :writer |
			writer
				outputsSelfClosingTags: false;
				writeWith: [ :wr | self writeXMLOn: wr ]]
		whenAbsent: [ self printOn: aStream ]! !

!XMLNode methodsFor: 'printing' stamp: 'monty 10/12/2014 21:27'!
printedWithoutSelfClosingTags
	^ String streamContents: [:stream | self printWithoutSelfClosingTagsOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'monty 11/19/2014 23:44'!
rawContentString
 	"This returns the combined content string of all descendants without spaces inserted."
	^ String streamContents: [:stream | self printRawContentOn: stream].! !

!XMLNode methodsFor: 'private' stamp: 'monty 10/31/2014 20:56'!
withNewXMLWriterOn: aStream do: aOneArgBlock
	^ self
		withNewXMLWriterOn: aStream
		do: aOneArgBlock
		whenAbsent: [self errorXMLWritingUnsupported]! !

!XMLNode methodsFor: 'private' stamp: 'monty 10/31/2014 20:59'!
withNewXMLWriterOn: aStream do: aOneArgBlock whenAbsent: aZeroArgBlock
	^ aOneArgBlock value:
		((Smalltalk
			at: #XMLWriter
			ifAbsent: [^ aZeroArgBlock value]) on: aStream)! !

!XMLNode methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	self subclassResponsibility! !

XMLNode subclass: #XMLNodeWithChildren
	instanceVariableNames: 'nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithChildren commentStamp: '<historical>' prior: 0!
This class represents a node that can contain child nodes. You probably will only use its API through one its subclasses, specifically the document, element and doctype declaration node classes.

This class provides message to access, add and remove child nodes under the "accessing" category. The nodes themselves are stored in some type of XMLObservableList, the exact type of which varies depending on the subclass and can be accessed directly using #nodes. You can modify the #nodes list directly if you like, but adding nodes to it or removing nodes from it will have the effect of adding them to removing them from the parent node. If this is not what you want, then make sure you copy the return value of #nodes before manipulating it.

The "enumerating" category provides two different modes of enumeration: the #nodes* enumerating messages enumerate child nodes of the receiver only, while the #allNode* forms recursively enumerate all descendent nodes, including the receiver itself, using depth-first traversal.

The inner XML of a node with children can be accessed as a string using #innerXML and set using #innerXML:.!

XMLNodeWithChildren subclass: #XMLDoctypeDeclaration
	instanceVariableNames: 'root publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDoctypeDeclaration commentStamp: '<historical>' prior: 0!
This class represents the <!!DOCTYPE ...> declaration one often finds at the top of an XML document. It stores the root element name, formal public identifier (FPI) if any, system identifier (a URI) and then any nodes comprising the internal subset.!

!XMLDoctypeDeclaration class methodsFor: 'instance creation' stamp: 'monty 11/23/2014 04:43'!
root: aRootElementName publicID: aPublicId systemID: aSystemId
	^ self new
		root: aRootElementName;
		publicID: aPublicId;
		systemID: aSystemId! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'monty 11/23/2014 22:09'!
newNodeListForSelect
	^ XMLInternalSubsetList new! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'monty 11/26/2013 21:51'!
nodeListClass
	^ XMLInternalSubsetList! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 10/10/2014 07:12'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 04:42'!
publicID: aPublicId
	publicId := aPublicId! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 10/10/2014 07:12'!
root
	^ root ifNil: [root := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 04:42'!
root: aRootElementName
	root := aRootElementName! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 10/10/2014 07:13'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'monty 11/23/2014 04:42'!
systemID: aSystemId
	systemId := aSystemId! !

!XMLDoctypeDeclaration methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	aWriter doctype
		root: self root;
		publicID: self publicID;
		systemID: self systemID;
		withInternalSubset: [self writeInnerXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 7/20/2010 10:22'!
addNode: aNode
	^ self nodes addLast: aNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode after: aPriorNode
	^ self nodes add: aNode after: aPriorNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode before: aSubsequentNode
	^ self nodes add: aNode before: aSubsequentNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 7/20/2010 10:22'!
addNodeFirst: aNode
	^ self nodes addFirst: aNode! !

!XMLNodeWithChildren methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 05:12'!
addNodes: aNodeCollection
	^ self nodes addAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'monty 2/2/2014 02:40'!
addedNode: aNode
	aNode parent: self! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'monty 10/31/2014 17:34'!
addingNode: aNode
	aNode hasParent
		ifTrue: [self errorAddingNodeWithParent]! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/28/2014 19:47'!
allNodes
	| allNodes |

	allNodes := self newNodeListForSelect.
	self allNodesDo: [:each | allNodes addLast: each].
	^ allNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/7/2014 02:17'!
allNodesCollect: aBlock
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self allNodesDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:17'!
allNodesDetect: aBlock
	^ self allNodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:40'!
allNodesDetect: aBlock ifNone: anExceptionBlock
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/19/2014 08:44'!
allNodesDo: aBlock
	aBlock value: self.
	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:49'!
allNodesSelect: aBlock
	| selectedNodes |

	selectedNodes := self newNodeListForSelect.
	self allNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes addLast: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'monty 11/23/2014 20:53'!
canHaveChildren
	^ true! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/6/2014 21:25'!
descendentNodes
	| descendentNodes |

	descendentNodes := self newNodeListForSelect.
	self descendentNodesDo: [:each | descendentNodes addLast: each].
	^ descendentNodes.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/7/2014 02:18'!
descendentNodesCollect: aBlock
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self descendentNodesDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:37'!
descendentNodesDetect: aBlock
	^ self descendentNodesDetect: aBlock ifNone: [nil]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:40'!
descendentNodesDetect: aBlock ifNone: anExceptionBlock
	self descendentNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/7/2014 01:55'!
descendentNodesDo: aBlock
	self hasChildren
		ifTrue: [self nodesDo: [:each | each allNodesDo: aBlock]]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 11/6/2014 21:33'!
descendentNodesSelect: aBlock
	| selectedNodes |

	selectedNodes := self newNodeListForSelect.
	self descendentNodesDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedNodes addLast: each]].
	^ selectedNodes.! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 10/31/2014 17:34'!
errorAddingNodeWithParent
	XMLDOMException signal: 'Cannot add child node that already has a parent'! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:52'!
firstNode
	"returns nil when empty"
	^ self nodes first! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'monty 11/22/2014 07:51'!
hasChildren
	"Direct variable access avoids possibly unnecessary lazy initialization of the node list"

	^ nodes notNil and: [nodes size > 0]! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:23'!
includesNode: aNode
	^ self nodes includes: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:51'!
innerXML
	self hasChildren
		ifFalse: [^ ''].
	self
		withNewXMLWriterOn: (String new: self nodes size * 10) writeStream
		do: [ :writer |
			self writeInnerXMLOn: writer.
			^ writer contents].! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:52'!
lastNode
	"returns nil when empty"
	^ self nodes last! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:26'!
nodeAfter: aNode
	^ self nodes after: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/17/2014 21:29'!
nodeAt: anIndex
	"returns nil if absent"
	^ self nodes at: anIndex! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/17/2014 16:46'!
nodeAt: anIndex ifAbsent: aBlock
	^ self nodes at: anIndex ifAbsent: aBlock! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 11/17/2014 16:47'!
nodeAt: anIndex put: aNode
	^ self nodes at: anIndex put: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
nodeBefore: aNode
	^ self nodes before: aNode! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 11/4/2014 19:04'!
nodeListClass
	^ XMLNodeList! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'monty 10/18/2014 01:00'!
nodes
	^ nodes ifNil: [nodes := self nodeListClass observer: self]! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesCollect: aBlock
	^ self nodes collect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'monty 10/28/2014 19:46'!
nodesDetect: aBlock
	^ self nodes detect: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:14'!
nodesDetect: aBlock ifNone: anExceptionBlock
	^ self nodes detect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 7/20/2010 10:27'!
nodesDo: aBlock
	self nodes do: aBlock! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 11/28/2010 12:13'!
nodesSelect: aBlock
	^ self nodes select: aBlock! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'monty 11/1/2014 15:02'!
parserHandlerClass
	^ XMLDOMParser! !

!XMLNodeWithChildren methodsFor: 'copying' stamp: 'monty 10/27/2014 18:29'!
postCopy
	super postCopy.

	nodes
		ifNotNil: [nodes := nodes deepCopyWithObserver: self].! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNode: aNode
	^ self nodes remove: aNode! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'monty 11/19/2014 20:24'!
removeNode: aNode ifAbsent: aBlock
	^ self nodes remove: aNode ifAbsent: aBlock! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes
	self nodes removeAll! !

!XMLNodeWithChildren methodsFor: 'removing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes: aNodeCollection
	^ self nodes removeAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'monty 2/2/2014 02:38'!
removedNode: aNode
	aNode parent: nil! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 22:39'!
replaceNode: aNode with: aReplacementNode
	^ self nodes
		at: (self nodes indexOf: aNode)
		put: aReplacementNode! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeInnerXMLOn: aWriter
	self nodesDo: [:each | each writeXMLOn: aWriter]! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAA 3/23/2011 09:42'!
writeXMLOn: aWriter
	self writeInnerXMLOn: aWriter! !

XMLNodeWithChildren subclass: #XMLNodeWithElements
	instanceVariableNames: 'configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeWithElements commentStamp: '<historical>' prior: 0!
This class represents a node with elements. You will likely use its API through its subclasses instead.

Instances provide "accessing" messages to retrieve child elements by their name and namespace information. The #elementAt: forms return the first matching element, while the #elementsAt: forms return all matching child elements.

As with node enumeration in the superclass, there are two different modes of enumeration: the #elements* enumerating messages that enumerate child elements only, and the #allElements* forms that enumerate all descendent elements, including the receiver if it is an element (but not if it is a document), using depth-first traversal.

The #findElementNamed:* forms can be used to search using depth-first traversal for a specific element. Searching will start with the receiver if it is an element.

All element name matching is done using both the qualified and local name of elements, so 'prefix:element-name' will only match 'prefix:element-name' while 'element-name' will match 'element-name', 'prefix:element-name' or 'different-prefix:element-name' and so on.!

XMLNodeWithElements subclass: #XMLDocument
	instanceVariableNames: 'version encoding doctypeDeclaration isStandalone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLDocument commentStamp: '<historical>' prior: 0!
This class represents a document node, which is often the root of a node tree. It can contain XML version and encoding information, a DOCTYPE declaration with a possible internal subset and a root element child node.!

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration
	^ doctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration: aDoctypeDeclaration
	doctypeDeclaration := aDoctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/24/2010 13:34'!
document
	^ self! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:03'!
encoding	
	^ encoding ifNil: [encoding := 'UTF-8']! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
encoding: aString	
	encoding := aString! !

!XMLDocument methodsFor: 'private' stamp: 'monty 11/17/2014 02:02'!
innerXMLContext
	^ XMLPrologContext new! !

!XMLDocument methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:51'!
isDocument
	^ true! !

!XMLDocument methodsFor: 'testing' stamp: 'monty 11/19/2013 05:38'!
isStandalone
	^ isStandalone ifNil: [isStandalone := false]! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:40'!
isStandalone: aBoolean
	isStandalone := aBoolean! !

!XMLDocument methodsFor: 'copying' stamp: 'monty 11/22/2014 08:20'!
postCopy
	super postCopy.

	doctypeDeclaration
		ifNotNil: [doctypeDeclaration := doctypeDeclaration copy].! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 15:28'!
root
	^ self firstElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'monty 12/6/2013 06:33'!
root: anElement
	self
		removeNodes: self elements;
		addNode: anElement! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
version	
	^ version! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 19:39'!
version: aVersionNumber
	version := aVersionNumber! !

!XMLDocument methodsFor: 'printing' stamp: 'monty 11/19/2013 05:47'!
writeXMLOn: aWriter
	self version ifNotNil: [
		aWriter xml
			version: self version;
			encoding: self encoding;
			standalone: self isStandalone].
	self doctypeDeclaration
		ifNotNil: [self doctypeDeclaration writeXMLOn: aWriter].
	
	self writeInnerXMLOn: aWriter.! !

XMLNodeWithElements subclass: #XMLElement
	instanceVariableNames: 'name localName attributes namespaceScope'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLElement commentStamp: '<historical>' prior: 0!
The class represents an element node. An element consists of a name and optionally attributes, namespace declarations and any number child nodes and elements.

The name can consist of a local name only or a qualified name, which is a namespace prefix and local name separated by a colon. If the name is qualfied and namespace support is enabled (the default), then the prefix must be mapped to a namespace URI in the element or its parent at the time the name is set. The class-side instance creation #name:namespaceURI:* messages and #name:namespaces:* messages allow you set both simulaniously, as does the instance-side #name:namespaceURI: message. If namespace support is disabled, prefixes are ignored (not validated). You can test the name of an element using #isNamed: and #isNamedAny:, which test the qualified and local names of an element for a match.

Namespaces can be declared using #declareNamespace:uri:, using #declareNamespaces: with a collection of associations (where an empty prefix represents the default namespace) or simultaniously while changing the element's name using #name:namespaceURI:. The default namespace can be accessed through #defaultNamespace and #defaultNamespace:. When you print an element, the minimum number of xmlns declarations necessary for it to be valid will be printed. You can query an element's namespace information using #namespaceURI, #prefix, #expandedName and other messages under "namespaces."" For an element to be namespaced, it must have a qualified name with a valid prefix or have a default namespace declaration in scope for it.

The #attribute- messages provide a Dictionary-like prototcol for manipulating attribute nodes. Attribute value accessors return empty strings when the specified value is not present. The underlying attribute node list can be accessed using #attributeNodes, and an order-preserving dictionary of attribute names/values can be obtained using #attributes.

See the comment for the superclass for how to access child nodes and elements.!

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 10/30/2010 22:09'!
name: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:05'!
name: aLocalName attributes: aDictionaryOrArray
	^ (self named: aLocalName)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri
	^ self new name: aQualifiedName namespaceURI: aUri! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri attributes: aDictionaryOrArray
	^ (self name: aQualifiedName namespaceURI: aUri)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray
	^ self new
		declareNamespaces: aNamespaceDictionaryOrArray;
		name: aQualifiedName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray attributes: anAttributeDictionaryOrArray
	^ (self
		name: aQualifiedName
		namespaces: aNamespaceDictionaryOrArray)
			setAttributes: anAttributeDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 23:10'!
named: aLocalName
	^ self new name: aLocalName! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 10/18/2014 00:39'!
addedAttribute: anAttribute
	anAttribute element: self.
	self validateAttributes.! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 2/2/2014 04:13'!
addedElement: anElement
	super addedElement: anElement.
	(self hasNamespaces
		and: [self namespaceScope ~= anElement namespaceScope])
		ifTrue: [anElement inheritNamespacesFrom: self]! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 10/31/2014 17:35'!
addingAttribute: anAttribute
	anAttribute element
		ifNotNil: [self errorAddingAttributeWithElement]! !

!XMLElement methodsFor: 'enumerating' stamp: 'monty 11/10/2014 21:05'!
allElementsDo: aBlock
	aBlock value: self.
	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]].! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/6/2014 15:43'!
attributeAssociations
	^ self attributeNodes attributeAssociations! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:52'!
attributeAt: aName
	"returns empty string when absent"
	^ self attributeNodes attributeValueAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:34'!
attributeAt: aName put: aValue
	^ self attributeNodes attributeValueAt: aName put: aValue! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/28/2014 18:38'!
attributeListClass
	^ XMLAttributeList! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self attributeNodes attributeNames! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self attributeNodes attributeNamesAndValuesDo: aTwoArgumentBlock! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self attributeNodes attributeNamesDo: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName
	^ self attributeNodes attributeNodeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:27'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aUri! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:28'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self attributeNodes attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/27/2014 18:23'!
attributeNodes
	^ attributes ifNil: [attributes := self attributeListClass observer: self]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:16'!
attributes
	^ self attributeNodes asDictionary! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/10/2014 05:56'!
declareNamespaces: aDictionaryOrArray
	aDictionaryOrArray associationsDo: [:each |
		self
			declarePrefix: each key
			uri: each value]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/17/2014 16:45'!
declarePrefix: aPrefix uri: aUri
	| oldMapping |

	(oldMapping := self namespaceScope prefixMappingAt: aPrefix ifAbsent: [nil]) = aUri
		ifFalse: [
			self
				updateEnclosedScopesWith: [:scope | scope mapPrefix: aPrefix to: aUri]
				whileTrue: [:scope |
					(scope includesPrefix: aPrefix) not
						or: [scope includesPrefix: aPrefix mappedTo: oldMapping]]
				validateAfter: false]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/10/2014 06:14'!
declaresDefaultNamespace
	"Did the receiver declare a new default namespace rather than inheriting one?"

	^ self namespaceScope hasDefaultNamespace
		and: [self isRoot or: [self defaultNamespace ~= self parent defaultNamespace]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 10/10/2014 06:38'!
declaresPrefix: aPrefix uri: aUri
	"Did the receiver declare a new prefix/uri pair rather than inheriting one?"

	^ (self isDeclaredPrefix: aPrefix uri: aUri)
		and: [self isRoot or: [(self parent isDeclaredPrefix: aPrefix uri: aUri) not]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
defaultNamespace
	^ self namespaceScope defaultNamespace! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/4/2014 00:28'!
defaultNamespace: aUri
	| oldNamespace |

	oldNamespace := self defaultNamespace.
	self
		updateEnclosedScopesWith: [:scope | scope defaultNamespace: aUri]
		whileTrue: [:scope | scope defaultNamespace = oldNamespace]
		validateAfter: false.! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/31/2014 17:35'!
errorAddingAttributeWithElement
	XMLDOMException signal: 'Cannot add attribute node from other element'! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/16/2014 19:44'!
expandedName
	^ self localName expandWithNamespaceURI: self namespaceURI! !

!XMLElement methodsFor: 'testing' stamp: 'monty 10/14/2014 07:51'!
hasAttributes
	^ attributes notNil and: [attributes notEmpty]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 10/16/2010 06:07'!
hasNamespaces
	^ namespaceScope notNil and: [namespaceScope hasNamespaces]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/16/2014 17:30'!
hasPrefix
	^ self name size > self localName size! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 12/6/2010 21:23'!
includesAttribute: aName
	^ self attributeNodes includesAttribute: aName.! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/10/2014 23:56'!
inheritNamespacesFrom: anElement
	self allElementsDo: [:each |
		each namespaceScope inheritMappingsFrom:
			anElement namespaceScope]! !

!XMLElement methodsFor: 'private' stamp: 'monty 11/17/2014 02:02'!
innerXMLContext
	^ XMLContentContext new! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
isDeclaredPrefix: aPrefix
	^ self namespaceScope includesPrefix: aPrefix! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/24/2010 12:54'!
isDeclaredPrefix: aPrefix uri: aUri
	^ self namespaceScope includesPrefix: aPrefix mappedTo: aUri! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ true! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/5/2014 11:37'!
isElementNamed: aName
	^ self isNamed: aName! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/5/2014 11:37'!
isElementNamedAny: aNameCollection
	^ self isNamedAny: aNameCollection! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/16/2014 17:28'!
isNamed: aName
	"Does the receiver's qualified or local name match the argument?'"
	^ aName
		isQualifiedName: self name
		orLocalName: self localName! !

!XMLElement methodsFor: 'testing' stamp: 'monty 11/16/2014 18:09'!
isNamedAny: aNameCollection
	"Does the receiver's qualified or local name match any of the names supplied as the argument?'"
	^ aNameCollection
		includesQualifiedName: self name
		orLocalName: self localName! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 11/26/2010 12:03'!
isRoot
	"Is the receiver the root element?"
	^ (self hasParent and: [self parent isElement]) not! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 20:21'!
localName
	^ localName ifNil: [localName := self name localNameAfterPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 17:36'!
name
	^ name ifNil: [name := '']! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 11/16/2014 20:21'!
name: aName
	self renameWith: [
		name := aName.
		localName := aName localNameAfterPrefix]! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/16/2014 20:23'!
name: aQualifiedName namespaceURI: aUri
	self
		declarePrefix: (aQualifiedName prefixBeforeLocalName) uri: aUri;
		name: aQualifiedName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:10'!
namespaceScope
	^ namespaceScope ifNil: [namespaceScope := XMLNamespaceScope new]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/14/2010 12:52'!
namespaceURI
	^ self resolvePrefix: self prefix! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:48'!
nextElement
	^ self parent ifNotNil: [:node | node elementAfter: self]! !

!XMLElement methodsFor: 'copying' stamp: 'monty 11/22/2014 08:20'!
postCopy
	super postCopy.

	namespaceScope
		ifNotNil: [namespaceScope := namespaceScope copy].
	attributes
		ifNotNil: [attributes := attributes deepCopyWithObserver: self].! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/16/2014 19:41'!
prefix
	^ self name prefixBeforeLocalName: self localName! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/16/2014 19:50'!
prefix: aPrefix
	self renameWith: [name := self localName qualifyWithPrefix:  aPrefix]! !

!XMLElement methodsFor: 'accessing' stamp: 'monty 10/29/2014 09:48'!
previousElement
	^ self parent ifNotNil: [:node | node elementBefore: self]! !

!XMLElement methodsFor: 'removing' stamp: 'JAAyer 12/6/2010 21:24'!
removeAttribute: aName
	^ (self attributeNodes removeAttributeNodeAt: aName) value.! !

!XMLElement methodsFor: 'removing' stamp: 'monty 11/23/2014 00:55'!
removeAttribute: aName ifAbsent: aBlock
	^ (self attributeNodes
		removeAttributeNodeAt: aName
		ifAbsent: [^ aBlock value]) value.! !

!XMLElement methodsFor: 'removing' stamp: 'JAAyer 7/22/2010 08:54'!
removeAttributeNode: aNode
	^ self attributeNodes remove: aNode! !

!XMLElement methodsFor: 'removing' stamp: 'monty 11/23/2014 00:55'!
removeAttributeNode: aNode ifAbsent: aBlock
	^ self attributeNodes remove: aNode ifAbsent: aBlock! !

!XMLElement methodsFor: 'removing' stamp: 'monty 11/23/2014 00:29'!
removeAttributes
	self attributeNodes removeAll! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 10/18/2014 00:43'!
removedAttribute: anAttribute
	anAttribute element: nil! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/18/2014 01:35'!
renameWith: aBlock
	| oldName |

	oldName := self name.
	aBlock value.
	self name = oldName
		ifFalse: [
			self validateName.
			self hasParent
				ifTrue: [
					self parent
						renamedElement: self
						from: oldName
						to: self name]].! !

!XMLElement methodsFor: 'notifications' stamp: 'monty 10/18/2014 05:56'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributeNodes	
		renamedAttribute: anAttribute
		from: anOldName
		to: aNewName.
	self validateAttributes.! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
resolvePrefix: aPrefix
	^ self namespaceScope resolvePrefix: aPrefix! !

!XMLElement methodsFor: 'initialization' stamp: 'JAAyer 7/12/2010 14:40'!
setAttributes: aDictionaryOrArray
	self attributeNodes removeAll.
	aDictionaryOrArray associationsDo: [:each |
		self attributeNodes
			addName: each key
			value: each value].! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 12/27/2013 03:42'!
setNamespaceScope: aNamespaceScope
	namespaceScope := aNamespaceScope! !

!XMLElement methodsFor: 'initialization' stamp: 'monty 11/16/2014 17:37'!
setQualifiedName: aQualifiedName localName: aLocalName attributes: anAttributeDictionary
	name := aQualifiedName.
	localName := aLocalName.

	anAttributeDictionary isEmpty "avoid ifNotEmpty: closure overhead"
		ifFalse: [
			"The setFrom method is used to avoid revalidating the
			attributes when adding them to the list"
			(attributes :=
				(self attributeListClass new: anAttributeDictionary size)
					observer: self;
					setFromDictionary: anAttributeDictionary)].! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/24/2014 06:54'!
undeclareDefaultNamespace
	self undeclarePrefix: ''! !

!XMLElement methodsFor: 'namespaces' stamp: 'monty 11/17/2014 16:45'!
undeclarePrefix: aPrefix
	| currentMapping |

	currentMapping := self namespaceScope prefixMappingAt: aPrefix ifAbsent: [^ self].
	self
		updateEnclosedScopesWith: [:scope | scope unmapPrefix: aPrefix]
		whileTrue: [:scope | scope includesPrefix: aPrefix mappedTo: currentMapping]
		validateAfter: true.! !

!XMLElement methodsFor: 'private' stamp: 'monty 11/4/2014 00:29'!
updateEnclosedScopesWith: aBlock whileTrue: aConditionalBlock validateAfter: aBoolean
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true. It stops descending down a path of a subtree
	as soon as aConditional is false, but continues down other paths."

	(aConditionalBlock value: self namespaceScope)
		ifFalse: [^ self].

	aBlock value: self namespaceScope.
	aBoolean
		ifTrue: [
			self
				validateName;
				validateAttributes].
	self elementsDo: [:each |
		each
			updateEnclosedScopesWith: aBlock
			whileTrue: aConditionalBlock
			validateAfter: aBoolean]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/9/2014 21:08'!
validateAttributes
	self usesNamespaces
		ifTrue: [
			self namespaceScope
				validateAttributePrefixes: self attributeNodes attributesByName]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/9/2014 06:42'!
validateName
	self usesNamespaces
		ifTrue: [
			self namespaceScope
				validateAndResolvePrefix: self prefix
				ofElement: self name]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/12/2014 00:58'!
writeAttributesOn: aTagWriter
	self attributeNodes do: [:each |
		aTagWriter
			attributeAt: each name
			put: each value]! !

!XMLElement methodsFor: 'private' stamp: 'monty 10/12/2014 00:58'!
writeNamespaceDeclarationsOn: aTagWriter
	self namespaceScope prefixesAndURIsDo: [:prefix :uri |
		((self declaresPrefix: prefix uri: uri)
			or: [aTagWriter isEmbedded not])
			ifTrue: [
				aTagWriter
					xmlnsAt: prefix
					put: uri]]! !

!XMLElement methodsFor: 'printing' stamp: 'monty 11/21/2014 23:31'!
writeXMLOn: aWriter
	| tagWriter |

	tagWriter := aWriter tag: self name.

	self hasNamespaces
		ifTrue: [self writeNamespaceDeclarationsOn: tagWriter].
	self hasAttributes
		ifTrue: [self writeAttributesOn: tagWriter].

	tagWriter with: [
		self hasChildren
			ifTrue: [self writeInnerXMLOn: aWriter]].! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 11/1/2014 22:58'!
addContent: aStringOrStringNode
	"Adds a string or string node. If the last child node is already a string node,
	the characters being added are concatenated to it."

	(self hasChildren and: [self lastNode acceptsContent])
		ifTrue: [self lastNode addContent: aStringOrStringNode]
		ifFalse: [self addNode:
			(aStringOrStringNode isString
				ifTrue: [self nodeFactory newString: aStringOrStringNode]
				ifFalse: [aStringOrStringNode])]! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 11/1/2014 21:06'!
addElement: anElement
	self deprecated: 'use #addNode: or #addElementNamed: instead'.
	self addNode: anElement.! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 11/6/2014 12:27'!
addElementNamed: aName
	"element is added first before naming so it inherits namespaces from self"
	^ (self addNode: (self nodeFactory classForElement: aName) new)
		name: aName! !

!XMLNodeWithElements methodsFor: 'adding' stamp: 'monty 11/6/2014 12:27'!
addElementNamed: aName namespaceURI: aUri
	^ self addNode:
		(self nodeFactory
			newElementNamed: aName
			namespaceURI: aUri)! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'monty 10/28/2014 17:59'!
addedElement: anElement
	self addedNode: anElement.
	anElement configuration: self configuration.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/28/2014 19:47'!
allElements
	| allElements |

	allElements := self newNodeListForSelect.
	self allElementsDo: [:each | allElements addLast: each].
	^ allElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 02:18'!
allElementsCollect: aBlock
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self allElementsDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 10:47'!
allElementsDetect: aBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	^ self allElementsDetect: aBlock ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:35'!
allElementsDetect: aBlock ifNone: anExceptionBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."

	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:36'!
allElementsDo: aBlock
	"Descend depth-first visiting each element with aBlock."

	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/25/2010 13:58'!
allElementsNamed: aName
	^ self allElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/25/2010 13:17'!
allElementsNamed: aName do: aBlock
	self allElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/18/2014 04:38'!
allElementsNamed: aName namespaceURI: aUri
	^ self allElementsSelect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aUri]]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/14/2014 22:49'!
allElementsSelect: aBlock
	| selectedElements |

	selectedElements := self newNodeListForSelect.
	self allElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:42'!
configuration
	^ configuration ifNil: [
		self configuration: self configurationClass new.
		configuration]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/14/2014 07:43'!
configuration: aConfiguration
	configuration := aConfiguration.

	self hasChildren
		ifTrue: [self elementsDo: [:each | each configuration: aConfiguration]]! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/2/2014 18:52'!
configurationClass
	^ XMLDOMConfiguration! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:08'!
contentNodes
	^ self nodes contentNodes! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:56'!
contentNodesDo: aBlock
	self nodes contentNodesDo: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/4/2014 17:44'!
contentString: aString
	self
		removeNodes;
		addContent: aString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/25/2014 22:02'!
contentStringAt: aName
	^ self nodes contentStringAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/7/2014 00:30'!
descendentElements
	| descendentElements |

	descendentElements := self newNodeListForSelect.
	self descendentElementsDo: [:each | descendentElements addLast: each].
	^ descendentElements.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 02:18'!
descendentElementsCollect: aBlock
	| collectedObjects |

	collectedObjects := self newListForCollect.
	self descendentElementsDo: [:each | collectedObjects addLast: (aBlock value: each)].
	^ collectedObjects.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:34'!
descendentElementsDetect: aBlock
	^ self descendentElementsDetect: aBlock ifNone: [nil]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:34'!
descendentElementsDetect: aBlock ifNone: anExceptionBlock
	self descendentElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [^ each]].
	^ anExceptionBlock value.! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 01:55'!
descendentElementsDo: aBlock
	self hasChildren
		ifTrue: [self elementsDo: [:each | each allElementsDo: aBlock]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/7/2014 00:35'!
descendentElementsNamed: aName
	^ self descendentElementsSelect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:33'!
descendentElementsNamed: aName do: aBlock
	self descendentElementsDo: [:each |
		(each isNamed: aName)
			ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/7/2014 00:35'!
descendentElementsNamed: aName namespaceURI: aUri
	^ self descendentElementsSelect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aUri]]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 11/7/2014 00:32'!
descendentElementsSelect: aBlock
	| selectedElements |

	selectedElements := self newNodeListForSelect.
	self descendentElementsDo: [:each |
		(aBlock value: each)
			ifTrue: [selectedElements addLast: each]].
	^ selectedElements.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 2/14/2014 05:31'!
elementAfter: anElement
	^ self nodes elementAfter: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:31'!
elementAt: aName
	^ self nodes elementAt: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName ifAbsent: aBlock
	^ self nodes elementAt: aName ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri
	^ self nodes elementAt: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ self nodes elementAt: aName namespaceURI: aUri ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 2/14/2014 05:31'!
elementBefore: anElement
	^ self nodes elementBefore: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/26/2014 08:18'!
elementNames
	^ self nodes elementNames! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 5/4/2010 13:18'!
elements
	^ self nodes elements! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:32'!
elementsAt: aName
	^ self nodes elementsAt: aName! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:57'!
elementsAt: aName do: aBlock
	self nodes elementsAt: aName do: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/26/2010 12:33'!
elementsAt: aName namespaceURI: aUri
	^ self nodes elementsAt: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsCollect: aBlock
	^ self nodes elementsCollect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock
	^ self nodes elementsDetect: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:35'!
elementsDetect: aBlock ifNone: anExceptionBlock
	^ self nodes elementsDetect: aBlock ifNone: anExceptionBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 6/26/2010 00:55'!
elementsDo: aBlock
	self nodes elementsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 11/26/2010 12:36'!
elementsSelect: aBlock
	^ self nodes elementsSelect: aBlock! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 17:24'!
findElementNamed: aName
	^ self allElementsDetect: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 10/18/2014 04:39'!
findElementNamed: aName namespaceURI: aUri
	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aUri]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'monty 10/18/2014 04:39'!
findElementNamed: aName namespaceURI: aUri with: aBlock
	^ self allElementsDetect: [:each |
		(each isNamed: aName)
			and: [each namespaceURI = aUri
				and: [aBlock value: each]]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 11/25/2010 13:48'!
findElementNamed: aName with: aBlock
	^ self allElementsDetect: [:each | (each isNamed: aName) and: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:34'!
firstElement
	"returns nil if no elements"
	^ self nodes firstElement! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 10/25/2014 21:02'!
hasContentNodes
	^ self nodes hasContentNodes! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/24/2010 09:27'!
hasElements
	"Faster version used when traversing to avoid initializing the node list
	in leaf elements"
	^ nodes notNil and: [nodes hasElements]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 10/25/2014 21:01'!
hasStringNodes
	^ self nodes hasStringNodes! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'monty 10/25/2014 21:04'!
includesElement: aName
	^ self nodes includesElement: aName! !

!XMLNodeWithElements methodsFor: 'parsing' stamp: 'monty 11/2/2014 19:48'!
innerXML: aStringOrStream
	self
		innerXML: aStringOrStream
		parsedWith: (self parserHandlerClass on: aStringOrStream)! !

!XMLNodeWithElements methodsFor: 'parsing' stamp: 'monty 11/14/2014 20:57'!
innerXML: aStringOrStream parsedWith: aParser
	self removeNodes.

	aParser
		pushInnerXMLRootNode: self;
		parseDocument.! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/2/2014 19:48'!
innerXMLContext
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:15'!
isContentNode
	^ true! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/29/2014 14:34'!
lastElement
	"returns nil if no elements"
	^ self nodes lastElement! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 8/7/2010 20:11'!
namespaceScope
	^ XMLNamespaceScope new! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:54'!
newCData: aString
	^ self nodeFactory newCData: aString! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:57'!
newComment: aString
	^ self nodeFactory newComment: aString! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/3/2014 19:37'!
newDocument
	^ self nodeFactory newDocument
		configuration: self configuration! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/3/2014 19:37'!
newElement
	^ self nodeFactory newElement
		configuration: self configuration! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/3/2014 19:37'!
newElementNamed: aName
	^ (self nodeFactory newElementNamed: aName)
		configuration: self configuration! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/3/2014 19:37'!
newElementNamed: aName namespaceURI: aUri
	^ (self nodeFactory
		newElementNamed: aName
		namespaceURI: aUri)
		configuration: self configuration! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/1/2014 23:00'!
newPI
	^ self nodeFactory newPI! !

!XMLNodeWithElements methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:58'!
newString: aString
	^ self nodeFactory newString: aString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:02'!
nodeFactory
	^ self configuration nodeFactory! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 11/1/2014 11:37'!
nodeFactory: aNodeFactory
	self configuration nodeFactory: aNodeFactory! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/26/2013 22:23'!
nodeListClass
	^ self configuration nodeListClass! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'monty 11/1/2014 15:03'!
parserHandlerClass
	^ self configuration parserHandlerClass! !

!XMLNodeWithElements methodsFor: 'printing' stamp: 'monty 11/19/2014 23:44'!
printRawContentOn: aStream
	self contentNodesDo: [:each | each printRawContentOn: aStream]! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'monty 10/28/2014 17:59'!
removedElement: anElement
	self removedNode: anElement! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'monty 10/18/2014 01:08'!
renamedElement: anElement from: anOldName to: aNewName
	self nodes
		renamedElement: anElement
		from: anOldName
		to: aNewName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:00'!
stringNodes
	^ self nodes stringNodes! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:00'!
stringNodesDo: aBlock
	self nodes stringNodesDo: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'monty 10/25/2014 21:00'!
strings
	^ self nodes strings! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'monty 10/25/2014 21:00'!
stringsDo: aBlock
	self nodes stringsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 11/23/2010 14:48'!
usesNamespaces
	"Uses direct access and isNil test for performance"
	^ configuration isNil or: [configuration usesNamespaces]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 12:59'!
usesNamespaces: aBoolean
	self configuration usesNamespaces: aBoolean! !

XMLNode subclass: #XMLPI
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPI commentStamp: 'JAAyer 6/25/2010 14:06' prior: 0!
This class represents a processing instruction.
!

!XMLPI class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
target: targetName data: aString
	^ self new
		target: targetName;
		data: aString! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
data
	^ data ifNil: [data := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
data: aString
	data := aString! !

!XMLPI methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ true! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 12:42'!
target
	^ target ifNil: [target := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
target: aString
	target := aString! !

!XMLPI methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter pi
		target: self target;
		data: self data! !

XMLNode subclass: #XMLString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLString commentStamp: '<historical>' prior: 0!
This class represents string nodes, or nodes that contain no markup. String nodes can be created directly using the class-side #string: message or by sending a String object #asXMLString. When printing, they escape special characters in their contents.!

XMLString subclass: #XMLCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLCData commentStamp: 'monty 11/20/2014 04:08' prior: 0!
This class represents a preserved CDATA section from a parsed document. It behaves like its superclass, except that its printed form differs in that it contains a "<!![CDATA[" prefix and a "]]>" suffix and its contents are NOT escaped. By default, CDATA sections are converted to regular string nodes during parsing.

To preserve them as CDATA nodes, use #preservesCDataNodes: with the DOM parser before parsing.!

!XMLCData methodsFor: 'private' stamp: 'monty 2/11/2014 00:14'!
acceptsContent
	^ false! !

!XMLCData methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ true! !

!XMLCData methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter cdata: self string! !

!XMLString class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
string: aString
	^ self new string: aString! !

!XMLString methodsFor: 'private' stamp: 'monty 2/11/2014 00:14'!
acceptsContent
	^ true! !

!XMLString methodsFor: 'adding' stamp: 'monty 10/26/2014 05:53'!
addContent: aStringOrStringNode
	string :=
		string
			ifNil: [aStringOrStringNode asString]
			ifNotNil: [string, aStringOrStringNode asString]! !

!XMLString methodsFor: 'adding' stamp: 'monty 11/3/2014 22:12'!
append: aStringOrStringNode
	self addContent: aStringOrStringNode! !

!XMLString methodsFor: 'converting' stamp: 'monty 1/1/2014 07:14'!
asString
	^ self string! !

!XMLString methodsFor: 'converting' stamp: 'monty 11/4/2014 14:51'!
asXMLEscapedString
	^ self string asXMLEscapedString! !

!XMLString methodsFor: 'converting' stamp: 'JAAyer 7/22/2010 05:19'!
asXMLString
	^ self! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 21:43'!
beginsWith: aStringOrStringNode
	^ self string beginsWith: aStringOrStringNode asString! !

!XMLString methodsFor: 'accessing' stamp: 'monty 11/4/2014 17:44'!
contentString: aString
	self string: aString! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 12:38'!
deleteFrom: aStart to: anEnd
	self replaceFrom: aStart to: anEnd with: ''! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 21:43'!
endsWith: aStringOrStringNode
	^ self string endsWith: aStringOrStringNode asString! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/6/2014 20:11'!
includesSubstring: aStringOrStringNode
	^ (self indexOfSubstring: aStringOrStringNode) > 0! !

!XMLString methodsFor: 'accessing' stamp: 'monty 11/6/2014 20:11'!
indexOfSubstring: aStringOrStringNode
	^ self string findString: aStringOrStringNode asString startingAt: 1! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:26'!
isContentNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/3/2014 22:28'!
isEmpty
	^ self string isEmpty! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/5/2014 12:05'!
isString: aStringOrStringNode
	^ self string = aStringOrStringNode asString! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:35'!
isStringNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/5/2014 12:09'!
isStringNode: aStringOrStringNode
	^ self isString: aStringOrStringNode! !

!XMLString methodsFor: 'testing' stamp: 'monty 11/4/2014 11:45'!
isWhitespace
	^ self isEmpty not
		and: [self string allSatisfy: [:each | each isSeparator]]! !

!XMLString methodsFor: 'split and merge' stamp: 'monty 11/22/2014 08:06'!
mergeAdjacent
	| siblingNodes selfIndex adjacentNode |

	self hasParent
		ifFalse: [^ self].

	siblingNodes := self parent nodes.
	selfIndex := siblingNodes indexOf: self.
	[(adjacentNode := siblingNodes at: selfIndex + 1) notNil
		and: [adjacentNode isStringNode]]
		whileTrue: [
			self append: adjacentNode.
			siblingNodes removeAt: selfIndex + 1].

	[(adjacentNode := siblingNodes at: selfIndex - 1) notNil
		and: [adjacentNode isStringNode]]
		whileTrue: [
			self prepend: adjacentNode.
			siblingNodes removeAt: selfIndex - 1.
			selfIndex := selfIndex - 1].! !

!XMLString methodsFor: 'adding' stamp: 'monty 11/3/2014 21:43'!
prepend: aStringOrStringNode
	self string: aStringOrStringNode asString, self string! !

!XMLString methodsFor: 'printing' stamp: 'monty 11/19/2014 23:44'!
printRawContentOn: aStream
	aStream nextPutAll: self string! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 18:51'!
replaceAll: aSource with: aReplacement
	"works like copyReplaceAll:with:"
	self string:
		(self string
			copyReplaceAll: aSource asString
			with: aReplacement asString)! !

!XMLString methodsFor: 'replacing' stamp: 'monty 11/4/2014 18:54'!
replaceFrom: aStart to: aStop with: aReplacement
	"works like copyReplaceFrom:to:with: and
	can be used to insert, delete, and replace"
	| start stop |

	start := (aStart max: 1) min: self size + 1.
	stop := (aStop max: start - 1) min: self size.
	self string:
		(self string
			copyReplaceFrom: start
			to: stop
			with: aReplacement asString)! !

!XMLString methodsFor: 'accessing' stamp: 'monty 11/3/2014 23:41'!
size
	^ self string size! !

!XMLString methodsFor: 'split and merge' stamp: 'monty 11/3/2014 22:02'!
splitAt: anOffset
	| newNode |

	newNode := self copy.
	self replaceFrom: anOffset to: self size with: ''.
	newNode replaceFrom: 1 to: anOffset - 1 with: ''.

	self hasParent
		ifTrue: [self parent addNode: newNode after: self].

	^ newNode.! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:19'!
string
	^ string ifNil: [string := '']! !

!XMLString methodsFor: 'accessing' stamp: 'monty 1/1/2014 07:23'!
string: aString
	string := aString asString! !

!XMLString methodsFor: 'printing' stamp: 'JAA 3/22/2011 15:15'!
writeXMLOn: aWriter
	aWriter string: self string! !

Object subclass: #XMLNodeContentWriteStream
	instanceVariableNames: 'stream lastChar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLNodeContentWriteStream commentStamp: 'monty 11/20/2014 03:56' prior: 0!
A stream for writing node content with whitespace insertion if needed between writes.!

!XMLNodeContentWriteStream class methodsFor: 'instance creation' stamp: 'monty 11/19/2014 23:29'!
on: aStringOrStream
	^ self new on:
		(aStringOrStream isStream
			ifTrue: [aStringOrStream]
			ifFalse: [WriteStream on: aStringOrStream])! !

!XMLNodeContentWriteStream methodsFor: 'private' stamp: 'monty 11/19/2014 23:21'!
addSpaceFor: aCharacter
	(lastChar == nil
		or: [lastChar isSeparator
			or: [aCharacter isSeparator]])
		ifFalse: [stream space]! !

!XMLNodeContentWriteStream methodsFor: 'accessing' stamp: 'monty 11/19/2014 23:14'!
contents
	^ stream contents! !

!XMLNodeContentWriteStream methodsFor: 'testing' stamp: 'monty 11/20/2014 01:57'!
isStream
	^ true! !

!XMLNodeContentWriteStream methodsFor: 'accessing' stamp: 'monty 11/19/2014 23:20'!
nextPut: aCharacter
	self addSpaceFor: aCharacter.
	stream nextPut: aCharacter.
	lastChar := aCharacter.! !

!XMLNodeContentWriteStream methodsFor: 'accessing' stamp: 'monty 11/19/2014 23:21'!
nextPutAll: aString
	aString size > 0
		ifTrue: [
			self addSpaceFor: aString first.
			stream nextPutAll: aString.
			lastChar := aString last]! !

!XMLNodeContentWriteStream methodsFor: 'initialization' stamp: 'monty 11/19/2014 23:14'!
on: aStream
	stream := aStream! !

Object subclass: #XMLNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLNodeFactory commentStamp: '<historical>' prior: 0!
This class is the default node factory. Its accessor messages return the class objects used by the DOM parser to build a node tree. You can override these messages to return different class objects and inject an instance of your factory using the DOM parser's #nodeFactory: accessor before parsing.!

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:58'!
attributeDeclarationClass
	^ XMLAttributeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
cdataClass
	^ XMLCData! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 13:23'!
classForElement: aQualifiedName
	^ self
		classForElement: aQualifiedName
		namespaceURI: ''! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 11/16/2014 20:23'!
classForElement: aQualifiedName namespaceURI: aUri
	| prefix |

	^ self
		classForElement: aQualifiedName
		prefix: (prefix := aQualifiedName prefixBeforeLocalName)
		uri: aUri
		localName: (aQualifiedName localNameAfterPrefix: prefix)! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:20'!
classForElement: aQualifiedName prefix: aPrefix uri: aNamespaceUri localName: aLocalName
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
commentClass
	^ XMLComment! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
doctypeDeclarationClass
	^ XMLDoctypeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
documentClass
	^ XMLDocument! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
elementClass
	^ XMLElement! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
elementDeclarationClass
	^ XMLElementDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
generalEntityDeclarationClass
	^ XMLGeneralEntityDeclaration! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:57'!
newCData: aString
	^ self cdataClass string: aString! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:57'!
newComment: aString
	^ self commentClass string: aString! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:00'!
newDocument
	^ self documentClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:00'!
newElement
	^ self elementClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:01'!
newElementNamed: aName
	^ (self classForElement: aName)
		named: aName! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/6/2014 22:00'!
newElementNamed: aName namespaceURI: aUri
	^ (self classForElement: aName namespaceURI: aUri)
		name: aName
		namespaceURI: aUri! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:55'!
newPI
	^ self piClass new! !

!XMLNodeFactory methodsFor: 'instance creation' stamp: 'monty 11/1/2014 22:58'!
newString: aString
	^ self stringClass string: aString! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
notationDeclarationClass
	^ XMLNotationDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:59'!
parameterEntityDeclarationClass
	^ XMLParameterEntityDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
piClass
	^ XMLPI! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
stringClass
	^ XMLString! !

XMLNodeFactory subclass: #XMLPluggableNodeFactory
	instanceVariableNames: 'documentClass elementClass stringClass cdataClass commentClass piClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPluggableNodeFactory commentStamp: 'monty 11/1/2014 11:23' prior: 0!
This is a pluggable factory that can be used to change which classes the DOM parser uses to build the DOM tree. Here is an example:

	(XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableFactory new
				documentClass: MyDocumentClass;
				elementClass: MyElementClass;
				stringClass: MyStringClass)
		parseDocument.!

XMLPluggableNodeFactory subclass: #XMLPluggableElementFactory
	instanceVariableNames: 'elementClassesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DOM'!

!XMLPluggableElementFactory commentStamp: '<historical>' prior: 0!
This class is a pluggable factory that can map elements to different XMLElement subclasses based on the name and namespace information of those elements. You have to create an instance, configure it to handle certain elements with certain classes, and then inject your instance into a DOM parser using #nodeFactory: before parsing. Here is an example of its use:

	doc := (XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableElementFactory new
				elementClass: GenericElement;
				handleElement: 'user' withClass: UserElement;
				handleElement: 'report' withClass: ReportElement;
				handleElement: 'report' namespaceURI: 'urn:specialreprot' withClass: SpecialReportElement)
		parseDocument.
		
The #handleElementsMatchingClass*: forms try to match elements to the specified classes based on the names of those classes when stripped of any prefix (like XML) and "Element" suffix and converted to both camel case and a hyphenated form. So this:
	XMLPluggableElementFactory new
		handleElementsMatchingClass: MYRootUserElement
		....
will handle 'rootUser' and 'root-user' elements with the MYRootUserElement class.!

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/14/2010 21:57'!
classForElement: aName prefix: aPrefix uri: aUri localName: aLocalName
	^ (self classForElement: aName uri: aUri)
		ifNil: [
			aPrefix
				ifEmpty: [self elementClass]
				ifNotEmpty: [
					(self classForElement: aLocalName uri: aUri)
						ifNil: [self elementClass]]]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/14/2010 21:55'!
classForElement: aName uri: aUri
	| classesForElement |

	classesForElement :=
		self elementClassesByName
			at: aName
			ifAbsent: [^ nil].

	^ classesForElement
		at: aUri
		ifAbsent: [
			classesForElement
				at: ''
				ifAbsent: [nil]].! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/13/2010 08:36'!
elementClassesByName
	^ elementClassesByName ifNil: [elementClassesByName := Dictionary new]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName namespaceURI: aUri withClass: aClass
	(self elementClassesByName
		at: anElementName
		ifAbsentPut: [Dictionary new])
			at: aUri put: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName withClass: aClass
	self
		handleElement: anElementName
		namespaceURI: ''
		withClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:39'!
handleElements: anElementNameCollection namespaceURI: aNamespaceURI withClass: aClass
	anElementNameCollection do: [:each |
		self
			handleElement: each
			namespaceURI: aNamespaceURI
			withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:36'!
handleElements: anElementNameCollection withClass: aClass
	anElementNameCollection do: [:each | self handleElement: each withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/22/2010 11:10'!
handleElementsMatchingClass: aClass
	| className |

	(className := self toNameWithoutPrefixAndElementSuffix: aClass name asString)
		ifEmpty: [^ self].

	self
		handleElement: (self toCamelCasedName: className)
		withClass: aClass.
	self
		handleElement: (self toHyphenatedName: className)
		withClass: aClass.! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:26'!
handleElementsMatchingClasses: aClassCollection
	aClassCollection do: [:each | self handleElementsMatchingClass: each]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:29'!
toCamelCasedName: aString
	^ aString allButFirst copyWithFirst: aString first asLowercase! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:34'!
toHyphenatedName: aString
	| nameStream hyphenatedStream previousChar nextChar |

	nameStream := aString readStream.
	hyphenatedStream := (String new: nameStream size) writeStream.
	[nameStream atEnd]
		whileFalse: [
			((nextChar := nameStream next) isUppercase
				and: [previousChar notNil and: [previousChar isLowercase]])
				ifTrue: [hyphenatedStream nextPut: $-].
			hyphenatedStream nextPut: nextChar asLowercase.
			previousChar := nextChar].
	^ hyphenatedStream contents.! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/22/2010 11:10'!
toNameWithoutPrefixAndElementSuffix: aString
	| nameStream |

	nameStream := aString readStream.
	[nameStream atEnd not and: [nameStream peek isUppercase]]
		whileTrue: [nameStream next].
	nameStream atEnd
		ifTrue: [^ ''].
	nameStream back.

	^ nameStream upToEnd copyReplaceAll: 'Element' with: ''.! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
cdataClass
	^ cdataClass ifNil: [cdataClass := super cdataClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
cdataClass: aClass
	cdataClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:45'!
commentClass
	^ commentClass ifNil: [commentClass := super commentClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:45'!
commentClass: aClass
	commentClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:42'!
documentClass
	^ documentClass ifNil: [documentClass := super documentClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:42'!
documentClass: aClass
	documentClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:41'!
elementClass
	^ elementClass ifNil: [elementClass := super elementClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:42'!
elementClass: aClass
	elementClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
piClass
	^ piClass ifNil: [piClass := super piClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:44'!
piClass: aClass
	piClass := aClass! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:43'!
stringClass
	^ stringClass ifNil: [stringClass := super stringClass]! !

!XMLPluggableNodeFactory methodsFor: 'accessing' stamp: 'monty 11/1/2014 10:43'!
stringClass: aClass
	stringClass := aClass! !

Object subclass: #XMLParser
	instanceVariableNames: 'tokenizer driver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParser commentStamp: '<historical>' prior: 0!
This class is a facade that parses XML using a driver and tokenizer. See SAXHandler and XMLDOMParser for ready-made front ends to it.!

!XMLParser class methodsFor: 'instance creation' stamp: 'JAAyer 11/28/2010 07:09'!
driver: aDriver on: aStringOrStream
	^ self new setDriver: aDriver tokenizer: (aDriver tokenizerOn: aStringOrStream)! !

!XMLParser methodsFor: 'testing' stamp: 'JAAyer 11/27/2010 07:17'!
atEnd
	^ self tokenizer atEnd! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 11/14/2013 23:34'!
currentLineNumber
	^ self tokenizer currentLineNumber! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/28/2010 07:07'!
driver
	^ driver! !

!XMLParser methodsFor: 'private' stamp: 'monty 11/24/2014 19:31'!
handleParserError: anError
	self tokenizer closeStreams.

	(anError respondsTo: #appendLineNumber:)
		ifTrue: [anError appendLineNumber: self currentLineNumber].
	anError pass.! !

!XMLParser methodsFor: 'accessing' stamp: 'monty 11/9/2014 16:01'!
initialContext: aContext
	self tokenizer tokenContext: aContext! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseDocument
	[[self tokenizer atEnd]
		whileFalse: [self tokenizer nextToken]]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseDocumentWhile: aBlock
	[[self tokenizer atEnd not and: [aBlock value]]
		whileTrue: [self tokenizer nextToken]]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'parsing' stamp: 'monty 11/24/2014 19:31'!
parseToken
	^ [self tokenizer nextToken]
		on: Error
		do: [:error | self handleParserError: error]! !

!XMLParser methodsFor: 'initialization' stamp: 'JAAyer 11/28/2010 07:09'!
setDriver: aDriver tokenizer: aTokenizer
	driver := aDriver.
	tokenizer := aTokenizer.! !

!XMLParser methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 07:16'!
tokenizer
	^ tokenizer! !

Object subclass: #XMLParserDriver
	instanceVariableNames: 'uris'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserDriver commentStamp: '<historical>' prior: 0!
This is a base class for parser drivers. Drivers are sent messages by a tokenizer to handle various XML tokens as they are read. To create a driver, subclass this class and override the handler messages and testing messages that control tokenization.!

XMLParserDriver subclass: #DTDExternalSubsetParserDriver
	instanceVariableNames: 'saxDriver dtd documentUri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!DTDExternalSubsetParserDriver commentStamp: '<historical>' prior: 0!
This class is a driver for parsing the external subset of a DTD.!

!DTDExternalSubsetParserDriver class methodsFor: 'instance creation' stamp: 'monty 11/19/2013 06:10'!
saxDriver: aSaxDriver documentURI: anExternalSubsetUri
	^ self new
		setSAXDriver: aSaxDriver
		documentURI: anExternalSubsetUri ! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:11'!
documentURI
	^ documentUri! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 17:58'!
dtd
	^ dtd! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
externalEntityResolver
	^ self saxDriver externalEntityResolver! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:55'!
handleAttributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForAttributeIn: anElement
				named: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/10/2013 06:32'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForElement: aName
				contentModel: aContentModel]! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/14/2013 02:15'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	aNotation ifNotEmpty: [
		self dtd
			addUnparsedEntity: aName
			ndata: aNotation.
		^ self].

	self dtd
		addExternalGeneralEntity: aName
		resolver: self externalEntityResolver
		uri: aSystemId
		from: self currentURI.! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:52'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self dtd
		addGeneralEntity: aName
		replacement: aReplacement
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/19/2013 02:20'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self dtd addNotation: aName! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:55'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self dtd
		addExternalParameterEntity: aName
		resolver: self externalEntityResolver
		uri: aSystemId
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:53'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self dtd
		addParameterEntity: aName
		replacement:aReplacement
		from: self currentURI! !

!DTDExternalSubsetParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/31/2014 21:14'!
handleParameterEntityReference: aName
	| entity |

	entity := self dtd parameterEntityAt: aName.
	(entity isNil and: [self isValidating])
		ifTrue: [self saxDriver errorUndeclaredParameterEntity: aName].

	^ entity.! !

!DTDExternalSubsetParserDriver methodsFor: 'initialization' stamp: 'monty 11/17/2014 02:02'!
initialContext
	^ XMLExternalSubsetContext new textDeclarationContext! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 2/7/2014 23:33'!
isParsingExternalSubset
	^ true! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:58'!
isValidating
	^ self saxDriver isValidating! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 19:10'!
maxEntityReplacementDepth
	^ self saxDriver maxEntityReplacementDepth! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 23:07'!
readLimit
	^ self saxDriver externalSubsetReadLimit! !

!DTDExternalSubsetParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 06:04'!
saxDriver
	^ saxDriver! !

!DTDExternalSubsetParserDriver methodsFor: 'initialization' stamp: 'monty 2/3/2014 17:58'!
setSAXDriver: aSaxdriver documentURI: aUri
	saxDriver := aSaxdriver.
	dtd := saxDriver dtd.
	documentUri := aUri.! !

!DTDExternalSubsetParserDriver methodsFor: 'testing' stamp: 'monty 11/30/2013 21:55'!
usesNamespaces
	^ self saxDriver usesNamespaces! !

XMLParserDriver subclass: #SAXParserDriver
	instanceVariableNames: 'saxHandler elementNester scopeAllocator dtd validator hasUnhandledReservedAttributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXParserDriver commentStamp: 'monty 2/9/2014 02:43' prior: 0!
SAXParserDriver together with SAXHandler provides a SAX-like API for XML parsing. The driver processes XML tokens and signals SAX events to a SAXHandler. To use it, create a subclass of SAXHandler and override messages to handle SAX events of interest. See "instance creation" on the class-side of SAXHandler for ways to instantiate such a parser.!

!SAXParserDriver class methodsFor: 'instance creation' stamp: 'monty 10/27/2014 09:12'!
saxHandler: aHandler
	^ self new setSAXHandler: aHandler! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/3/2014 19:19'!
checkForReservedAttributesIn: anAttributeDictionary after: anIndex
	"validation may add attributes"
	self hasUnhandledReservedAttributes
		ifFalse: [
			anIndex + 1
				to: anAttributeDictionary size
				do: [:i |
					(anAttributeDictionary keyAtIndex: i) isReservedAttributeName
						ifTrue: [^ self haveUnhandledReservedAttributes]]]! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 04:13'!
currentElement
	^ self elementNester currentElement! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 2/8/2014 03:11'!
declareNamespaces: aDictionary inScope: aScope
	aDictionary keysAndValuesDo: [:prefix :uri |
		aScope mapPrefix: prefix to: uri.
		self saxHandler startPrefixMapping: prefix uri: uri]! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:44'!
documentReadLimit
	^ self saxHandler documentReadLimit! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ self saxHandler documentURI! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 19:30'!
dtd
	^ dtd ifNil: [dtd := DTDDoctypeDefinition new]! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 10/27/2014 09:11'!
elementNester
	^ elementNester! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 12/27/2013 04:08'!
elementNesterClass
	^ XMLWellFormedElementNester! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 2/11/2014 01:53'!
endNestingOfElement: anElementName withScopedAttributes: anAttributeDictionary
	| base |

	base :=
		anAttributeDictionary
			at: 'xml:base'
			ifAbsent: [
				self elementNester endElement: anElementName.
				^ self].
	(self elementNester
		endElement: anElementName;
		scopedAttributeAt: 'xml:base') = base
			ifFalse: [self removeURI: base].! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/2/2014 18:39'!
errorCharacterDataOutsideElement
	XMLWellFormednessException signal: 'Character data outside of element'! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 17:23'!
errorDocumentConstraintsRequired
	XMLValidationException signal: 'Document validation constraints required'! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 21:41'!
errorUndeclaredGeneralEntity: aName
	XMLValidationException
		formatSignal: 'Undeclared general entity reference &{1};'
		with: aName! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 18:19'!
errorUndeclaredNotation: aNotation referencedBy: anEntity
	XMLValidationException
		formatSignal: 'Undeclared notation "{1}" referenced by &{2}; entity'
		with: aNotation
		with: anEntity! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 21:14'!
errorUndeclaredParameterEntity: aName
	XMLValidationException
		formatSignal: 'Undeclared parameter entity reference %{1};'
		with: aName! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 21:41'!
errorUndeclaredUnparsedEntity: aName
	XMLValidationException
		formatSignal: 'Undeclared unparsed entity reference &{1}; as attribute value'
		with: aName! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 2/3/2014 17:42'!
externalEntityResolver
	^ self saxHandler externalEntityResolver! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/3/2014 00:19'!
externalSubsetParserOnURI: aUri
	^ self saxHandler parserClass
		driver:
			(DTDExternalSubsetParserDriver
				saxDriver: self
				documentURI: aUri asString)
		on: (self externalEntityResolver resolveEntityURI: aUri)
	! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
externalSubsetReadLimit
	^ self saxHandler externalSubsetReadLimit! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:56'!
handleAttributeDeclaration: anElement name: aName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForAttributeIn: anElement
				named: aName
				type: aType
				defaultPragma: aDefaultPragma
				defaultValue: aDefaultValue].
	self saxHandler
		attributeDeclaration: anElement
		name: aName
		type: aType
		defaultPragma: aDefaultPragma
		defaultValue: aDefaultValue! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/3/2014 13:14'!
handleCData: aString
	self elementNester isInElement
		ifFalse: [self errorCharacterDataOutsideElement].
	self isValidating
		ifTrue: [self validator validatePCData: aString].

	self saxHandler
		startCData;
		startCDataSection;
		characters: aString;
		endCDataSection;
		endCData.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/11/2013 22:26'!
handleComment: aString
	self saxHandler comment: aString! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/10/2013 06:32'!
handleElementDeclaration: aName contentModel: aContentModel
	self isValidating
		ifTrue: [
			self dtd
				addValidatorForElement: aName
				contentModel: aContentModel].

	self saxHandler
		elementDeclaration: aName
		contentModel: aContentModel! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
	self saxHandler endEntity: aName! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/27/2014 14:40'!
handleEndDTD
	(self resolvesExternalEntities
		and: [self dtd hasExternalSubset])
		ifTrue: [self parseExternalSubset: self dtd externalSubsetURI].

	self saxHandler
		endInternalSubset;
		endDTD.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/2/2014 02:44'!
handleEndDocument
	self elementNester endNesting.
	self isValidating
		ifTrue: [self validator validateEndDocument].

	self saxHandler endDocument.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2014 20:23'!
handleEndTag: anElementName
	"performance critical method"
	| prefix currentScope |

	self elementNester hasScopedAttributes
		ifTrue: [
			self
				endNestingOfElement: anElementName
				withScopedAttributes: self elementNester currentScopedAttributes]
		ifFalse: [self elementNester endElement: anElementName].

	prefix := anElementName prefixBeforeLocalName.
	self saxHandler 
		endElement: anElementName
		prefix: prefix
		uri:
			(self usesNamespaces
				ifTrue: [
					currentScope := self scopeAllocator exitScope.
					currentScope resolvePrefix: prefix]
				ifFalse: [''])
		localName: (anElementName localNameAfterPrefix: prefix).

	currentScope ifNotNil: [
		currentScope hasNamespaces
			ifTrue: [self undeclareNamespacesInScope: currentScope]].
	self isValidating
		ifTrue: [self validator validateEndTag: anElementName].! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/9/2014 02:28'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation
	aNotation
		ifNotEmpty: [
			self dtd
				addUnparsedEntity: aName
				ndata: aNotation]
		ifEmpty: [
			self resolvesExternalEntities
				ifTrue: [
					self dtd
						addExternalGeneralEntity: aName
						resolver: self externalEntityResolver
						uri: aSystemId
						from: self currentURI]].

	self saxHandler
		generalEntityDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId
		ndata: aNotation.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:49'!
handleGeneralEntityDeclaration: aName replacement: aReplacement
	self dtd
		addGeneralEntity: aName
		replacement: aReplacement
		from: self currentURI.

	self saxHandler
		generalEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/31/2014 21:15'!
handleGeneralEntityReference: aName
	| entity |

	(entity := self dtd generalEntityAt: aName)
		ifNil: [
			self isValidating
				ifTrue: [self errorUndeclaredGeneralEntity: aName].
			self preservesUndeclaredEntityReferences
				ifTrue: [entity := DTDBypassedGeneralEntity named: aName].
			self saxHandler skippedEntity: aName].
	^ entity.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/19/2013 02:19'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId
	self isValidating
		ifTrue: [self dtd addNotation: aName].

	self saxHandler
		notationDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/3/2014 13:09'!
handlePCData: aString
	self elementNester isInElement
		ifFalse: [self errorCharacterDataOutsideElement].
	self isValidating
		ifTrue: [self validator validatePCData: aString].

	self saxHandler characters: aString.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/2/2014 04:03'!
handlePI: piTarget data: piData
	self saxHandler 
		processingInstruction: piTarget
		data: piData! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 2/9/2014 02:28'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId
	self resolvesExternalEntities
		ifTrue: [
			self dtd
				addExternalParameterEntity: aName
				resolver: self externalEntityResolver
				uri: aSystemId
				from: self currentURI].

	self saxHandler
		parameterEntityDeclaration: aName
		publicID: aPublicId
		systemID: aSystemId.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/13/2013 18:54'!
handleParameterEntityDeclaration: aName replacement: aReplacement
	self dtd
		addParameterEntity: aName
		replacement:aReplacement
		from: self currentURI.

	self saxHandler
		parameterEntityDeclaration: aName
		replacement: aReplacement.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/31/2014 21:14'!
handleParameterEntityReference: aName
	| entity |

	(entity := self dtd parameterEntityAt: aName)
		ifNil: [
			self isValidating
				ifTrue: [self errorUndeclaredParameterEntity: aName].
			self preservesUndeclaredEntityReferences
				ifTrue: [entity := DTDBypassedParameterEntity named: aName]].
	^ entity.! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/6/2014 02:28'!
handleReservedAttributes: anAttributeDictionary
	anAttributeDictionary
		at: 'xml:base'
		ifPresent: [:uri | self currentURI: uri].
	self isValidating
		ifTrue: [self validator validateXMLIDAttributeIn: anAttributeDictionary].

	self elementNester
		setScopedAttributesNamed: self scopedAttributeNames
		from: anAttributeDictionary.

	hasUnhandledReservedAttributes := false.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/18/2013 22:08'!
handleStartContentEntityReplacement: aName
	self saxHandler startEntity: aName! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 10/27/2014 14:41'!
handleStartDTD: aRoot publicID: aPublicId systemID: aSystemId
	self dtd
		rootElement: aRoot
		externalSubsetURI: aSystemId.
	
	self saxHandler
		startDTD: aRoot
		publicID: aPublicId
		systemID: aSystemId;
		startInternalSubset.! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:29'!
handleStartDocument
	self saxHandler startDocument! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2014 20:23'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary
	"performance critical method"
	| prefix totalAttributes namespaceUri currentScope |

	self elementNester startElement: anElementName.

	prefix := anElementName prefixBeforeLocalName.
	totalAttributes := anAttributeDictionary size.
	self usesNamespaces
		ifTrue: [
			currentScope := self scopeAllocator enterScope.
			aNamespaceDictionary
				ifNotNil: [self declareNamespaces: aNamespaceDictionary inScope: currentScope].
			namespaceUri := currentScope validateAndResolvePrefix: prefix ofElement: anElementName.
			totalAttributes > 0
				ifTrue: [currentScope validateAttributePrefixes: anAttributeDictionary]].
	self isValidating
		ifTrue: [
			self elementNester isCurrentElementRoot
				ifTrue: [self validateRootElement: anElementName].
			self validator
				validateStartTag: anElementName
				attributes: anAttributeDictionary.
			anAttributeDictionary size > totalAttributes
				ifTrue: [self checkForReservedAttributesIn: anAttributeDictionary after: totalAttributes]].
	self hasUnhandledReservedAttributes
		ifTrue: [self handleReservedAttributes: anAttributeDictionary].

	self saxHandler
		startElement: anElementName
		prefix: prefix
		uri: (namespaceUri ifNil: [''])
		localName: (anElementName localNameAfterPrefix: prefix)
		attributes: anAttributeDictionary.
	currentScope
		ifNotNil: [self saxHandler currentScope: currentScope].! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/2/2014 19:58'!
handleWhitespace: aString
	(self elementNester hasScopedAttributes
		and: [(self elementNester scopedAttributeAt: 'xml:space') = 'preserve'])
		ifTrue: [self handlePCData: aString]
		ifFalse: [self saxHandler ignorableWhitespace: aString]! !

!SAXParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/18/2014 19:36'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo
	| versionNumber isStandalone |

	versionNumber := Float readFrom: aVersion readStream.
	isStandalone := aYesOrNo = 'yes'.

	self dtd isStandalone: isStandalone.

	self saxHandler
		xmlVersion: versionNumber
		encoding: anEncoding
		standalone: isStandalone.! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/3/2014 11:19'!
hasUnhandledReservedAttributes
	^ hasUnhandledReservedAttributes! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/3/2014 16:16'!
haveUnhandledReservedAttributes
	hasUnhandledReservedAttributes := true! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:43'!
isValidating
	^ self saxHandler isValidating! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
maxEntityReplacementDepth
	^ self saxHandler maxEntityReplacementDepth! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 1/10/2014 02:29'!
parseExternalSubset: aUri
	| parser |

	parser :=
		self externalSubsetParserOnURI:
			(self externalEntityResolver
				combineURI: self documentURI
				with: aUri).
	parser parseDocument.! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/11/2014 01:23'!
preservesUndeclaredEntityReferences
	^ self saxHandler preservesUndeclaredEntityReferences! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 1/19/2014 04:45'!
readLimit
	^ self documentReadLimit! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:28'!
resolvesExternalEntities
	^ self saxHandler resolvesExternalEntities! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'JAAyer 11/27/2010 08:00'!
saxHandler
	^ saxHandler! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 12/27/2013 03:40'!
scopeAllocator
	^ scopeAllocator ifNil: [scopeAllocator := XMLNamespaceScopeAllocator new]! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 11/2/2014 20:45'!
scopedAttributeNames
	^ #('xml:space' 'xml:base' 'xml:lang')! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 10/27/2014 15:01'!
setRootElement: anElementName currentScope: aNamespaceScope
	self elementNester setExternalRootElement: anElementName.
	self usesNamespaces
		ifTrue: [self scopeAllocator currentScope: aNamespaceScope].! !

!SAXParserDriver methodsFor: 'initialization' stamp: 'monty 11/3/2014 11:20'!
setSAXHandler: aHandler
	saxHandler := aHandler.
	elementNester := self elementNesterClass new.
	hasUnhandledReservedAttributes := false.! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/10/2014 05:53'!
undeclareNamespacesInScope: aScope
	| currentScope |

	currentScope := self scopeAllocator currentScope.
	aScope prefixesAndURIsDo: [:prefix :uri |
		(currentScope includesPrefix: prefix mappedTo: uri)
			ifFalse: [self saxHandler endPrefixMapping: prefix]].! !

!SAXParserDriver methodsFor: 'testing' stamp: 'monty 2/3/2014 17:43'!
usesNamespaces
	^ self saxHandler usesNamespaces! !

!SAXParserDriver methodsFor: 'private' stamp: 'monty 10/31/2014 17:23'!
validateRootElement: aStartTag
	(self saxHandler requiresDocumentConstraints
		and: [self validator hasElementValidators not])
		ifTrue: [self errorDocumentConstraintsRequired].

	self validator validateRootElement: aStartTag.! !

!SAXParserDriver methodsFor: 'accessing' stamp: 'monty 11/19/2013 01:10'!
validator
	^ validator ifNil: [validator := DTDDocumentValidator dtd: self dtd]! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 1/20/2014 21:45'!
currentURI
	^ self uris
		ifEmpty: [self documentURI]
		ifNotEmpty: [self uris last]! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 1/20/2014 02:25'!
currentURI: aUri
	self uris addLast: aUri! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 11/16/2013 04:35'!
documentURI
	^ nil! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 11/30/2013 04:17'!
dtd
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 12/30/2013 23:46'!
handleAttributeDeclaration: anElement name: anName type: aType defaultPragma: aDefaultPragma defaultValue: aDefaultValue! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleCData: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleComment: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:46'!
handleElementDeclaration: aName contentModel: aContentModel! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleEndContentEntityReplacement: aName
! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/12/2013 17:36'!
handleEndDTD! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndDocument! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleEndTag: anElementName! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:08'!
handleGeneralEntityDeclaration: aName publicID: aPublicId systemID: aSystemId ndata: aNotation! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:10'!
handleGeneralEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 20:44'!
handleGeneralEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:56'!
handleNotationDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handlePCData: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handlePI: piTarget data: piData! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:09'!
handleParameterEntityDeclaration: aName publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 03:10'!
handleParameterEntityDeclaration: aName replacement: aReplacement! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/14/2013 20:44'!
handleParameterEntityReference: aName
	^ nil! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 1/4/2014 07:42'!
handleParseError: anErrorString
	XMLWellFormednessException signal: anErrorString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/16/2013 05:10'!
handleStartContentEntityReplacement: aName! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'monty 11/12/2013 17:15'!
handleStartDTD: aRoot publicID: aPublicId systemID: aSystemId! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartDocument! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleWhitespace: aString! !

!XMLParserDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/27/2010 07:23'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo! !

!XMLParserDriver methodsFor: 'initialization' stamp: 'monty 11/2/2014 19:09'!
initialContext
	^ nil! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/8/2014 02:03'!
isParsingExternalSubset
	^ false! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:41'!
isValidating
	^ false! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 1/16/2014 19:45'!
readLimit
	^ nil! !

!XMLParserDriver methodsFor: 'accessing' stamp: 'monty 2/2/2014 07:33'!
removeURI: aUri
	"remove: removes the first match"
	self uris removeAt:
		(self uris
			lastIndexOf: aUri
			ifAbsent: [^ self])! !

!XMLParserDriver methodsFor: 'private' stamp: 'monty 2/10/2014 02:43'!
tokenizerClass
	^ XMLWellFormedParserTokenizer! !

!XMLParserDriver methodsFor: 'initialization' stamp: 'monty 1/17/2014 01:28'!
tokenizerOn: aStringOrStream
	^ self tokenizerClass
		driver: self
		on: aStringOrStream
		readLimit: self readLimit! !

!XMLParserDriver methodsFor: 'private' stamp: 'monty 1/20/2014 02:25'!
uris
	^ uris ifNil: [uris := OrderedCollection new]! !

!XMLParserDriver methodsFor: 'testing' stamp: 'monty 2/9/2014 02:41'!
usesNamespaces
	^ true! !

Object subclass: #XMLParserLimits
	instanceVariableNames: 'documentReadLimit externalSubsetReadLimit maxEntityReplacementDepth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserLimits commentStamp: 'monty 1/25/2014 04:00' prior: 0!
This class places security limits on parser input. You can set these using the configuration messages in SAXHandler or remove them using removeLimits.!

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 10/3/2014 18:56'!
defaultDocumentReadLimit
	^ 10000000! !

!XMLParserLimits class methodsFor: 'accessing' stamp: 'monty 10/3/2014 23:22'!
defaultExternalSubsetReadLimit
	^ (self defaultDocumentReadLimit / 4) asInteger! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:43'!
documentReadLimit
	^ documentReadLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:44'!
documentReadLimit: aLimit
	documentReadLimit := aLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:44'!
externalSubsetReadLimit
	^ externalSubsetReadLimit! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:44'!
externalSubsetReadLimit: aLimit
	externalSubsetReadLimit := aLimit! !

!XMLParserLimits methodsFor: 'initialization' stamp: 'monty 10/3/2014 23:22'!
initialize
	super initialize.

	documentReadLimit := self class defaultDocumentReadLimit.
	externalSubsetReadLimit := self class defaultExternalSubsetReadLimit.
	maxEntityReplacementDepth := 3.! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:45'!
maxEntityReplacementDepth
	^ maxEntityReplacementDepth! !

!XMLParserLimits methodsFor: 'accessing' stamp: 'monty 1/16/2014 18:45'!
maxEntityReplacementDepth: aLimit
	maxEntityReplacementDepth := aLimit! !

!XMLParserLimits methodsFor: 'removing' stamp: 'monty 1/16/2014 18:45'!
removeAll
	documentReadLimit := nil.
	externalSubsetReadLimit := nil.
	maxEntityReplacementDepth := nil.! !

Object subclass: #XMLParserTokenizer
	instanceVariableNames: 'driver context streamReader writeStream nameStream'
	classVariableNames: 'PredefinedEntities'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLParserTokenizer commentStamp: 'monty 2/9/2014 02:44' prior: 0!
This is a base class for tokenizers. It reads tokens from a stream using a NestedStreamReader. Sending it #nextToken causes a token to be read and one or more handler messages to be dispatched to a driver.

Be careful changing the code in this class or subclass because it is optimized.!

!XMLParserTokenizer class methodsFor: 'instance creation' stamp: 'monty 1/17/2014 01:27'!
driver: aDriver on: aStringOrStream
	^self
		driver: aDriver
		on: aStringOrStream
		readLimit: nil! !

!XMLParserTokenizer class methodsFor: 'instance creation' stamp: 'monty 1/17/2014 01:26'!
driver: aDriver on: aStringOrStream readLimit: aReadLimit
	^self new
		setDriver: aDriver
		stream:
			(aStringOrStream isStream
				ifTrue: [aStringOrStream]
				ifFalse: [aStringOrStream readStream])
		readLimit: aReadLimit! !

!XMLParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 07:11'!
initialize
	"self initialize"

	(PredefinedEntities := Dictionary new)
		at: 'lt' put: $<;
		at: 'gt' put: $>;
		at: 'amp' put: $&;
		at: 'apos' put: $';
		at: 'quot' put: $"! !

!XMLParserTokenizer class methodsFor: 'accessing' stamp: 'monty 11/18/2014 07:11'!
predefinedEntities
	^ PredefinedEntities! !

!XMLParserTokenizer methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
atEnd
	^ context isTerminatedContext! !

!XMLParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 2/3/2014 22:40'!
characterFromCodePoint: aCodePoint
	^ (aCodePoint notNil and: [aCodePoint > 0])
		ifTrue: [
			[aCodePoint asCharacter]
				on: Error
				do: [:error | nil]]
		ifFalse: [nil]! !

!XMLParserTokenizer methodsFor: 'file open/close' stamp: 'monty 11/24/2014 19:27'!
closeStreams
	streamReader closeStreams! !

!XMLParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/17/2014 01:52'!
constructReplacementForBypassedGeneralEntity: anEntity! !

!XMLParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/17/2014 01:52'!
constructReplacementForBypassedParameterEntity: anEntity! !

!XMLParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/10/2014 00:46'!
constructReplacementForGeneralEntity: anEntity! !

!XMLParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/10/2014 00:37'!
constructReplacementForParameterEntity: anEntity! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
currentLineNumber
	^ streamReader currentLineNumber! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
driver
	^ driver! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:35'!
expectNext: aCharacter
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:37'!
expectNextAll: aString
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:36'!
expectTerminator: aCharacter
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/8/2013 04:38'!
expectUpToAll: aString
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/30/2014 18:25'!
nextAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 01:15'!
nextCDataSection
	self expectNextAll: '[CDATA['.

	driver handleCData: (self expectUpToAll: ']]>').
! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/18/2014 02:24'!
nextComment
	| nextChar |

	"Skip first -"
	streamReader next.
	self expectNext: $-.

	writeStream reset.
	[(nextChar := streamReader next) == nil
		or: [nextChar == $-
			and: [(streamReader peek) == $-]]]
		whileFalse: [writeStream nextPut: nextChar].
	self
		expectNext: $-;
		expectTerminator: $>.

	driver handleComment: writeStream contents.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 17:32'!
nextContentMarkupToken
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDeclaration].
	streamReader peek == $!!
		ifTrue: [
			streamReader next.
			streamReader peek == $-
				ifTrue: [^self nextComment].
			^ self nextCDataSection].
	self nextTag.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:15'!
nextContentToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/3/2014 22:26'!
nextDecodedCharReference
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/18/2014 00:54'!
nextDoctypeDeclaration
	|  root externalId |

	context := context doctypeContext.
	self expectNextAll: 'DOCTYPE'.
	streamReader skipSeparators.

	root := self nextName.
	streamReader skipSeparators.
	externalId := self nextExternalIDSystemLiteralRequired: true.
	driver
		handleStartDTD: root
		publicID: externalId key
		systemID: externalId value.

	streamReader skipSeparators.
	streamReader peek == $[
		ifTrue: [self nextInternalSubsetStart]
		ifFalse: [self nextEndDoctypeDeclaration].! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/18/2014 00:54'!
nextEndDoctypeDeclaration
	streamReader skipSeparators.
	self expectTerminator: $>.
	context := context prologContext.

	driver handleEndDTD.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/24/2014 21:11'!
nextEndDocument
	context isTerminatedContext
		ifFalse: [
			self closeStreams.
			context := context terminatedContext.
			driver handleEndDocument]! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/18/2014 00:54'!
nextEndInternalSubset
	self
		expectTerminator: $];
		nextEndDoctypeDeclaration! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:36'!
nextExternalIDSystemLiteralRequired: aBoolean
	| publicId systemId |

	(streamReader atEnd not
		and: [streamReader peek == $P
			or: [streamReader peek == $S]])
		ifTrue: [
			streamReader peek == $P
				ifTrue: [
					self
						expectNextAll: 'PUBLIC';
						skipSeparatorsReplacingParameterEntities.
					publicId := self nextPubidLiteral.

					self skipSeparatorsReplacingParameterEntities.
					systemId := self nextSystemLiteralRequired: aBoolean]
				ifFalse: [
					self
						expectNextAll: 'SYSTEM';
						skipSeparatorsReplacingParameterEntities.
					systemId := self nextSystemLiteralRequired: true]].

	^ (publicId ifNil: ['']) -> (systemId ifNil: ['']).! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/11/2013 01:47'!
nextGeneralEntityReference
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/9/2014 16:35'!
nextInternalSubsetStart
	"skip ["
	streamReader next.
	context := context internalSubsetContext.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 01:14'!
nextName
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/3/2014 01:12'!
nextPCDataToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 17:32'!
nextPIOrXMLDeclaration
	| piTarget piData |

	"Skip ?"
	streamReader next.
	piTarget := self nextPITarget.
	(piTarget = 'xml'
		and: [context isPrologContext])
		ifTrue: [^ self nextXMLDeclaration].
	streamReader skipSeparators.
	piData := self expectUpToAll: '?>'.

	driver handlePI: piTarget data: piData.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:16'!
nextPITarget
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:47'!
nextParameterEntityReference
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 03:19'!
nextPrologToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:18'!
nextPubidLiteral
	self subclassResponsiblity! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/17/2014 02:02'!
nextStartDocument
	context := driver initialContext ifNil: [XMLPrologContext new].
	context isPrologContext
		ifTrue: [streamReader detectCurrentEncoding].
	driver handleStartDocument.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:21'!
nextSubsetMarkupToken
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/9/2014 16:14'!
nextSubsetToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].

	streamReader peek == $%
		ifTrue: [^ self replaceParameterEntityReference].
	(context isInternalSubsetContext and: [streamReader peek == $]])
		ifTrue: [^ self nextEndInternalSubset].

	self nextSubsetMarkupToken.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 03:22'!
nextSystemLiteralRequired: aBoolean
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/7/2013 04:30'!
nextTag
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 18:25'!
nextToken
	(context isInitializedContext
		and: [streamReader atEnd])
		ifTrue: [^ self nextEndDocument].
	context nextTokenFrom: self.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:33'!
nextXMLAttributeName: aName
	self expectNextAll: aName.
	streamReader skipSeparators.
	self expectNext: $=.
	streamReader skipSeparators.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 17:31'!
nextXMLDeclaration
	| version encoding standalone |

	version := self nextXMLVersionAttributeRequired: true.
	(encoding := self nextXMLEncodingAttributeRequired: false)
		ifNotEmpty: [streamReader convertFromEncoding: encoding].
	standalone := self nextXMLStandaloneAttribute.

	streamReader skipSeparators.
	self
		expectNext: $?;
		expectTerminator: $>.

	driver
		handleXMLVersion: version
		encoding: encoding
		standalone: standalone.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLEncodingAttributeRequired: aBoolean
	streamReader skipSeparators.
	(aBoolean or: [streamReader peek == $e])
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'encoding';
		nextXMLEncodingAttributeValue.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:47'!
nextXMLEncodingAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:43'!
nextXMLStandaloneAttribute
	streamReader skipSeparators.
	(streamReader peek == $s)
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'standalone';
		nextXMLStandaloneAttributeValue.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:33'!
nextXMLVersionAttributeRequired: aBoolean
	streamReader skipSeparators.
	(aBoolean or: [streamReader peek == $v])
		ifFalse: [^ ''].

	^ self
		nextXMLAttributeName: 'version';
		nextXMLVersionAttributeValue.! !

!XMLParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:34'!
nextXMLVersionAttributeValue
	self subclassResponsibility! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/16/2014 09:08'!
normalizedLineEndingChar
	^ streamReader normalizedLineEndingChar! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/18/2013 17:26'!
normalizedLineEndingChar: aCharacter
	streamReader normalizedLineEndingChar: aCharacter! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/7/2013 04:31'!
replaceParameterEntityReference
	streamReader next.
	self nextParameterEntityReference! !

!XMLParserTokenizer methodsFor: 'initialization' stamp: 'monty 11/17/2014 02:00'!
setDriver: aDriver stream: aStream readLimit: aReadLimit
	driver := aDriver.
	streamReader :=
		(XMLNestedStreamReader
			on: aStream
			readLimit: aReadLimit).
	writeStream := WriteStream on: (String new: 128).
	nameStream := WriteStream on: (String new: 16).
	context := XMLUnitializedContext new.! !

!XMLParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/9/2014 16:14'!
skipSeparatorsReplacingParameterEntities
	"this can be used to replace references within declarations"

	streamReader skipSeparators.
	context isExternalSubsetContext
		ifFalse: [^ self].
	[streamReader peek == $%]
		whileTrue: [
			self replaceParameterEntityReference.
			streamReader skipSeparators].! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
streamReader
	^ streamReader! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 12/7/2013 00:35'!
tokenContext
	^ context! !

!XMLParserTokenizer methodsFor: 'accessing' stamp: 'monty 11/9/2014 15:59'!
tokenContext: aTokenContext
	context := aTokenContext! !

XMLParserTokenizer subclass: #XMLWellFormedParserTokenizer
	instanceVariableNames: 'activeEntities'
	classVariableNames: 'AdditionalNameChars AttributeTypeDelimiters EncodingNameChars NameChars NormalizedSpace PubidChars UnaryOperatorChars ValueListChars'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLWellFormedParserTokenizer commentStamp: 'monty 1/17/2014 01:50' prior: 0!
This is tokenizer for well-formed XML that supports DTD subsets and entity replacement.!

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 06:57'!
initialize
	"self initialize"

	self
		initializeAdditionalNameChars;
		initializeNameChars;
		initializePubidChars;
		initializeAttributeTypeDelimiters;
		initializeValueListChars;
		initializeUnaryOperatorChars;
		initializeEncodingNameChars;
		initializeNormalizedSpace! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'JAA 3/22/2011 12:43'!
initializeAdditionalNameChars
	"This creates a character set that contains all those defined in the XML spec for the
	NameChar production rule minus those also defined for NameStartChar
	(NameChar - NameStartChar)"

	(AdditionalNameChars := BitmapCharacterSet new: 16r2041)
		add: $-;
		add: $.;
		addAll: ($0 to: $9);
		add: (Character value: 16rB7).
	16r0300 to: 16r036F do: [:each | AdditionalNameChars add: (Character value: each)].
	16r203F to: 16r2040 do: [:each | AdditionalNameChars add: (Character value: each)].! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:44'!
initializeAttributeTypeDelimiters
	AttributeTypeDelimiters := BitmapCharacterSet with: $# with: $' with: $" with: $>! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:45'!
initializeEncodingNameChars
	(EncodingNameChars := BitmapCharacterSet new)
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9);
		add: $-;
		add: $_;
		add: $.! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 2/1/2014 02:50'!
initializeNameChars
	"This creates a character set that coveres the entire range of characters speicified for
	the NameChar production rule in the XML spec."

	(NameChars := BitmapCharacterSet new: 16rF0000)
		add: $:;
		addAll: ($A to: $Z);
		add: $_;
		addAll: ($a to: $z).
	16rC0 to: 16rD6 do: [:each | NameChars add: (Character value: each)].
	16rD8 to: 16rF6 do: [:each | NameChars add: (Character value: each)].
	16rF8 to: 16r2FF do: [:each | NameChars add: (Character value: each)].
	16r370 to: 16r37D do: [:each | NameChars add: (Character value: each)].
	16r37F to: 16r1FFF do: [:each | NameChars add: (Character value: each)].
	16r200C to: 16r200D do: [:each | NameChars add: (Character value: each)].
	16r2070 to: 16r218F do: [:each | NameChars add: (Character value: each)].
	16r2C00 to: 16r2FEF do: [:each | NameChars add: (Character value: each)].
	16r3001 to: 16rD7FF do: [:each | NameChars add: (Character value: each)].
	16rF900 to: 16rFDCF do: [:each | NameChars add: (Character value: each)].
	16rFDF0 to: 16rFFFD do: [:each | NameChars add: (Character value: each)].
	16r10000 to: 16rEFFFF do: [:each | NameChars add: (Character value: each)].

	AdditionalNameChars do: [:each | NameChars add: each].! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 2/10/2014 01:18'!
initializeNormalizedSpace
	NormalizedSpace := Character space! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 11/12/2013 01:44'!
initializePubidChars
	(PubidChars := BitmapCharacterSet new)
		add: Character space;
		add: Character cr;
		add: Character lf;
		addAll: ($a to: $z);
		addAll: ($A to: $Z);
		addAll: ($0 to: $9);
		addAll: '-''()+,./:=?;!!*#@$_%'! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:45'!
initializeUnaryOperatorChars
	UnaryOperatorChars := BitmapCharacterSet with: $? with: $* with: $+! !

!XMLWellFormedParserTokenizer class methodsFor: 'class initialization' stamp: 'monty 1/21/2014 23:45'!
initializeValueListChars
	ValueListChars := BitmapCharacterSet with: $( with: $| with: $)! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 2/10/2014 03:44'!
isName: aString
	aString ifEmpty: [^ false].

	aString doWithIndex: [:each :i | 
		((NameChars includes: each)
			and: [i > 1 or: [(AdditionalNameChars includes: each) not]])
			ifFalse: [^ false]].
	^ true.! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 11/19/2014 18:48'!
isNmtoken: aString
	^ aString notEmpty
		and: [aString allSatisfy: [:each | self isNmtokenChar: each]]! !

!XMLWellFormedParserTokenizer class methodsFor: 'testing' stamp: 'monty 2/1/2014 02:50'!
isNmtokenChar: aCharacter
	^ NameChars includes: aCharacter! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/17/2014 05:28'!
activeEntities
	^ activeEntities ifNil: [activeEntities := OrderedCollection new]! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 10/31/2014 17:45'!
addActiveEntity: anEntity
	(self hasActiveEntityNamed: anEntity name)
		ifTrue: [self errorSelfReferentialEntity: anEntity].
	self maxEntityReplacementDepth
		ifNotNil: [:maxDepth |
			(self activeEntities size + 1> maxDepth)
				ifTrue: [self errorEntityDepthLimitExceeded]].
	
	self activeEntities addLast: anEntity.
	anEntity isExternal
		ifTrue: [driver currentURI: anEntity uri].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/10/2014 00:47'!
constructReplacementForBypassedEntity: anEntity
	streamReader pushBack: anEntity asEscapedReference! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/17/2014 01:50'!
constructReplacementForBypassedGeneralEntity: anEntity
	self constructReplacementForBypassedEntity: anEntity! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/17/2014 01:51'!
constructReplacementForBypassedParameterEntity: anEntity
	context isLiteralValueContext
		ifTrue: [self constructReplacementForBypassedEntity: anEntity]! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/17/2014 21:38'!
constructReplacementForGeneralEntity: anEntity
	context isLiteralValueContext
		ifTrue: [
			anEntity isExternal
				ifTrue: [self errorLiteralContextExternalEntity: anEntity].
			^ self constructReplacementForLiteralContextEntity: anEntity].

	self addActiveEntity: anEntity.
	driver handleStartContentEntityReplacement: anEntity name.
	self
		pushBackEntity: anEntity
		onClose: [
			driver handleEndContentEntityReplacement: anEntity name.
			self removeActiveEntity: anEntity].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/17/2014 21:38'!
constructReplacementForLiteralContextEntity: anEntity
	anEntity replacement
		ifEmpty: [^ self].

	self addActiveEntity: anEntity.
	streamReader
		pushStream: (ReadStream on: anEntity replacement)
		onClose: [
			streamReader isEscapingQuotes: false.
			self removeActiveEntity: anEntity].
	anEntity isExternal
		ifTrue: [
			streamReader detectCurrentEncoding.
			(context := context textDeclarationContext)
				nextTokenFrom: self].

	self currentActiveEntity == anEntity
		ifTrue: [streamReader isEscapingQuotes: true].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/17/2014 21:38'!
constructReplacementForParameterEntity: anEntity
	context isLiteralValueContext
		ifTrue: [^ self constructReplacementForLiteralContextEntity: anEntity].

	streamReader pushBackCharacter: NormalizedSpace.
	self addActiveEntity: anEntity.
	self
		pushBackEntity: anEntity
		onClose: [self removeActiveEntity: anEntity].
	streamReader pushBackCharacter: NormalizedSpace.! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 11/16/2014 05:53'!
currentActiveEntity
	^ self activeEntities lastOrNil! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 15:40'!
errorEntityDepthLimitExceeded
	XMLLimitException signal: 
		'Cannot further nest entity replacements; ',
		'use #maxEntityReplacementDepth: to change or disable'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 12/10/2013 07:28'!
errorExpected: anExpectedString
	self parseError: 'Expected ', anExpectedString! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 15:34'!
errorExpected: anExpectedCharacterOrString butGot: aReceivedCharacterOrString
	self errorExpected:
		('{1} but got "{2}"' format:
			(Array
				with: anExpectedCharacterOrString asString
				with: (aReceivedCharacterOrString ifNil: ['']) asString))! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 17:41'!
errorExpectedAttributeValueEnd
	self errorExpected: 'end of quoted attribute value '! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 17:41'!
errorExpectedAttributeValueStart
	self errorExpected: 'start of quoted attribute value'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/6/2014 12:38'!
errorExpectedLiteral: aCharacterOrString
	self
		errorExpectedLiteral: aCharacterOrString
		butGot: nil! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 15:26'!
errorExpectedLiteral: anExpectedLiteral butGot: aCharacter
	self
		errorExpectedLiteral: anExpectedLiteral
		butGot: aCharacter
		at: 1! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 15:46'!
errorExpectedLiteral: anExpectedLiteral butGot: aCharacter at: aPosition
	| literal replacement |

	literal := anExpectedLiteral asString.
	replacement := (aCharacter ifNil: ['']) asString.
	self errorExpected:
		('"{1}" but got "{2}"' format:
			(Array
				with: literal
				with:
					(literal
						copyReplaceFrom: aPosition
						to: literal size
						with: replacement)))! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 15:10'!
errorExpectedName
	self errorExpected: 'a name'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/6/2014 12:38'!
errorExpectedSectionTerminator
	self errorExpectedLiteral: ']]>'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 11/1/2014 12:55'!
errorInternalSubsetIncludeOrIgnoreSection
	self parseError: 'Cannot have <!![INCLUDE[...]> or <!![IGNORE[...]> sections in internal subset'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 20:36'!
errorInvalidCodePoint: aCodePoint
	self parseError: 'Invalid character code point ', aCodePoint printString! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 20:39'!
errorLiteralContextExternalEntity: anEntity
	self parseError: 'External general entity ', anEntity asReference, ' in literal context'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 20:43'!
errorParameterEntityTerminatingSubset
	self parseError: 'Parameter entity replacement cannot terminate internal subset'! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 17:45'!
errorSelfReferentialEntity: anEntity
	self parseError: 'Illegal self-referential entity ', anEntity asReference! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 10/31/2014 20:38'!
errorXMLDeclarationInContent
	self parseError: 'XML declaration outside of prolog or text declaration'! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/4/2014 06:50'!
expectDigit
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [nextChar isDigit])
		ifFalse: [
			self
				errorExpected: 'digit'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 1/4/2014 06:52'!
expectLetter
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [nextChar isLetter])
		ifFalse: [
			self
				errorExpected: 'letter'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/1/2014 15:27'!
expectNext: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpectedLiteral: aCharacter
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/2/2014 15:02'!
expectNextAll: anExpectedLiteral
	1 to: anExpectedLiteral size do: [:i | | nextChar |
		(nextChar := streamReader next) == (anExpectedLiteral at: i)
			ifFalse: [
				self
					errorExpectedLiteral: anExpectedLiteral
					butGot: nextChar
					at: i]].
	^ anExpectedLiteral.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 12/9/2013 06:51'!
expectQuote
	| nextChar |

	(((nextChar := streamReader next) == $")
		or: [nextChar == $'])
		ifFalse: [
			self
				errorExpected: 'quote character delimiter'
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/1/2014 15:35'!
expectTerminator: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpectedLiteral: aCharacter
				butGot: nextChar].
	^ nextChar.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - expecting' stamp: 'monty 11/6/2014 12:38'!
expectUpToAll: aString
	| isTerminated |

	"upToAll: can't distinguish between a missing terminator or terminator at the end of a string"
	isTerminated := false.
	writeStream reset.
	[streamReader atEnd
		or: [(isTerminated := streamReader nextMatchAll: aString)]]
		whileFalse: [writeStream nextPut: streamReader next].

	isTerminated
		ifFalse: [self errorExpectedLiteral: aString].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:55'!
hasActiveEntities
	^ activeEntities notNil and: [activeEntities notEmpty]! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:58'!
hasActiveEntityNamed: aName
	^ self activeEntities anySatisfy: [:each | each name = aName]! !

!XMLWellFormedParserTokenizer methodsFor: 'testing' stamp: 'monty 12/13/2013 19:59'!
hasActiveExternalEntity
	^ self activeEntities anySatisfy: [:each | each isExternal]! !

!XMLWellFormedParserTokenizer methodsFor: 'accessing' stamp: 'monty 1/17/2014 05:28'!
maxEntityReplacementDepth
	^ driver maxEntityReplacementDepth! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 2/6/2014 01:58'!
nextAttlistDeclaration
	| element defaultPragma |

	self
		expectNextAll: 'ATTLIST';
		skipSeparatorsReplacingParameterEntities.

	element := self nextName.
	[self skipSeparatorsReplacingParameterEntities.
	(streamReader atEnd or: [streamReader peek == $>])]
		whileFalse: [
			driver
				handleAttributeDeclaration: element
				name: self nextName
				type: self nextAttributeType
				defaultPragma: (defaultPragma := self nextAttributeDefaultPragma)
				defaultValue: (self nextAttributeDefaultValueForPragma: defaultPragma)].

	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/28/2014 05:38'!
nextAttributeDefaultPragma
	self skipSeparatorsReplacingParameterEntities.
	streamReader peek == $#
		ifFalse: [^ ''].
	streamReader next.

	^ streamReader peek == $R
		ifTrue: [self expectNextAll: 'REQUIRED']
		ifFalse: [
			streamReader peek == $I
				ifTrue: [self expectNextAll: 'IMPLIED']
				ifFalse: [self expectNextAll: 'FIXED']].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/15/2014 23:54'!
nextAttributeDefaultValueForPragma: aPragma
	self skipSeparatorsReplacingParameterEntities.

	(aPragma isEmpty or: [aPragma first == $F]) "FIXED"
		ifTrue: [ | previousContext attributeValue |
			previousContext := context.
			context := context contentContext.
			attributeValue := self nextAttributeValue.
			context := previousContext.
			^ attributeValue]
		ifFalse: [^ ''].
! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 10/29/2014 08:09'!
nextAttributeType
	| nextChar |

	writeStream reset.
	[self skipSeparatorsReplacingParameterEntities.
	(nextChar := streamReader peek) == nil
		or: [AttributeTypeDelimiters includes: nextChar]]
		whileFalse: [
			(ValueListChars includes: nextChar)
				ifTrue: [
					writeStream position > 0
						ifTrue: [writeStream space].
					writeStream nextPut: streamReader next]
				ifFalse: [self nextNmtokenOn: writeStream]].
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/15/2014 23:42'!
nextAttributeValue
	| quote nextChar |

	((quote := streamReader next) == $"
		or: [quote == $'])
		ifFalse: [self errorExpectedAttributeValueStart].
	
	context enterLiteralValue.
	writeStream reset.
	[(nextChar := streamReader peek) == quote
		or: [nextChar == $<
			or: [nextChar == nil]]]
		whileFalse: [
			(streamReader next) == $&
				ifTrue: [
					streamReader peek == $#
						ifTrue: [writeStream nextPut: self nextDecodedCharReference]
						ifFalse: [
							(nextChar := self nextGeneralEntityReference)
								ifNotNil: [writeStream nextPut: nextChar]]]
				ifFalse: [
					nextChar isSeparator
						ifTrue: [nextChar := NormalizedSpace].
					writeStream nextPut: nextChar]].
	context exitLiteralValue.

	(streamReader next == quote)
		ifFalse: [self errorExpectedAttributeValueEnd].

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/28/2014 01:30'!
nextConditionalSection
	"skip ["
	streamReader next.

	writeStream reset.
	self nextConditionalSectionOn: writeStream.
	streamReader pushBack: writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/6/2014 18:33'!
nextConditionalSectionOn: aWriteStream
	self
		skipSeparatorsReplacingParameterEntities;
		expectNext: $I.

	streamReader peek == $N
		ifTrue: [self nextIncludeSectionOn: aWriteStream]
		ifFalse: [self nextIgnoreSection].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 2/3/2014 01:12'!
nextContentToken
	streamReader peek == $<
		ifTrue: [
			streamReader next.
			^ self nextContentMarkupToken].
	self nextPCDataToken.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 10/31/2014 17:46'!
nextDecodedCharReference
	| codePoint |

	"skip #"
	streamReader next.
	(codePoint := streamReader nextInteger)
		ifNil: [self errorExpected: 'integer'].
	self expectNext: $;.

	^ (self characterFromCodePoint: codePoint)
		ifNil: [self errorInvalidCodePoint: codePoint].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 1/30/2014 03:21'!
nextElementDeclaration
	| name nextChar contentModel |

	self
		expectNextAll: 'LEMENT';
		skipSeparatorsReplacingParameterEntities.

	name := self nextName.
	writeStream reset.
	[self skipSeparatorsReplacingParameterEntities.
	(streamReader atEnd or: [(nextChar := streamReader peek) == $>])]
		whileFalse: [
			nextChar == $#
				ifTrue: [writeStream nextPutAll: (self expectNextAll: '#PCDATA')]
				ifFalse: [
					(nextChar == $, or: [ValueListChars includes: nextChar])
						ifTrue: [writeStream nextPut: streamReader next]
						ifFalse: [self nextNameOn: writeStream]].
			(UnaryOperatorChars includes: streamReader peek)
				ifTrue: [writeStream nextPut: streamReader next]].
	contentModel := writeStream contents.

	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	driver
		handleElementDeclaration: name
		contentModel: contentModel.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:14'!
nextEndDocument
	context isInternalSubsetContext
		ifTrue: [self errorExpected: 'end of internal subset'].
	super nextEndDocument.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 10/31/2014 20:42'!
nextEndInternalSubset
	self hasActiveEntities
		ifTrue: [self errorParameterEntityTerminatingSubset].

	super nextEndInternalSubset.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 12/9/2013 01:27'!
nextEndTag
	| tagName |

	"Skip /"
	streamReader next.
	tagName := self nextName.
	streamReader skipSeparators.
	self expectTerminator: $>.

	driver handleEndTag: tagName.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/9/2014 16:14'!
nextEntityDeclaration
	| isParameterEntity name  |

	self expectNextAll: 'NTITY'.

	"this is to distinguish a parameter reference from declaration"
	isParameterEntity := false.
	[streamReader skipSeparators.
	streamReader peek == $% and: [isParameterEntity not]]
		whileTrue: [
			streamReader next.
			(context isExternalSubsetContext
				and: [streamReader atEnd not
					and: [streamReader peek isSeparator not]])
				ifTrue: [self nextParameterEntityReference]
				ifFalse: [isParameterEntity := true]].

	self skipSeparatorsReplacingParameterEntities.
	name := self nextName.
	self skipSeparatorsReplacingParameterEntities.
	streamReader atQuote
		ifTrue: [
			self
				nextInternalEntityDeclaration: name
				isParameterEntity: isParameterEntity]
		ifFalse: [
			self
				nextExternalEntityDeclaration: name
				isParameterEntity: isParameterEntity].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/9/2014 17:48'!
nextEntityValue
	| quote value nextChar |

	quote := streamReader next.
	context enterLiteralValue.
	writeStream reset.
	[(nextChar := streamReader peek) == quote
		or: [nextChar == nil]]
		whileFalse: [
			streamReader next == $&
				ifTrue: [
					writeStream nextPut: 
						(streamReader peek == $#
							ifTrue: [self nextDecodedCharReference]
							ifFalse: [$&])]
				ifFalse: [
					nextChar == $%
						ifTrue: [self nextParameterEntityReference]
						ifFalse: [writeStream nextPut: nextChar]]].
	value := writeStream contents.
	context exitLiteralValue.
	self expectNext: quote.

	^ value.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:09'!
nextExternalEntityDeclaration: aName isParameterEntity: aBoolean
	| externalId notation |

	externalId := self nextExternalIDSystemLiteralRequired: true.
	aBoolean
		ifTrue: [
			driver
				handleParameterEntityDeclaration: aName
				publicID: externalId key
				systemID: externalId value]
		ifFalse: [
			self skipSeparatorsReplacingParameterEntities.
			streamReader peek == $>
				ifTrue: [notation := '']
				ifFalse: [
					self
						expectNextAll: 'NDATA';
						skipSeparatorsReplacingParameterEntities.
					notation := self nextName].
			driver
				handleGeneralEntityDeclaration: aName
				publicID: externalId key
				systemID: externalId value
				ndata: notation].
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/18/2014 07:11'!
nextGeneralEntityReference
	| name |

	name := self nextName.
	self expectNext: $;.
	PredefinedEntities
		at: name
		ifPresent: [:character | ^ character].

	(driver handleGeneralEntityReference: name)
		ifNotNil: [:entity | entity constructReplacementOn: self].
	^ nil.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/6/2014 18:34'!
nextIgnoreSection
	| openSections |

	self expectNextAll: 'GNORE'.
	streamReader skipSeparators.
	self expectNext: $[.

	openSections := 1.
	[openSections > 0 and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: ']]>')
				ifTrue: [openSections := openSections - 1]
				ifFalse: [
					(streamReader nextMatchAll: '<!![')
						ifTrue: [openSections := openSections + 1]
						ifFalse: [streamReader next]]].

	openSections > 0
		ifTrue: [self errorExpectedSectionTerminator].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 10/31/2014 18:32'!
nextIncludeSectionOn: aWriteStream
	| isOpen  |

	self expectNextAll: 'NCLUDE'.
	streamReader skipSeparators.
	self expectNext: $[.

	isOpen := true.
	[isOpen and: [streamReader atEnd not]]
		whileTrue: [
			(streamReader nextMatchAll: '<!![')
				ifTrue: [self nextConditionalSectionOn: aWriteStream]
				ifFalse: [
					(streamReader nextMatchAll: ']]>')
						ifTrue: [isOpen := false]
						ifFalse: [aWriteStream nextPut: streamReader next]]].

	isOpen
		ifTrue: [self errorExpectedSectionTerminator].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/11/2013 01:44'!
nextInternalEntityDeclaration: aName isParameterEntity: aBoolean
	| value |

	value := self nextEntityValue.
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	aBoolean
		ifTrue: [driver handleParameterEntityDeclaration: aName replacement: value]
		ifFalse: [driver handleGeneralEntityDeclaration: aName replacement: value].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 10/31/2014 15:11'!
nextName
	| nextChar |

	nameStream reset.
	"inlined from nextNameOn:"
	((nextChar := streamReader next) notNil
		and: [(NameChars includes: nextChar)
			and: [(AdditionalNameChars includes: nextChar) not]])
		ifFalse: [self errorExpectedName].
	nameStream nextPut: nextChar.

	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [nameStream nextPut: streamReader next].
	^ nameStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 10/31/2014 15:12'!
nextNameOn: aWriteStream
	| nextChar |

	((nextChar := streamReader next) notNil
		and: [(NameChars includes: nextChar)
			and: [(AdditionalNameChars includes: nextChar) not]])
		ifFalse: [self errorExpectedName].
	aWriteStream nextPut: nextChar.

	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [aWriteStream nextPut: streamReader next].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 2/1/2014 02:53'!
nextNmtokenOn: aWriteStream
	"matches the nmtoken production"
	| nextChar |

	((nextChar := streamReader peek) notNil
		and: [(NameChars includes: nextChar)])
		ifFalse: [self errorExpected: 'name token'].
	aWriteStream nextPut: streamReader next.
		
	[(nextChar := streamReader peek) notNil
		and: [NameChars includes: nextChar]]
		whileTrue: [aWriteStream nextPut: streamReader next].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/12/2013 00:10'!
nextNotationDeclaration
	| name externalId |

	self
		expectNextAll: 'NOTATION';
		skipSeparatorsReplacingParameterEntities.

	name := self nextName.
	self skipSeparatorsReplacingParameterEntities.

	externalId := self nextExternalIDSystemLiteralRequired: false.
	self
		skipSeparatorsReplacingParameterEntities;
		expectTerminator: $>.

	driver
		handleNotationDeclaration: name
		publicID: externalId key
		systemID: externalId value.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 10/30/2014 17:18'!
nextPCDataToken
	| isIgnorableWhitespace nextChar |

	isIgnorableWhitespace := true.
	writeStream reset.
	[(nextChar := streamReader peek) == $<
		or: [nextChar == nil]]
		whileFalse: [
			streamReader next == $&
				ifTrue: [
					(nextChar :=
						streamReader peek == $#
							ifTrue: [self nextDecodedCharReference]
							ifFalse: [self nextGeneralEntityReference])
						ifNotNil: [
							writeStream nextPut: nextChar.
							isIgnorableWhitespace := false]]
				ifFalse: [
					isIgnorableWhitespace
						ifTrue: [isIgnorableWhitespace := nextChar isSeparator].
					writeStream nextPut: nextChar]].
	isIgnorableWhitespace
		ifTrue: [
			writeStream position > 0
				ifTrue: [driver handleWhitespace: writeStream contents]]
		ifFalse: [driver handlePCData: writeStream contents].! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 18:40'!
nextPITarget
	| target |

	target := self nextName.
	(target = 'xml'
		and: [context isPrologContext not])
		ifTrue: [self errorXMLDeclarationInContent].

	^ target.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/16/2014 21:00'!
nextParameterEntityReference
	| name |

	name := self nextName.
	self expectNext: $;.

	(driver handleParameterEntityReference: name)
		ifNotNil: [:entity | entity constructReplacementOn: self]! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/18/2014 00:54'!
nextPrologToken
	streamReader skipSeparators.
	streamReader atEnd
		ifTrue: [^ self].

	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDeclaration].
	streamReader peek == $!!
		ifTrue: [
			streamReader next.
			streamReader peek == $-
				ifTrue: [^ self nextComment].
			^ self nextDoctypeDeclaration].

	context := context contentContext.
	self nextTag.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 10/31/2014 21:08'!
nextPubidLiteral
	| quote nextChar |

	quote := self expectQuote.
	writeStream reset.
	[(nextChar := streamReader peek) == nil
		or: [nextChar == quote
			or: [(PubidChars includes: nextChar) not]]]
		whileFalse: [writeStream nextPut: streamReader next].
	self expectNext: quote.

	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/10/2013 07:20'!
nextSubsetDeclaration
	streamReader peek == $E
		ifTrue: [
			streamReader next.
			^ streamReader peek == $N
				ifTrue: [self nextEntityDeclaration]
				ifFalse: [self nextElementDeclaration]].
	streamReader peek == $A
		ifTrue: [^ self nextAttlistDeclaration].
	streamReader peek == $N
		ifTrue: [^ self nextNotationDeclaration].

	self errorExpected: 'declaration'.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 11/9/2014 17:34'!
nextSubsetMarkupToken
	self expectNext: $<.
	streamReader peek == $?
		ifTrue: [^ self nextPIOrXMLDeclaration].

	self expectNext: $!!.
	streamReader peek == $-
		ifTrue: [^ self nextComment].

	(streamReader peek == $[)
		ifTrue: [
			(context isExternalSubsetContext
				or: [self hasActiveExternalEntity])
				ifTrue: [^ self nextConditionalSection].
			self errorInternalSubsetIncludeOrIgnoreSection].

	self nextSubsetDeclaration.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing - dtd' stamp: 'monty 12/6/2013 08:36'!
nextSystemLiteralRequired: aBoolean	
	| quote value |

	(aBoolean or: [streamReader atQuote])
		ifTrue: [
			quote := self expectQuote.
			value := streamReader upTo: quote]
		ifFalse: [value := ''].
	^ value.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/20/2014 18:27'!
nextTag
	"performance critical method"
	| tagName attributes namespaces nextChar |

	(streamReader peek == $/)
		ifTrue: [^ self nextEndTag].
	tagName := self nextName.

	attributes := OrderPreservingStringDictionary new: 10.
	[streamReader skipSeparators.
	((nextChar := streamReader peek) == $>
		or: [nextChar == $/
			or: [nextChar == nil]])]
		whileFalse: [ | attributeName attributeValue |
			attributeName := self nextName.
			streamReader skipSeparators.
			self expectNext: $=.
			streamReader skipSeparators.
			attributeValue := self nextAttributeValue.
	
			(attributeName at: 1) == $x
				ifTrue: [
					(attributeName isNamespaceAttributeName
						and: [driver usesNamespaces])
						ifTrue: [
							"initialize only if needed"
							(namespaces
								ifNil: [namespaces := OrderPreservingStringDictionary new: 5])
								at:
									((attributeName size > 6)
										ifTrue: [attributeName copyFrom: 7 to: attributeName size]
										ifFalse: [''])
								put: attributeValue]
						ifFalse: [							
							attributeName isReservedAttributeName
								ifTrue: [driver haveUnhandledReservedAttributes].
							attributes at: attributeName put: attributeValue]]
				ifFalse: [attributes at: attributeName put: attributeValue]].

	driver
		handleStartTag: tagName
		attributes: attributes
		namespaces: namespaces.

	streamReader peek == $/
		ifTrue: [
			streamReader next.
			driver handleEndTag: tagName].
	self expectTerminator: $>.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/28/2014 01:39'!
nextXMLEncodingAttributeValue
	| quote |

	quote := self expectQuote.

	writeStream
		reset;
		nextPut: self expectLetter.
	[streamReader atEnd not
		and: [EncodingNameChars includes: streamReader peek]]
		whileTrue: [writeStream nextPut: streamReader next].

	self expectNext: quote.
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 1/4/2014 06:44'!
nextXMLStandaloneAttributeValue
	| quote yesOrNo |

	quote := self expectQuote.
	yesOrNo := 
		streamReader peek == $y
			ifTrue: [self expectNextAll: 'yes']
			ifFalse: [self expectNextAll: 'no'].
	self expectNext: quote.

	^ yesOrNo.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 18:41'!
nextXMLTextDeclaration
	(streamReader nextMatchAll: '<?xml')
		ifTrue: [
			self nextXMLVersionAttributeRequired: false.
			(self nextXMLEncodingAttributeRequired: true)
				ifNotEmpty: [:encoding | streamReader convertFromEncoding: encoding].

			streamReader skipSeparators.
			self
				expectNext: $?;
				expectTerminator: $>].
	context := context previousContext.! !

!XMLWellFormedParserTokenizer methodsFor: 'tokenizing' stamp: 'monty 11/18/2014 19:37'!
nextXMLVersionAttributeValue
	| quote |

	quote := self expectQuote.

	writeStream
		reset;
		nextPut: self expectDigit;
		nextPut: (self expectNext: $.);
		nextPut: self expectDigit.

	[streamReader atEnd
		or: [streamReader peek == quote]]
		whileFalse: [writeStream nextPut: self expectDigit].

	self expectNext: quote.
	^ writeStream contents.! !

!XMLWellFormedParserTokenizer methodsFor: 'errors' stamp: 'monty 12/7/2013 01:15'!
parseError: anErrorString
	driver handleParseError: anErrorString! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 11/16/2014 05:46'!
pushBackEntity: anEntity onClose: aBlock
	anEntity replacement
		ifEmpty: [^ aBlock value].

	streamReader
		pushStream: (ReadStream on: anEntity replacement)
		onClose: aBlock.
	anEntity isExternal
		ifTrue: [
			streamReader detectCurrentEncoding.
			(context := context textDeclarationContext)
				nextTokenFrom: self].! !

!XMLWellFormedParserTokenizer methodsFor: 'entity replacement' stamp: 'monty 1/20/2014 22:14'!
removeActiveEntity: anEntity
	self activeEntities remove: anEntity.
	anEntity isExternal
		ifTrue: [driver removeURI: anEntity uri].! !

Object subclass: #XMLSingleCharacterReadStream
	instanceVariableNames: 'character'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLSingleCharacterReadStream commentStamp: 'monty 11/20/2014 00:14' prior: 0!
A read stream for single characters.!

!XMLSingleCharacterReadStream class methodsFor: 'instance creation' stamp: 'monty 11/16/2014 04:43'!
on: aCharacter
	^ self new on: aCharacter! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/16/2014 04:44'!
atEnd
	^ character == nil! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/20/2014 04:18'!
basicNext
	^ self next! !

!XMLSingleCharacterReadStream methodsFor: 'file open/close' stamp: 'monty 11/20/2014 01:56'!
close! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/20/2014 02:02'!
isBinary
	^ false! !

!XMLSingleCharacterReadStream methodsFor: 'testing' stamp: 'monty 11/20/2014 01:56'!
isStream
	^ true! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/16/2014 04:44'!
next
	^ character ifNotNil: [:nextChar |
		character := nil.
		nextChar]! !

!XMLSingleCharacterReadStream methodsFor: 'initialization' stamp: 'monty 11/16/2014 04:50'!
on: aCharacter
	character := aCharacter! !

!XMLSingleCharacterReadStream methodsFor: 'accessing' stamp: 'monty 11/16/2014 04:44'!
peek
	^ character! !

Object subclass: #XMLStreamDecoder
	instanceVariableNames: ''
	classVariableNames: 'PreferredStreamDecoders StreamDecoderCache'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLStreamDecoder commentStamp: 'monty 11/11/2014 21:31' prior: 0!
An abstract class for stream decoders.!

!XMLStreamDecoder class methodsFor: 'testing' stamp: 'monty 11/11/2014 21:43'!
canDecodeEncoding: anEncoding
	^ false! !

!XMLStreamDecoder class methodsFor: 'accessing' stamp: 'monty 11/13/2014 05:34'!
defaultStreamDecoder
	^ XMLUTF8StreamDecoder! !

!XMLStreamDecoder class methodsFor: 'private' stamp: 'monty 11/12/2014 04:59'!
findStreamDecoderForEncoding: anEncoding
	self preferredStreamDecoders do: [:each |
		(each canDecodeEncoding: anEncoding)
			ifTrue: [^ each]].

	self allSubclassesDo: [:each |
		((self preferredStreamDecoders includes: each) not
			and: [each canDecodeEncoding: anEncoding])
			ifTrue: [^ each]].

	^ nil.! !

!XMLStreamDecoder class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 06:27'!
initialize
	"self initialize"

	StreamDecoderCache := XMLKeyValueCache maxSize: 128.
	PreferredStreamDecoders := IdentitySet newFrom: XMLUTFStreamDecoder allSubclasses.! !

!XMLStreamDecoder class methodsFor: 'testing' stamp: 'monty 11/13/2014 05:35'!
isDefaultStreamDecoder
	^ self == self defaultStreamDecoder! !

!XMLStreamDecoder class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 21:02'!
newForEncoding: anEncoding
	^ (StreamDecoderCache
		at: anEncoding
		ifAbsentPut: [self findStreamDecoderForEncoding: anEncoding])
			ifNotNil: [:decoderClass | decoderClass newForEncoding: anEncoding]! !

!XMLStreamDecoder class methodsFor: 'accessing' stamp: 'monty 11/12/2014 04:56'!
preferredStreamDecoders
	^ PreferredStreamDecoders! !

!XMLStreamDecoder methodsFor: 'testing' stamp: 'monty 11/14/2014 21:13'!
isDefaultStreamDecoder
	^ self class isDefaultStreamDecoder! !

!XMLStreamDecoder methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	self subclassResponsibility! !

XMLStreamDecoder subclass: #XMLTextConverterStreamDecoder
	instanceVariableNames: 'textConverter'
	classVariableNames: 'TextConverterCache'
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

XMLTextConverterStreamDecoder class
	instanceVariableNames: 'textConverterCache'!

!XMLTextConverterStreamDecoder commentStamp: 'monty 11/12/2014 04:34' prior: 0!
A stream decoder that uses Pharo TextConverter classes.!

XMLTextConverterStreamDecoder class
	instanceVariableNames: 'textConverterCache'!

!XMLTextConverterStreamDecoder class methodsFor: 'testing' stamp: 'monty 11/12/2014 04:36'!
canDecodeEncoding: anEncoding
	^ (Smalltalk includesKey: #TextConverter)
		and: [(self textConverterForEncoding: anEncoding) notNil]! !

!XMLTextConverterStreamDecoder class methodsFor: 'class initialization' stamp: 'monty 11/18/2014 06:29'!
initialize
	"self initialize"

	TextConverterCache := XMLKeyValueCache maxSize: 32.! !

!XMLTextConverterStreamDecoder class methodsFor: 'instance creation' stamp: 'monty 11/12/2014 04:36'!
newForEncoding: anEncoding
	^ self new textConverter: (self textConverterForEncoding: anEncoding) new! !

!XMLTextConverterStreamDecoder class methodsFor: 'instance creation' stamp: 'monty 11/15/2014 21:02'!
textConverterForEncoding: anEncoding
	^ TextConverterCache
		at: anEncoding
		ifAbsentPut: [
			(Smalltalk at: #TextConverter)
				defaultConverterClassForEncoding: anEncoding asLowercase]! !

!XMLTextConverterStreamDecoder methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	^ textConverter nextFromStream: aStream! !

!XMLTextConverterStreamDecoder methodsFor: 'accessing' stamp: 'monty 11/12/2014 01:55'!
textConverter
	^ textConverter! !

!XMLTextConverterStreamDecoder methodsFor: 'accessing' stamp: 'monty 11/12/2014 01:54'!
textConverter: aTextConverter
	textConverter := aTextConverter! !

XMLStreamDecoder subclass: #XMLUTFStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

XMLUTFStreamDecoder class
	instanceVariableNames: 'encodingNames'!

!XMLUTFStreamDecoder commentStamp: 'monty 11/20/2014 04:00' prior: 0!
An abstract class for Unicode stream decoders.!

XMLUTFStreamDecoder class
	instanceVariableNames: 'encodingNames'!

XMLUTFStreamDecoder subclass: #XMLUTF16StreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16StreamDecoder commentStamp: 'monty 11/20/2014 03:58' prior: 0!
An abstract class for UTF-16 stream decoders.!

XMLUTF16StreamDecoder subclass: #XMLUTF16BigEndianStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16BigEndianStreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for big endian UTF-16 streams.!

!XMLUTF16BigEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/12/2014 01:23'!
basicEncodingNames
	^ #('utf16' 'utf-16' 'utf16be' 'utf-16be' 'utf-16-be' 'utf16-be')! !

!XMLUTF16BigEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFE 16rFF)! !

!XMLUTF16BigEndianStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 14:28'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	^ (aFirstByte bitShift: 8) + aSecondByte! !

!XMLUTF16BigEndianStreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:47'!
isBigEndian
	^ true! !

XMLUTF16StreamDecoder subclass: #XMLUTF16LittleEndianStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF16LittleEndianStreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for little endian UTF-16 streams.!

!XMLUTF16LittleEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 22:05'!
basicEncodingNames
	^ #('utf16le' 'utf-16le' 'utf-16-le' 'utf16-le')! !

!XMLUTF16LittleEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFF 16rFE)! !

!XMLUTF16LittleEndianStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 14:28'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	^ (aSecondByte bitShift: 8) + aFirstByte! !

!XMLUTF16LittleEndianStreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:47'!
isBigEndian
	^ false! !

!XMLUTF16StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 14:44'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte
	self subclassResponsibility! !

!XMLUTF16StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 15:54'!
errorInvalidCodePoint
	XMLEncodingException signal: 'Invalid UTF-16 code point'! !

!XMLUTF16StreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:46'!
isBigEndian
	self subclassResponsibility! !

!XMLUTF16StreamDecoder methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	| codePoint |

	codePoint :=
		self
			codePointFromFirstByte:
				(aStream basicNext ifNil: [^ nil]) asciiValue
			secondByte:
				(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue.

	(codePoint < 16rD800 or: [codePoint > 16rDBFF])
		ifTrue: [^ self characterWithValue: codePoint]
		ifFalse: [
			^ self
				surrogatePairFromStream: aStream
				withHighSurrogate: codePoint]! !

!XMLUTF16StreamDecoder methodsFor: 'private' stamp: 'monty 11/12/2014 20:21'!
surrogatePairFromStream: aStream withHighSurrogate: aHighSurrogate
	| lowSurrogate |

	lowSurrogate :=
		self
			codePointFromFirstByte:
				(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue
			secondByte:
				(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue.

	^ self characterWithValue:
		((aHighSurrogate bitAnd: 16r3FF) bitShift: 10) +
		(lowSurrogate bitAnd: 16r3FF) +
		16r10000.! !

XMLUTFStreamDecoder subclass: #XMLUTF32StreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32StreamDecoder commentStamp: 'monty 11/20/2014 03:58' prior: 0!
An abstract class for UTF-32 stream decoders.!

XMLUTF32StreamDecoder subclass: #XMLUTF32BigEndianStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32BigEndianStreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for big endian UTF-32 streams.!

!XMLUTF32BigEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/12/2014 01:24'!
basicEncodingNames
	^ #('utf32' 'utf-32' 'utf32be' 'utf-32be' 'utf-32-be' 'utf32-be')! !

!XMLUTF32BigEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16r00 16r00 16rFE 16rFF)! !

!XMLUTF32BigEndianStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 18:57'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	^ self characterWithValue:
		(aFirstByte bitShift: 24) +
		(aSecondByte bitShift: 16) +
		(aThirdByte bitShift: 8) +
		aFourthByte! !

!XMLUTF32BigEndianStreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:49'!
isBigEndian
	^ true! !

XMLUTF32StreamDecoder subclass: #XMLUTF32LittleEndianStreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF32LittleEndianStreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for little endian UTF-32 streams.!

!XMLUTF32LittleEndianStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:59'!
basicEncodingNames
	^ #('utf32le' 'utf-32le' 'utf-32-le' 'utf32-le')! !

!XMLUTF32LittleEndianStreamDecoder class methodsFor: 'accessing' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rFF 16rFE 16r00 16r00)! !

!XMLUTF32LittleEndianStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 18:57'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	^ self characterWithValue:
		(aFourthByte bitShift: 24) +
		(aThirdByte bitShift: 16) +
		(aSecondByte bitShift: 8) +
		aFirstByte! !

!XMLUTF32LittleEndianStreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:50'!
isBigEndian
	^ false! !

!XMLUTF32StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 14:58'!
codePointFromFirstByte: aFirstByte secondByte: aSecondByte thirdByte: aThirdByte fourthByte: aFourthByte
	self subclassResponsibility! !

!XMLUTF32StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 15:54'!
errorInvalidCodePoint
	XMLEncodingException signal: 'Invalid UTF-32 code point'! !

!XMLUTF32StreamDecoder methodsFor: 'testing' stamp: 'monty 11/11/2014 14:49'!
isBigEndian
	self subclassResponsibility! !

!XMLUTF32StreamDecoder methodsFor: 'decoding' stamp: 'monty 11/17/2014 21:14'!
nextFromStream: aStream
	^ self
		codePointFromFirstByte:
			(aStream basicNext ifNil: [^ nil]) asciiValue
		secondByte:
			(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue
		thirdByte:
			(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue
		fourthByte: 
			(aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue! !

XMLUTFStreamDecoder subclass: #XMLUTF8StreamDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Streams'!

!XMLUTF8StreamDecoder commentStamp: 'monty 11/20/2014 03:59' prior: 0!
A stream decoder for UTF-8 streams.!

!XMLUTF8StreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:57'!
basicEncodingNames
	^ #('utf8' 'utf-8')! !

!XMLUTF8StreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 21:55'!
byteOrderMarkBytes
	^ #(16rEF 16rBB 16rBF)! !

!XMLUTF8StreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 04:39'!
errorInvalidCodePoint
	XMLEncodingException signal: 'Invalid UTF-8 code point'! !

!XMLUTF8StreamDecoder methodsFor: 'decoding' stamp: 'monty 11/20/2014 03:06'!
nextFromStream: aStream
	| firstChar firstByte |

	((firstChar := aStream basicNext) == nil
		or: [(firstByte := firstChar asciiValue) < 16r80]) "single byte character"
		ifTrue: [^ firstChar].

	firstByte < 16rC0 "second bit must be set"
		ifTrue: [self errorInvalidCodePoint].
	(firstByte bitAnd: 16rF0) < 16rE0 "two bytes"
		ifTrue: [
			^ self characterWithValue: 
				((firstByte bitAnd: 16r1F) bitShift: 6) +
				((aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue bitAnd: 16r3F)].
	(firstByte bitAnd: 16rF0) = 16rE0 "three bytes"
		ifTrue: [
			^ self characterWithValue: 
				((firstByte bitAnd: 16r0F) bitShift: 12) +
				(((aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue bitAnd: 16r3F) bitShift: 6) +
				((aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue bitAnd: 16r3F)].
	^ self characterWithValue: "four"
		((firstByte bitAnd: 16r07) bitShift: 18) +
		(((aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue bitAnd: 16r3F) bitShift: 12) +
		(((aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue bitAnd: 16r3F) bitShift: 6) +
		((aStream basicNext ifNil: [self errorInvalidCodePoint]) asciiValue bitAnd: 16r3F).! !

!XMLUTFStreamDecoder class methodsFor: 'private' stamp: 'monty 11/12/2014 03:25'!
basicEncodingNames
	^ #()! !

!XMLUTFStreamDecoder class methodsFor: 'accessing' stamp: 'monty 11/12/2014 05:10'!
byteOrderMark
	^ String newFrom: (self byteOrderMarkBytes collect: [:each | each asCharacter])! !

!XMLUTFStreamDecoder class methodsFor: 'private' stamp: 'monty 11/11/2014 22:01'!
byteOrderMarkBytes
	^ #()! !

!XMLUTFStreamDecoder class methodsFor: 'testing' stamp: 'monty 11/11/2014 22:07'!
canDecodeEncoding: anEncoding
	^ self encodingNames includes: anEncoding! !

!XMLUTFStreamDecoder class methodsFor: 'accessing' stamp: 'monty 11/18/2014 06:28'!
encodingNames
	^ encodingNames ifNil: [
		(encodingNames := self basicEncodingNames asSet)
			addAll: (self basicEncodingNames collect: [:each | each asUppercase]);
			yourself]! !

!XMLUTFStreamDecoder class methodsFor: 'instance creation' stamp: 'monty 11/12/2014 01:06'!
newForEncoding: anEncoding
	^ self new! !

!XMLUTFStreamDecoder methodsFor: 'converting' stamp: 'monty 11/11/2014 18:55'!
characterWithValue: aCodePoint
	aCodePoint > 16r10FFFF
		ifTrue: [self errorInvalidCodePoint].

	^ Character value: aCodePoint.! !

!XMLUTFStreamDecoder methodsFor: 'private' stamp: 'monty 11/11/2014 18:50'!
errorInvalidCodePoint
	self subclassResponsibility! !

Object subclass: #XMLTokenContext
	instanceVariableNames: 'isLiteralValueContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTokenContext commentStamp: '<historical>' prior: 0!
The context a token occurs in.!

XMLTokenContext subclass: #XMLContentContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLContentContext commentStamp: 'monty 11/9/2014 16:20' prior: 0!
A context for content tokens.!

!XMLContentContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isContextContext
	^ true! !

!XMLContentContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 15:41'!
nextTokenFrom: aTokenizer
	aTokenizer nextContentToken! !

XMLTokenContext subclass: #XMLPrologContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLPrologContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for prolog tokens.!

XMLPrologContext subclass: #XMLDoctypeContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDoctypeContext commentStamp: 'monty 11/9/2014 16:25' prior: 0!
A context for doctype tokens.!

!XMLDoctypeContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isDoctypeContext
	^ true! !

!XMLPrologContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isPrologContext
	^ true! !

!XMLPrologContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 15:41'!
nextTokenFrom: aTokenizer
	aTokenizer nextPrologToken! !

XMLTokenContext subclass: #XMLSubsetContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLSubsetContext commentStamp: 'monty 11/10/2014 00:34' prior: 0!
An abstract class for subset contexts.!

XMLSubsetContext subclass: #XMLExternalSubsetContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLExternalSubsetContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for external subset tokens.!

!XMLExternalSubsetContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isExternalSubsetContext
	^ true! !

XMLSubsetContext subclass: #XMLInternalSubetContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLInternalSubetContext commentStamp: 'monty 11/9/2014 16:23' prior: 0!
A context for internal subset tokens.!

!XMLInternalSubetContext methodsFor: 'testing' stamp: 'monty 11/10/2014 00:33'!
isDoctypeContext
	^ true! !

!XMLInternalSubetContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isInternalSubsetContext
	^ true! !

!XMLInternalSubetContext methodsFor: 'testing' stamp: 'monty 11/10/2014 00:33'!
isPrologContext
	^ true! !

!XMLSubsetContext methodsFor: 'testing' stamp: 'monty 11/10/2014 00:33'!
isSubsetContext
	^ true! !

!XMLSubsetContext methodsFor: 'tokenizing' stamp: 'monty 11/10/2014 00:32'!
nextTokenFrom: aTokenizer
	aTokenizer nextSubsetToken! !

XMLTokenContext subclass: #XMLTerminatedContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTerminatedContext commentStamp: 'monty 11/9/2014 16:28' prior: 0!
A context for after parsing.!

!XMLTerminatedContext methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
isTerminatedContext
	^ true! !

!XMLTerminatedContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:27'!
nextTokenFrom: aTokenizer
	aTokenizer nextEndDocument! !

XMLTokenContext subclass: #XMLTextDeclarationContext
	instanceVariableNames: 'previousContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTextDeclarationContext commentStamp: 'monty 11/9/2014 17:24' prior: 0!
A context for text declaration tokens.!

!XMLTextDeclarationContext class methodsFor: 'instance creation' stamp: 'monty 11/9/2014 17:13'!
previousContext: aTokenContex
	^ self new previousContext: aTokenContex! !

!XMLTextDeclarationContext methodsFor: 'testing' stamp: 'monty 11/9/2014 17:18'!
isTextDeclarationContext
	^ true! !

!XMLTextDeclarationContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 17:15'!
nextTokenFrom: aTokenizer
	aTokenizer nextXMLTextDeclaration! !

!XMLTextDeclarationContext methodsFor: 'accessing' stamp: 'monty 11/9/2014 18:27'!
previousContext
	^ previousContext! !

!XMLTextDeclarationContext methodsFor: 'accessing' stamp: 'monty 11/9/2014 17:13'!
previousContext: aTokenContex
	previousContext := aTokenContex! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
contentContext
	^ XMLContentContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
doctypeContext
	^ XMLDoctypeContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/9/2014 17:22'!
enterLiteralValue
	isLiteralValueContext := true! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/9/2014 17:22'!
exitLiteralValue
	isLiteralValueContext := false! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
externalSubsetContext
	^ XMLExternalSubsetContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:01'!
internalSubsetContext
	^ XMLInternalSubetContext new! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isContextContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isDoctypeContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isExternalSubsetContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:15'!
isInitializedContext
	^ true! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:14'!
isInternalSubsetContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 17:39'!
isLiteralValueContext
	^ isLiteralValueContext ifNil: [isLiteralValueContext := false]! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:12'!
isPrologContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:13'!
isSubsetContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/17/2014 02:03'!
isTerminatedContext
	^ false! !

!XMLTokenContext methodsFor: 'testing' stamp: 'monty 11/9/2014 17:18'!
isTextDeclarationContext
	^ false! !

!XMLTokenContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:40'!
nextTokenFrom: aTokenizer
	self subclassResponsibility! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:02'!
prologContext
	^ XMLPrologContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:03'!
terminatedContext
	^ XMLTerminatedContext new! !

!XMLTokenContext methodsFor: 'changing' stamp: 'monty 11/17/2014 02:01'!
textDeclarationContext
	^ XMLTextDeclarationContext previousContext: self! !

XMLTokenContext subclass: #XMLUnitializedContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLUnitializedContext commentStamp: 'monty 11/9/2014 16:28' prior: 0!
A context for before parsing.!

!XMLUnitializedContext methodsFor: 'testing' stamp: 'monty 11/9/2014 16:15'!
isInitializedContext
	^ false! !

!XMLUnitializedContext methodsFor: 'tokenizing' stamp: 'monty 11/9/2014 16:27'!
nextTokenFrom: aTokenizer
	aTokenizer nextStartDocument! !

Object subclass: #XMLURI
	instanceVariableNames: 'scheme host port pathSegments queryString fragment'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-URI'!

!XMLURI commentStamp: 'monty 2/2/2014 02:42' prior: 0!
A class for XML URIs. It uses the XMLURIParser to parse URIs and can combine URIs using /, can decode URI paths, and can retrieve contents using XMLHTTPRequest and XMLFileHandle.!

!XMLURI class methodsFor: 'instance creation' stamp: 'monty 1/10/2014 00:16'!
fromString: aString
	^ self new fromString: aString! !

!XMLURI methodsFor: 'combining' stamp: 'monty 1/12/2014 22:05'!
/ aUriOrString
	| uriToCombineWith |

	uriToCombineWith := aUriOrString asXMLURI.
	^ (uriToCombineWith isRelative)
		ifTrue: [self copy combineWithURI: uriToCombineWith]
		ifFalse: [uriToCombineWith].! !

!XMLURI methodsFor: 'comparing' stamp: 'monty 10/22/2014 06:46'!
= anObject
	self == anObject
		ifTrue: [^ true].

	self class == anObject class
		ifFalse: [^ false].

	^ scheme = anObject scheme
		and: [host = anObject host
			and: [port = anObject port
				and: [pathSegments = anObject pathSegments
					and: [queryString = anObject queryString
						and: [fragment = anObject fragment]]]]].! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/16/2014 08:54'!
addPathSegment: aSegment
	(aSegment = '.' or: [aSegment = '..'])
		ifTrue: [
			(aSegment = '..' and: [pathSegments notEmpty])
				ifTrue: [pathSegments removeLast].
			^ self].

	(aSegment notEmpty
		and: [pathSegments isEmpty
			and: [self hasHost]])
		ifTrue: [pathSegments addFirst: ''].

	pathSegments addLast: aSegment.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/17/2014 02:43'!
addPathSegmentsFrom: aUri
	aUri pathSegments
		ifEmpty: [^ self].

	aUri hasAbsolutePath
		ifTrue: [pathSegments removeAll]
		ifFalse: [
			pathSegments
				ifNotEmpty: [pathSegments removeLast]].

	aUri pathSegments do: [:each | self addPathSegment: each].! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/21/2014 23:37'!
asRetrievableResource
	^ self isLocalURI
		ifTrue: [XMLFileHandle path: self unescapedPath]
		ifFalse: [XMLHTTPRequest url: self asString]! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/13/2014 00:42'!
asString
	"for compatibility if Object>>asString isn't defined"
	^ self printString! !

!XMLURI methodsFor: 'converting' stamp: 'monty 1/10/2014 00:56'!
asXMLURI
	^ self! !

!XMLURI methodsFor: 'private' stamp: 'monty 10/22/2014 07:26'!
combineWithURI: aUri
	aUri hasHost
		ifTrue: [host := aUri host].
	aUri hasPort
		ifTrue: [port := aUri port].

	self addPathSegmentsFrom: aUri.
	queryString := aUri queryString.
	fragment := aUri fragment.! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 02:02'!
fragment
	^ fragment! !

!XMLURI methodsFor: 'initialization' stamp: 'monty 10/22/2014 06:44'!
fromString: aString
	| parser  |

	parser := XMLURIParser on: aString.

	scheme := parser parseScheme.
	host := parser parseHost.
	port := parser parsePort.
	self setPathSegmentsFromString: parser parsePath.
	queryString := parser parseQueryString.
	fragment := parser parseFragment.! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 1/10/2014 00:13'!
get
	^ self getUpToLimit: nil! !

!XMLURI methodsFor: 'retrieving' stamp: 'monty 1/10/2014 00:13'!
getUpToLimit: aMaxSize
	^ self asRetrievableResource getUpToLimit: aMaxSize! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 06:51'!
hasAbsolutePath
	^ pathSegments notEmpty
		and: [pathSegments first isEmpty]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 10/22/2014 07:32'!
hasFragment
	^ fragment notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/16/2014 08:56'!
hasHost
	^ host notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 10/22/2014 07:27'!
hasPort
	^ port notNil! !

!XMLURI methodsFor: 'testing' stamp: 'monty 10/22/2014 07:31'!
hasQueryString
	^ queryString notEmpty! !

!XMLURI methodsFor: 'testing' stamp: 'monty 10/22/2014 07:30'!
hasScheme
	^ scheme notEmpty! !

!XMLURI methodsFor: 'comparing' stamp: 'monty 1/13/2014 05:05'!
hash
	^ host hash bitXor: pathSegments hash! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 01:50'!
host
	^ host! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/7/2014 01:54'!
isFileURI
	^ scheme = 'file'! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 00:47'!
isLocalURI
	^ self isFileURI or: [self isRelative]! !

!XMLURI methodsFor: 'testing' stamp: 'monty 1/10/2014 06:54'!
isRelative
	^ scheme isEmpty
		or: [host isEmpty
			and: [self isFileURI not]]! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/20/2014 23:55'!
path
	^ self pathPrintStringUnescaped: false! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 09:42'!
pathPrintStringUnescaped: aBoolean
	| pathStream |

	pathSegments
		ifEmpty: [^ ''].

	pathStream := (String new: pathSegments size * 10) writeStream.
	self
		printPathSegmentsOn: pathStream
		unescaped: aBoolean.
	^ pathStream contents.! !

!XMLURI methodsFor: 'private' stamp: 'monty 1/10/2014 00:07'!
pathSegments
	^ pathSegments! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 10/22/2014 06:42'!
port
	^ port! !

!XMLURI methodsFor: 'copying' stamp: 'monty 1/10/2014 00:13'!
postCopy
	pathSegments := pathSegments copy! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:32'!
printFragmentOn: aStream
	self hasFragment
		ifTrue: [
			aStream
				nextPut: $#;
				nextPutAll: fragment]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:35'!
printHostOn: aStream
	self hasHost
		ifTrue: [
			self hasScheme
				ifFalse: [aStream nextPutAll: '//'].
			aStream nextPutAll: host]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:28'!
printOn: aStream
	self
		printSchemeOn: aStream;
		printHostOn: aStream;
		printPortOn: aStream;
		printPathSegmentsOn: aStream unescaped: false;
		printQueryStringOn: aStream;
		printFragmentOn: aStream
		! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/21/2014 01:01'!
printPathSegmentsOn: aStream unescaped: aBoolean
	pathSegments
		do: [:each |
			aBoolean
				ifTrue: [self printUnescaped: each on: aStream]
				ifFalse: [aStream nextPutAll: each]]
		separatedBy: [aStream nextPut: $/]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:29'!
printPortOn: aStream
	self hasPort
		ifTrue: [
			aStream
				nextPut: $:;
				nextPutAll: port asString]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:32'!
printQueryStringOn: aStream
	self hasQueryString
		ifTrue: [
			aStream
				nextPut: $?;
				nextPutAll: queryString]! !

!XMLURI methodsFor: 'printing' stamp: 'monty 10/22/2014 07:30'!
printSchemeOn: aStream
	self hasScheme
		ifTrue: [
			aStream
				nextPutAll: scheme;
				nextPutAll: '://']! !

!XMLURI methodsFor: 'printing' stamp: 'monty 1/21/2014 02:03'!
printUnescaped: aString on: aStream
	| readStream nextChar charCode |

	readStream := aString readStream.
	[readStream atEnd]
		whileFalse: [
			(nextChar := readStream next) == $%
				ifTrue: [
					charCode := 
						XMLIntegerReader
							readFrom: readStream
							withBase: 16
							upTo: 2.
					nextChar := (charCode ifNil: [$%]) asCharacter].
			aStream nextPut: nextChar].! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 03:38'!
queryString
	^ queryString! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/7/2014 01:50'!
scheme
	^ scheme! !

!XMLURI methodsFor: 'private' stamp: 'monty 10/22/2014 11:36'!
setPathSegmentsFromString: aString
	pathSegments := OrderedCollection new.

	aString ifNotEmpty: [
		$/
			xmlSplit: aString
			do: [:segment | pathSegments addLast: segment]]! !

!XMLURI methodsFor: 'accessing' stamp: 'monty 1/21/2014 00:32'!
unescapedPath
	^ self pathPrintStringUnescaped: true! !

Object subclass: #XMLURIParser
	instanceVariableNames: 'uriStream writeStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-URI'!

!XMLURIParser commentStamp: 'monty 1/7/2014 04:42' prior: 0!
This is a basic URI parser that can split a URI into scheme, host (with port), path, query string, and fragment.!

!XMLURIParser class methodsFor: 'instance creation' stamp: 'monty 1/7/2014 01:22'!
on: aString
	^ self new uri: aString! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/7/2014 04:18'!
parseDoubleSlashOrReset
	2 timesRepeat: [
		(uriStream atEnd not
			and: [uriStream peek == $/])
				ifTrue: [uriStream next]
				ifFalse: [
					uriStream reset.
					^ false]].
	^ true.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/10/2014 06:59'!
parseFragment
	^ self
		parseUpToAnyNotSkipping: ''
		startingAfter: $#! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 10/22/2014 06:31'!
parseHost
	^ (uriStream position > 0
		or: [self parseDoubleSlashOrReset])
		ifTrue: [(self parseUpToAnyNotSkipping: ':/?#') asLowercase]
		ifFalse: [''].! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/7/2014 01:34'!
parsePath
	^ self parseUpToAnyNotSkipping: '?#'! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 10/22/2014 07:06'!
parsePort
	(uriStream atEnd not
		and: [uriStream peek == $:])
		ifFalse: [^ nil].

	^ XMLIntegerReader
		readFrom:
			(self
				parseUpToAnyNotSkipping: '/?#'
				startingAfter: $:) readStream
		withBase: 10.! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/10/2014 06:59'!
parseQueryString
	^ self
		parseUpToAnyNotSkipping: '#'
		startingAfter: $?! !

!XMLURIParser methodsFor: 'parsing' stamp: 'monty 1/7/2014 04:18'!
parseScheme
	| scheme |

	scheme := uriStream upTo: $:.
	^ self parseDoubleSlashOrReset
		ifTrue: [scheme asLowercase]
		ifFalse: [''].

	
	
		! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/7/2014 01:38'!
parseUpToAnyNotSkipping: aCollection
	^ self
		parseUpToAnyNotSkipping: aCollection
		startingAfter: nil! !

!XMLURIParser methodsFor: 'private' stamp: 'monty 1/7/2014 02:06'!
parseUpToAnyNotSkipping: aCollection startingAfter: aCharacter
	aCharacter ifNotNil: [
		(uriStream atEnd not
			and: [uriStream peek == aCharacter])
			ifFalse: [^ ''].
		uriStream next].

	^ aCollection
		ifEmpty: [uriStream upToEnd]
		ifNotEmpty: [
			writeStream reset.
			[uriStream atEnd
				or: [aCollection includes: uriStream peek]]
				whileFalse: [writeStream nextPut: uriStream next].
			writeStream contents].! !

!XMLURIParser methodsFor: 'accessing' stamp: 'monty 1/7/2014 06:29'!
uri: aString
	uriStream := aString readStream.
	writeStream := (String new: aString size) writeStream.! !
XMLParserTokenizer initialize!
XMLStreamDecoder initialize!
XMLEncodingDetector initialize!
XMLWellFormedParserTokenizer initialize!
XMLIntegerReader initialize!
DTDValueListParser initialize!
DTDAttributeDefinitionParser initialize!
DTDDFAContentModelParser initialize!
XMLTextConverterStreamDecoder initialize!
XMLNestedStreamReader initialize!
